@@ -6,10 +6,10 @@
  *    Michael Meeks (michael@ximian.com)
  *    Jody Goldberg (jody@gnome.org)
  *
- * (C) 1998-2002 Michael Meeks
+ * (C) 1998-2003 Michael Meeks, Jody Goldberg
  **/
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 
 #include "file.h"
