@@ -23,7 +23,6 @@
 #include <value.h>
 #include <gutils.h>
 #include <graph.h>
-#include <style-color.h>
 #include <sheet-object-graph.h>
 
 #include <goffice/graph/goffice-graph.h>
@@ -59,7 +58,6 @@ typedef struct _XLChartSeries {
 		GOData *data;
 	} data [GOG_MS_DIM_TYPES];
 	int chart_group;
-	gboolean  has_legend;
 	GogStyle *style;
 } XLChartSeries;
 
@@ -73,7 +71,6 @@ typedef struct {
 	SheetObject	*sog;
 	GogGraph	*graph;
 	GogChart	*chart;
-	GogObject	*legend;
 	GogPlot		*plot;
 	GogStyle	*default_plot_style;
 	GogObject	*axis;
@@ -115,7 +112,6 @@ excel_chart_series_new (void)
 	series = g_new (XLChartSeries, 1);
 
 	series->chart_group = -1;
-	series->has_legend = TRUE;
 	series->style = NULL;
 	for (i = GOG_MS_DIM_TYPES; i-- > 0 ; ) {
 		series->data [i].data = NULL;
@@ -217,13 +213,13 @@ BC_R(3d)(XLChartHandler const *handle,
 		fprintf (stderr, "Gap = %hu\n", gap);
 
 		if (use_perspective)
-			fputs ("Use perspective;\n", stderr);
+			fputs ("Use perspective", stderr);
 		if (cluster)
-			fputs ("Cluster;\n", stderr);
+			fputs ("Cluster", stderr);
 		if (auto_scale)
-			fputs ("Auto Scale;\n", stderr);
+			fputs ("Auto Scale", stderr);
 		if (walls_2d)
-			fputs ("2D Walls;\n", stderr);
+			fputs ("2D Walls", stderr);
 	});
 
 	return FALSE;
@@ -259,24 +255,24 @@ BC_R(ai)(XLChartHandler const *handle,
 			style_format_unref (fmt);
 		}
 	} else {
-		d (2, fputs ("Uses number format from data source;\n", stderr););
+		d (2, fputs ("Uses number format from data source", stderr););
 	}
 
 	g_return_val_if_fail (purpose < GOG_MS_DIM_TYPES, TRUE);
 	d (0, {
 	switch (purpose) {
-	case GOG_MS_DIM_LABELS :     fputs ("Linking labels;\n", stderr); break;
-	case GOG_MS_DIM_VALUES :     fputs ("Linking values;\n", stderr); break;
-	case GOG_MS_DIM_CATEGORIES : fputs ("Linking categories;\n", stderr); break;
-	case GOG_MS_DIM_BUBBLES :    fputs ("Linking bubbles;\n", stderr); break;
+	case GOG_MS_DIM_LABELS :     fputs ("Linking labels", stderr); break;
+	case GOG_MS_DIM_VALUES :     fputs ("Linking values", stderr); break;
+	case GOG_MS_DIM_CATEGORIES : fputs ("Linking categories", stderr); break;
+	case GOG_MS_DIM_BUBBLES :    fputs ("Linking bubbles", stderr); break;
 	default :
 		g_assert_not_reached ();
 	};
 	switch (ref_type) {
-	case 0 : fputs ("Use default categories;\n", stderr); break;
-	case 1 : fputs ("Text/Value entered directly;\n", stderr); break;
-	case 2 : fputs ("Linked to Container;\n", stderr); break;
-	case 4 : fputs ("'Error reported' what the heck is this ??;\n", stderr); break;
+	case 0 : fputs ("Use default categories", stderr); break;
+	case 1 : fputs ("Text/Value entered directly", stderr); break;
+	case 2 : fputs ("Linked to Container", stderr); break;
+	case 4 : fputs ("'Error reported' what the heck is this ??", stderr); break;
 	default :
 		 fprintf (stderr, "UKNOWN : reference type (%x)\n", ref_type);
 	};
@@ -421,21 +417,21 @@ BC_R(attachedlabel)(XLChartHandler const *handle,
 	gboolean const show_label = (flags&0x10) ? TRUE : FALSE;
 
 	if (show_value)
-		fputs ("Show Value;\n", stderr);
+		fputs ("Show Value", stderr);
 	if (show_percent)
-		fputs ("Show as Percentage;\n", stderr);
+		fputs ("Show as Percentage", stderr);
 	if (show_label_prercent)
-		fputs ("Show as Label Percentage;\n", stderr);
+		fputs ("Show as Label Percentage", stderr);
 	if (smooth_line)
-		fputs ("Smooth line;\n", stderr);
+		fputs ("Smooth line", stderr);
 	if (show_label)
-		fputs ("Show the label;\n", stderr);
+		fputs ("Show the label", stderr);
 
 	if (s->container.ver >= MS_BIFF_V8)
 	{
 		gboolean const show_bubble_size = (flags&0x20) ? TRUE : FALSE;
 		if (show_bubble_size)
-			fputs ("Show bubble size;\n", stderr);
+			fputs ("Show bubble size", stderr);
 	}
 	});
 	return FALSE;
@@ -486,51 +482,25 @@ BC_R(axcext)(XLChartHandler const *handle,
 
 /****************************************************************************/
 
-static gboolean
-BC_R(lineformat)(XLChartHandler const *handle,
-		 XLChartReadState *s, BiffQuery *q);
 static gboolean
 BC_R(axislineformat)(XLChartHandler const *handle,
 		     XLChartReadState *s, BiffQuery *q)
 {
-	guint16 opcode;
+	d (0, {
 	guint16 const type = GSF_LE_GET_GUINT16 (q->data);
 
-	d (0, {
 	fprintf (stderr, "Axisline is ");
 	switch (type)
 	{
-	case 0 : fputs ("the axis line.\n", stderr); break;
-	case 1 : fputs ("a major grid along the axis.\n", stderr); break;
-	case 2 : fputs ("a minor grid along the axis.\n", stderr); break;
+	case 0 : fputs ("the axis line.", stderr); break;
+	case 1 : fputs ("a major grid along the axis.", stderr); break;
+	case 2 : fputs ("a minor grid along the axis.", stderr); break;
 
 	/* TODO TODO : floor vs wall */
-	case 3 : fputs ("a floor/wall along the axis.\n", stderr); break;
+	case 3 : fputs ("a floor/wall along the axis.", stderr); break;
 	default : fprintf (stderr, "an ERROR.  unkown type (%x).\n", type);
 	};
 	});
-
-	if (!ms_biff_query_peek_next (q, &opcode) || opcode != BIFF_CHART_lineformat) {
-		g_warning ("I had hoped that a lineformat would always follow an axislineformat");
-		return FALSE;
-	}
-	ms_biff_query_next (q);
-	if (BC_R(lineformat)(handle, s, q))
-		return TRUE;
-
-	if (type == 0 && s->axis != NULL) {
-		g_object_set (G_OBJECT (s->axis),
-			"style", s->style,
-			NULL);
-		/* deleted axis sets flag here, rather than in TICK */
-		if (0 == (0x4 & GSF_LE_GET_GUINT16 (q->data+8)))
-			g_object_set (G_OBJECT (s->axis),
-				"major-tick-labeled",	FALSE,
-				NULL);
-	}
-	g_object_unref (s->style);
-	s->style = NULL;
-
 	return FALSE;
 }
 
@@ -871,126 +841,17 @@ BC_R(frame)(XLChartHandler const *handle,
 
 /****************************************************************************/
 
-static GOColor
-ms_chart_map_color (XLChartReadState const *s, guint32 raw, guint32 alpha)
-{
-	GOColor res;
-	if ((~0x7ffffff) & raw) {
-		GnmColor *c= excel_palette_get (s->container.ewb->palette,
-			(0x7ffffff & raw));
-		res = GDK_TO_UINT (c->color);
-		style_color_unref (c);
-	} else {
-		guint8 r, g, b;
-		r = (raw)       & 0xff;
-		g = (raw >> 8)  & 0xff;
-		b = (raw >> 16) & 0xff;
-		res = RGBA_TO_UINT (r, g, b, 0xff);
-	}
-	return res;
-}
-
 static gboolean
 BC_R(gelframe) (XLChartHandler const *handle,
 		XLChartReadState *s, BiffQuery *q)
 {
-	MSObjAttrBag *attrs = ms_escher_parse (q, &s->container, TRUE);
-	guint32 type = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_TYPE, 0);
-	guint32 shade_type = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_SHADE_TYPE, 0);
-	guint32 fill_color = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_COLOR, 0);
-	guint32 fill_alpha = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_ALPHA, 0x10000);
-	guint32 fill_back_color = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_BACKGROUND, 0);
-	guint32 fill_back_alpha = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_BACKGROUND_ALPHA, 0x10000);
-	guint32 preset = ms_obj_attr_get_uint (attrs,
-		MS_OBJ_ATTR_FILL_PRESET, 0);
-
-	s->style->fill.type = GOG_FILL_STYLE_GRADIENT;
-	s->style->fill.u.gradient.start = 
-		ms_chart_map_color (s, fill_color, fill_alpha);
-
-	/* FIXME : make presets the same as 2 color for now */
-	if (!(shade_type & 8) || preset > 0) {
-		s->style->fill.u.gradient.end = ms_chart_map_color (s,
-			fill_back_color, fill_back_alpha);
-	} else {
-		float brightness;
-		unsigned frac = (fill_back_color >> 16) & 0xff;
-
-		/**
-		 * 0x10 const
-		 * 0x00..0xff fraction
-		 * 0x02 == bright 0x01 == dark
-		 * 0xf0 == const
-		 **/
-		switch ((fill_back_color & 0xff00)) {
-		default :
-			g_warning ("looks like our theory of 1-color gradient brightness is incorrect");
-		case 0x0100 : brightness = 0. + frac/512.; break;
-		case 0x0200 : brightness = 1. - frac/512.; break;
-		}
-		gog_style_set_fill_brightness (s->style, (1. - brightness) * 100.);
-		d (1, fprintf (stderr, "%x : frac = %u, flag = 0x%hx ::: %f",
-			       fill_back_color, frac, fill_back_color & 0xff00, brightness););
-	}
-
-	if (type == 5) { /* Fill from corner */
-		/* TODO */
-	} else if (type == 6) { /* fill from center */
-		/* TODO */
-	} else if (type == 7) {
-		GOGradientDirection dir;
-		guint32 angle = ms_obj_attr_get_uint (attrs, MS_OBJ_ATTR_FILL_ANGLE, 0);
-		gint32 focus = ms_obj_attr_get_int (attrs, MS_OBJ_ATTR_FILL_FOCUS, 0);
-
-		if (focus < 0)
-			focus = ((focus - 25) / 50) % 4 + 4;
-		else
-			focus = ((focus + 25) / 50) % 4;
-
-		switch (angle) {
-		default :
-			g_warning ("non standard gradient angle %u, using horizontal", angle);
-		case 0 : /* horizontal */
-			switch (focus) {
-			case 0 : dir = GO_GRADIENT_S_TO_N; break;
-			case 1 : dir = GO_GRADIENT_N_TO_S_MIRRORED; break;
-			case 2 : dir = GO_GRADIENT_N_TO_S; break;
-			case 3 : dir = GO_GRADIENT_S_TO_N_MIRRORED; break;
-			}
-			break;
-		case 0xffd30000 : /* diag down (-45 in 16.16 fixed) */
-			switch (focus) {
-			case 0 : dir = GO_GRADIENT_SE_TO_NW; break;
-			case 1 : dir = GO_GRADIENT_SE_TO_NW_MIRRORED; break;
-			case 2 : dir = GO_GRADIENT_NW_TO_SE; break;
-			case 3 : dir = GO_GRADIENT_NW_TO_SE_MIRRORED; break;
-			}
-			break;
-		case 0xffa60000 : /* vertical (-90 in 16.16 fixed) */
-			switch (focus) {
-			case 0 : dir = GO_GRADIENT_E_TO_W; break;
-			case 1 : dir = GO_GRADIENT_E_TO_W_MIRRORED; break;
-			case 2 : dir = GO_GRADIENT_W_TO_E; break;
-			case 3 : dir = GO_GRADIENT_W_TO_E_MIRRORED; break;
-			}
-			break;
-		case 0xff790000 : /* diag up (-135 in 16.16 fixed) */
-			switch (focus) {
-			case 0 : dir = GO_GRADIENT_SE_TO_NW; break;
-			case 1 : dir = GO_GRADIENT_SE_TO_NW_MIRRORED; break;
-			case 2 : dir = GO_GRADIENT_NW_TO_SE; break;
-			case 3 : dir = GO_GRADIENT_NW_TO_SE_MIRRORED; break;
-			}
-		}
-		s->style->fill.u.gradient.dir = dir;
-	}
+	MSObjAttrBag *attrs = ms_escher_parse (q, &s->container);
 
+#if 0
+	MSObjAttr *crop_bottom_attr = ms_obj_attr_bag_lookup
+		(obj->attrs, MS_OBJ_ATTR_BLIP_CROP_BOTTOM);
+	MS_OBJ_ATTR_FILL_TYPE;
+#endif
 	ms_obj_attr_bag_destroy (attrs);
 
 	return FALSE;
@@ -1037,6 +898,7 @@ BC_R(legend)(XLChartHandler const *handle,
 #endif
 	guint16 const XL_pos = GSF_LE_GET_GUINT8 (q->data+16);
 	GogObjectPosition pos;
+	GogObject *legend;
 
 	switch (XL_pos) {
 	case 0: pos = GOG_POSITION_S | GOG_POSITION_ALIGN_CENTER; break;
@@ -1051,8 +913,8 @@ BC_R(legend)(XLChartHandler const *handle,
 		pos = GOG_POSITION_E | GOG_POSITION_ALIGN_CENTER; break; 
 	};
 
-	s->legend = gog_object_add_by_name (GOG_OBJECT (s->chart), "Legend", NULL);
-	gog_object_set_pos (s->legend, pos);
+	legend = gog_object_add_by_name (GOG_OBJECT (s->chart), "Legend", NULL);
+	gog_object_set_pos (legend, pos);
 
 #if 0
 	fprintf (stderr, "Legend @ %f,%f, X=%f, Y=%f\n",
@@ -1070,9 +932,6 @@ static gboolean
 BC_R(legendxn)(XLChartHandler const *handle,
 	       XLChartReadState *s, BiffQuery *q)
 {
-	guint16 const flags = GSF_LE_GET_GUINT16 (q->data+2);
-	if ((flags & 1) && s->currentSeries != NULL)
-		s->currentSeries->has_legend = FALSE;
 	return FALSE;
 }
 
@@ -1379,7 +1238,7 @@ BC_R(radar)(XLChartHandler const *handle,
 	    XLChartReadState *s, BiffQuery *q)
 {
 	g_return_val_if_fail (s->plot == NULL, TRUE);
-	s->plot = gog_plot_new_by_name ("GogRadialPlot UNIMPLEMENTED");
+	s->plot = gog_plot_new_by_name ("GogRadarPlot");
 
 	return FALSE;
 }
@@ -1391,7 +1250,10 @@ BC_R(radararea)(XLChartHandler const *handle,
 		XLChartReadState *s, BiffQuery *q)
 {
 	g_return_val_if_fail (s->plot == NULL, TRUE);
-	s->plot = gog_plot_new_by_name ("GogFilledRadialPlot UNIMPLEMENTED");
+	s->plot = gog_plot_new_by_name ("GogRadarPlot");
+	g_object_set (G_OBJECT (s->plot),
+		"area",		TRUE,
+		NULL);
 
 	return FALSE;
 }
@@ -1537,7 +1399,7 @@ BC_R(seriestext)(XLChartHandler const *handle,
 		return FALSE;
 
 	str = biff_get_text (q->data + 3, slen, NULL);
-	d (2, fprintf (stderr, "'%s';\n", str););
+	d (2, fputs (str, stderr););
 
 	/* A quick heuristic */
 	if (s->currentSeries != NULL &&
@@ -1610,7 +1472,7 @@ BC_R(shtprops)(XLChartHandler const *handle,
 
 	g_return_val_if_fail (tmp < MS_CHART_BLANK_MAX, TRUE);
 	blanks = tmp;
-	d (2, fprintf (stderr, "%s;", ms_chart_blank[blanks]););
+	d (2, fputs (ms_chart_blank[blanks], stderr););
 
 	if (s->container.ver >= MS_BIFF_V8)
 		ignore_pos_record = (flags&0x10) ? TRUE : FALSE;
@@ -1660,18 +1522,16 @@ BC_R(text)(XLChartHandler const *handle,
 	   XLChartReadState *s, BiffQuery *q)
 {
 	if (s->prev_opcode == BIFF_CHART_defaulttext) {
-		d (4, fputs ("Text follows defaulttext;\n", stderr););
+		d (4, fputs ("Text follows defaulttext", stderr););
 	} else {
 	}
-	BC_R(get_style) (s);
-	s->style->font.color = BC_R(color) (q->data+0, "Font");
 
 #if 0
 case BIFF_CHART_chart :
-	fputs ("Text follows chart;\n", stderr);
+	fputs ("Text follows chart", stderr);
 	break;
 case BIFF_CHART_legend :
-	fputs ("Text follows legend;\n", stderr);
+	fputs ("Text follows legend", stderr);
 	break;
 default :
 	fprintf (stderr, "BIFF ERROR : A Text record follows a %x\n",
@@ -1705,25 +1565,25 @@ BC_R(tick)(XLChartHandler const *handle,
 	guint16 const flags = GSF_LE_GET_GUINT8 (q->data+24);
 
 	switch (major) {
-	case 0: fputs ("no major tick;\n", stderr); break;
-	case 1: fputs ("major tick inside axis;\n", stderr); break;
-	case 2: fputs ("major tick outside axis;\n", stderr); break;
-	case 3: fputs ("major tick across axis;\n", stderr); break;
-	default : fputs ("unknown major tick type;\n", stderr);
+	case 0: fputs ("no major tick;", stderr); break;
+	case 1: fputs ("major tick inside axis;", stderr); break;
+	case 2: fputs ("major tick outside axis;", stderr); break;
+	case 3: fputs ("major tick across axis;", stderr); break;
+	default : fputs ("unknown major tick type", stderr);
 	}
 	switch (minor) {
-	case 0: fputs ("no minor tick;\n", stderr); break;
-	case 1: fputs ("minor tick inside axis;\n", stderr); break;
-	case 2: fputs ("minor tick outside axis;\n", stderr); break;
-	case 3: fputs ("minor tick across axis;\n", stderr); break;
-	default : fputs ("unknown minor tick type;\n", stderr);
+	case 0: fputs ("no minor tick;", stderr); break;
+	case 1: fputs ("minor tick inside axis;", stderr); break;
+	case 2: fputs ("minor tick outside axis;", stderr); break;
+	case 3: fputs ("minor tick across axis;", stderr); break;
+	default : fputs ("unknown minor tick type", stderr);
 	}
 	switch (label) {
-	case 0: fputs ("no tick label;\n", stderr); break;
-	case 1: fputs ("tick label at low end (NOTE mapped to near axis);\n", stderr); break;
-	case 2: fputs ("tick label at high end (NOTE mapped to near axis);\n", stderr); break;
-	case 3: fputs ("tick label near axis;\n", stderr); break;
-	default : fputs ("unknown tick label position;\n", stderr);
+	case 0: fputs ("no tick label;", stderr); break;
+	case 1: fputs ("tick label at low end (NOTE mapped to near axis);", stderr); break;
+	case 2: fputs ("tick label at high end (NOTE mapped to near axis);", stderr); break;
+	case 3: fputs ("tick label near axis;", stderr); break;
+	default : fputs ("unknown tick label position", stderr);
 	}
 
 	/*
@@ -1739,15 +1599,15 @@ BC_R(tick)(XLChartHandler const *handle,
 		fprintf (stderr, "background mode = %d\n", (unsigned)GSF_LE_GET_GUINT8 (q->data+3));
 
 	switch (flags&0x1c) {
-	case 0: fputs ("no rotation;\n", stderr); break;
-	case 1: fputs ("top to bottom letters upright;\n", stderr); break;
-	case 2: fputs ("rotate 90deg counter-clockwise;\n", stderr); break;
-	case 3: fputs ("rotate 90deg clockwise;\n", stderr); break;
-	default : fputs ("unknown rotation;\n", stderr);
+	case 0: fputs ("no rotation;", stderr); break;
+	case 1: fputs ("top to bottom letters upright;", stderr); break;
+	case 2: fputs ("rotate 90deg counter-clockwise;", stderr); break;
+	case 3: fputs ("rotate 90deg clockwise;", stderr); break;
+	default : fputs ("unknown rotation", stderr);
 	}
 
 	if (flags&0x20)
-		fputs ("Auto rotate;\n", stderr);
+		fputs ("Auto rotate", stderr);
 	});
 
 	return FALSE;
@@ -1798,15 +1658,15 @@ BC_R(valuerange)(XLChartHandler const *handle,
 
 	if (flags & 0x20) {
 		g_object_set (s->axis, "log-scale", TRUE, NULL);
-		d (1, fputs ("Log scaled;\n", stderr););
+		d (1, fputs ("Log scaled", stderr););
 	}
 	if (flags & 0x40) {
 		g_object_set (s->axis, "invert-axis", TRUE, NULL);
-		d (1, fputs ("Values in reverse order;\n", stderr););
+		d (1, fputs ("Values in reverse order", stderr););
 	}
 	if (flags & 0x80) {
 		g_object_set (s->axis, "pos_str", "high", NULL);
-		d (1, fputs ("Cross over at max value;\n", stderr););
+		d (1, fputs ("Cross over at max value", stderr););
 	}
 
 	return FALSE;
@@ -1849,27 +1709,32 @@ BC_R(end)(XLChartHandler const *handle,
 
 	switch (popped_state) {
 	case BIFF_CHART_axis :
+		if (s->style != NULL) {
+			g_object_set (G_OBJECT (s->axis),
+				"style", s->style,
+				NULL);
+			g_object_unref (s->style);
+			s->style = NULL;
+		}
 		s->axis = NULL;
 		break;
 
 	case BIFF_CHART_frame :
 		if (s->style != NULL) {
-			int top_state = BC_R(top_state) (s);
-			GogObject *obj = NULL;
-			if (top_state == BIFF_CHART_legend)
-				obj = s->legend;
-			else if (top_state == BIFF_CHART_chart)
-				obj = GOG_OBJECT (s->chart);
-			else if (s->frame_for_grid) {
-				GogGrid *tmp = gog_chart_get_grid (s->chart);
-				obj = (tmp == NULL)
-					? gog_object_add_by_name (GOG_OBJECT (s->chart), "Grid", NULL)
-					: GOG_OBJECT (tmp);
+			GogObject *obj;
+			if (s->chart != NULL) {
+				if (s->frame_for_grid) {
+					GogGrid *tmp = gog_chart_get_grid (s->chart);
+					obj = (tmp == NULL)
+						? gog_object_add_by_name (GOG_OBJECT (s->chart), "Grid", NULL)
+						: GOG_OBJECT (tmp);
+				} else
+					obj = GOG_OBJECT (s->chart);
+				if (obj != NULL)
+					g_object_set (G_OBJECT (obj),
+						"style", s->style,
+						NULL);
 			}
-			if (obj != NULL)
-				g_object_set (G_OBJECT (obj),
-					"style", s->style,
-					NULL);
 			g_object_unref (s->style);
 			s->style = NULL;
 		}
@@ -1925,10 +1790,6 @@ BC_R(end)(XLChartHandler const *handle,
 				g_object_set (G_OBJECT (series),
 					"style", style,
 					NULL);
-			if (!eseries->has_legend)
-				g_object_set (G_OBJECT (series),
-					"has-legend", FALSE,
-					NULL);
 		}
 
 		gog_object_add_by_name (GOG_OBJECT (s->chart),
@@ -2146,16 +2007,8 @@ ms_excel_read_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 	state.text	    = NULL;
 
 	if (NULL != (state.sog = sog)) {
-		GogStyle *style = gog_style_new ();
-		style->outline.width = -1;
-		style->fill.type = GOG_FILL_STYLE_NONE;
-
 		state.graph = sheet_object_graph_get_gog (sog);
 		state.chart = GOG_CHART (gog_object_add_by_name (GOG_OBJECT (state.graph), "Chart", NULL));
-
-		g_object_set (G_OBJECT (state.graph), "style", style, NULL);
-		g_object_set (G_OBJECT (state.chart), "style", style, NULL);
-		g_object_unref (style);
 	} else {
 		state.graph = NULL;
 		state.chart = NULL;
@@ -2165,7 +2018,7 @@ ms_excel_read_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 	state.axis  = NULL;
 	state.style = NULL;
 
-	d (0, fputs ("{ /* CHART */\n", stderr););
+	d (0, fputs ("{ CHART", stderr););
 
 	while (!done && ms_biff_query_next (q)) {
 		int const lsb = q->opcode & 0xff;
@@ -2197,7 +2050,7 @@ ms_excel_read_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 			switch (lsb) {
 			case BIFF_EOF:
 				done = TRUE;
-				d (0, fputs ("}; /* CHART */\n", stderr););
+				d (0, fputs ("}; /* CHART */", stderr););
 				g_return_val_if_fail(state.stack->len == 0, TRUE);
 				break;
 
@@ -2225,14 +2078,15 @@ ms_excel_read_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 				guint16 xf  = GSF_LE_GET_GUINT16 (q->data + 4);
 				guint16 len = GSF_LE_GET_GUINT16 (q->data + 6);
 				char *label = biff_get_text (q->data + 8, len, NULL);
-				d (10, {fprintf (stderr, "'%s'\n;hmm, what are these values for a chart ???\n"
-						"row = %d, col = %d, xf = %d\n", label, row, col, xf);});
+				d (10, {fputs (label, stderr);
+					fprintf (stderr, "hmm, what are these values for a chart ???\n"
+						"row = %d, col = %d, xf = %d\n", row, col, xf);});
 				g_free (label);
 				break;
 			}
 
 			case BIFF_MS_O_DRAWING:
-				ms_escher_parse (q, &state.container, FALSE);
+				ms_obj_attr_bag_destroy (ms_escher_parse (q, &state.container));
 				break;
 
 			case BIFF_EXTERNCOUNT: /* ignore */ break;
@@ -2310,12 +2164,8 @@ ms_excel_read_chart_BOF (BiffQuery *q, MSContainer *container, SheetObject *sog)
 	g_return_val_if_fail (bof != NULL, TRUE);
 	g_return_val_if_fail (bof->type == MS_BIFF_TYPE_Chart, TRUE);
 
-	/* NOTE : _Ignore_ the verison in the BOF, it lies!
-	 * XP saving as 95 will mark the book as biff7
-	 * but sheets and charts are marked as biff8, even though they are not
-	 * using unicode */
-	res = ms_excel_read_chart (q, container, container->ver, sog);
-
+	if (bof->version != MS_BIFF_V_UNKNOWN)
+		res = ms_excel_read_chart (q, container, bof->version, sog);
 	ms_biff_bof_data_destroy (bof);
 	return res;
 }
