@@ -7,7 +7,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include <plugin.h>
 #include <plugin-util.h>
@@ -33,9 +33,9 @@
 
 GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
-gboolean pln_file_probe (GnumFileOpener const *fo, GsfInput *input,
+gboolean pln_file_probe (GnmFileOpener const *fo, GsfInput *input,
 			 FileProbeLevel pl);
-void     pln_file_open (GnumFileOpener const *fo, IOContext *io_context,
+void     pln_file_open (GnmFileOpener const *fo, IOContext *io_context,
 			WorkbookView *wb_view, GsfInput *input);
 
 static char const *formula1[] = {
@@ -157,7 +157,7 @@ static guint8 const signature[] =
     { 0xff, 'W','P','C', 0x10, 0, 0, 0, 0x9, 0xa };
 
 /* in charset.c. */
-guint8 *pln_get_str (guint8 const *ch, unsigned len);
+guint8 *pln_get_str (guint8 const *ch, int len);
 
 static char const *
 pln_get_func_table1 (unsigned i)
@@ -171,9 +171,9 @@ pln_get_func_table2 (unsigned i)
 	g_return_val_if_fail (0 < i && i < G_N_ELEMENTS (formula2), "ERROR");
 	return formula2 [i];
 }
-		
+
 gboolean
-pln_file_probe (GnumFileOpener const *fo, GsfInput *input,
+pln_file_probe (GnmFileOpener const *fo, GsfInput *input,
 		FileProbeLevel pl)
 {
 	/*
@@ -195,12 +195,12 @@ pln_file_probe (GnumFileOpener const *fo, GsfInput *input,
 		memcmp (header, signature, sizeof (signature)) == 0;
 }
 
-static MStyle *
+static GnmStyle *
 pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 {
 	guint16 attr, fmt, font;
 	guint32 key;
-	MStyle *res;
+	GnmStyle *res;
 
 	attr = GSF_LE_GET_GUINT16 (data);
 	fmt  = GSF_LE_GET_GUINT16 (data+2);
@@ -208,7 +208,7 @@ pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 
 	/* Check for use of sheet defaults */
 	if (is_cell) {
-		MStyle *def = sheet_style_default (state->sheet);
+		GnmStyle *def = sheet_style_default (state->sheet);
 		if ((attr & 0x0700) == 0x0400) {
 			attr &= 0xf8ff;
 			switch (mstyle_get_align_h (def)) {
@@ -254,11 +254,11 @@ pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 	return res;
 }
 
-static gnum_float
+static gnm_float
 pln_get_number (guint8 const * ch)
 {
 	int exp;
-	gnum_float dvalue, scale = 256.0;
+	gnm_float dvalue, scale = 256.0;
 	int i;
 
 	dvalue = 0.0;
@@ -275,11 +275,12 @@ pln_get_number (guint8 const * ch)
 }
 
 static char *
-pln_get_addr (ParsePos const *pp, guint8 const *ch)
+pln_get_addr (GnmParsePos const *pp, guint8 const *ch)
 {
 	guint16 r = GSF_LE_GET_GUINT16 (ch);
 	guint16 c = GSF_LE_GET_GUINT16 (ch+2);
-	CellRef ref;
+	GnmCellRef ref;
+	GString *str = g_string_new (NULL);
 
 	ref.sheet = NULL;
 	ref.col_relative = ref.row_relative = FALSE;
@@ -301,21 +302,23 @@ pln_get_addr (ParsePos const *pp, guint8 const *ch)
 		break;
 	}
 
-	return cellref_name (&ref, pp, TRUE);
+	cellref_as_string (str, gnm_expr_conventions_default, &ref, pp, TRUE);
+
+	return g_string_free (str, FALSE);
 }
 
 static char *
-pln_convert_expr (ParsePos const *pp, guint8 const *ch)
+pln_convert_expr (GnmParsePos const *pp, guint8 const *ch)
 {
-	GString *expr = g_string_new ("");
-	guint8 *str, *escaped;
+	GString *expr = g_string_new (NULL);
+	guint8 *str;
 	guint8 const *end;
 	int i, len, code;
 
 	/* Expressions are stored INFIX so it is easier to just generate text */
 	i = GSF_LE_GET_GUINT16 (ch); ch += 2;
 #if DEBUG_EXPR
-	puts (cell_pos_name (&pp->eval));
+	puts (cellpos_as_string (&pp->eval));
 	gsf_mem_dump (ch, i);
 #endif
 	for (end = ch + i ; ch < end ; ) {
@@ -333,13 +336,11 @@ pln_convert_expr (ParsePos const *pp, guint8 const *ch)
 		case 9:	/* Text constant */
 			len = *ch;
 			str = pln_get_str (ch + 1, len);
-			escaped = gnumeric_strescape (str);
-			g_free (str);
 			g_string_append_c (expr, '\"');
-			g_string_append (expr, escaped);
+			gnm_strescape (expr, str);
 			g_string_append_c (expr, '\"');
-			g_free (escaped);
 			ch += len + 1;
+			g_free (str);
 			break;
 
 		case 10: /* Named block */
@@ -488,18 +489,18 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 	int max_row = SHEET_MAX_ROWS;
 	int i, rcode, rlength;
 	guint8 const *data;
-	Cell    *cell;
-	Value   *v;
+	GnmCell    *cell;
+	GnmValue   *v;
 	GnmExpr const *expr;
-	MStyle  *style;
-	ParsePos pp;
-	Range    r;
+	GnmStyle  *style;
+	GnmParsePos pp;
+	GnmRange r;
 
 	range_init (&r, 0,0,0, SHEET_MAX_ROWS);
-	parse_pos_init (&pp, NULL, state->sheet, 0, 0);
+	parse_pos_init_sheet (&pp, state->sheet);
 
-	data = gsf_input_read (input, 6, NULL);
-	if (data == NULL || GSF_LE_GET_GUINT16 (data + 2) != 0)
+	data = gsf_input_read (input, 16, NULL);
+	if (data == NULL || GSF_LE_GET_GUINT16 (data + 12) != 0)
 		return error_info_new_str (_("PLN : Spreadsheet is password encrypted"));
 
 	/* Process the record based sections
@@ -527,7 +528,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 
 		case 0x02:
 			/* char ascii char of decimal point		0
-			 * char ascii char of thousands seperator	1
+			 * char ascii char of thousands separator	1
 			 * WPCHAR 1-6 bytes for currency symbol		2
 			 * guint16 default worksheet attributs		8
 			 * guint16 default worksheet format		10
@@ -549,6 +550,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 			break;
 
 		default:
+			;
 			/* g_warning("PLN : Record handling code for code %d not yet written", rcode); */
 		}
 	} while (rcode != 25);
@@ -575,6 +577,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 
 		v = NULL;
 		expr = NULL;
+		cell = NULL;
 		if ((type & 0x7) != 0) {
 			style = pln_get_style (state, data, TRUE);
 			if (style != NULL)
@@ -582,7 +585,6 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 			if ((type & 0x7) != 6)
 				cell = sheet_cell_fetch (state->sheet, pp.eval.col, pp.eval.row);
 		} else {
-			cell = NULL;
 			style = NULL;
 		}
 
@@ -609,10 +611,10 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 					pln_get_str (data + 2, GSF_LE_GET_GUINT16 (data)));
 			break;
 		/* Error Cell */
-		case 4: v = value_new_error (NULL, gnumeric_err_VALUE);
+		case 4: v = value_new_error_VALUE (NULL);
 			break;
 		/* na Cell */
-		case 5: v = value_new_error (NULL, gnumeric_err_NA);
+		case 5: v = value_new_error_NA (NULL);
 			break;
 		/* format only, no data in cell */
 		case 6: break;
@@ -625,7 +627,9 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 
 				if (expr_txt != NULL) {
 					expr = gnm_expr_parse_str (expr_txt, &pp,
-						GNM_EXPR_PARSE_DEFAULT, NULL);
+								   GNM_EXPR_PARSE_DEFAULT,
+								   gnm_expr_conventions_default,
+								   NULL);
 					if (expr == NULL) {
 						if (v != NULL)
 							value_release (v);
@@ -650,7 +654,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 }
 
 void
-pln_file_open (GnumFileOpener const *fo, IOContext *io_context,
+pln_file_open (GnmFileOpener const *fo, IOContext *io_context,
                WorkbookView *wb_view, GsfInput *input)
 {
 	Workbook *wb;
@@ -659,12 +663,6 @@ pln_file_open (GnumFileOpener const *fo, IOContext *io_context,
 	ErrorInfo *error;
 	PlanPerfectImport state;
 
-	if (!pln_file_probe (NULL, input, FILE_PROBE_CONTENT_FULL)) {
-		gnumeric_io_error_info_set (io_context,
-			error_info_new_str (_("PLN : Not a PlanPerfect File")));
-		return;
-	}
-
 	wb    = wb_view_workbook (wb_view);
 	name  = workbook_sheet_get_free_name (wb, "PlanPerfect", FALSE, TRUE);
 	sheet = sheet_new (wb, name);
@@ -679,7 +677,7 @@ pln_file_open (GnumFileOpener const *fo, IOContext *io_context,
 	error = pln_parse_sheet (input, &state);
 	g_hash_table_destroy (state.styles);
 	if (error != NULL) {
-		workbook_sheet_detach (wb, sheet);
+		workbook_sheet_detach (wb, sheet, FALSE);
 		gnumeric_io_error_info_set (io_context, error);
 	}
 }
