@@ -6,7 +6,7 @@
  *    Sean Atkinson <sca20@cam.ac.uk>
  **/
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "xbase.h"
 
@@ -72,18 +72,20 @@ xb_setdouble (guint8 *p, double d)
 
 #endif
 
-static Value *
+static GnmValue *
 xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 {
 	gchar *s = g_strndup (content, field->len);
-	Value *val;
+	GnmValue *val;
 
 	switch (field->type) {
 	case 'C': {
-		val = value_new_string_nocopy (
-			g_convert_with_iconv (g_strchomp (s), -1,
-			      file->char_map, NULL, NULL, NULL));
-		g_free (s);
+		if (file->char_map != (GIConv)-1)
+			val = value_new_string_nocopy (
+				g_convert_with_iconv (g_strchomp (s), -1,
+				      file->char_map, NULL, NULL, NULL));
+		else
+			val = value_new_string_nocopy (g_strchomp (s));
 		return val;
 	}
 	case 'N':
@@ -113,7 +115,8 @@ xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 		int year, month, day;
 		if (sscanf (s, "%4d%2d%2d", &year, &month, &day) == 3) {
 			GDate *date = g_date_new_dmy (day, month, year);
-			val = value_new_int (datetime_g_to_serial (date));
+			/* Use default date convention */
+			val = value_new_int (datetime_g_to_serial (date, NULL));
 			g_date_free (date);
 		} else
 			val = value_new_string (s);
@@ -155,8 +158,8 @@ xbase_file_open (GnmFileOpener const *fo, IOContext *io_context,
 	XBrecord  *record;
 	char	  *name;
 	Sheet	  *sheet = NULL;
-	Cell	  *cell;
-	Value	  *val;
+	GnmCell	  *cell;
+	GnmValue	  *val;
 	XBfield	  *field;
 	ErrorInfo *open_error;
 	guint row, i;
@@ -180,8 +183,8 @@ xbase_file_open (GnmFileOpener const *fo, IOContext *io_context,
 		cell_set_text (cell, file->format [i]->name);
 	}
 	{
-		Range r;
-		MStyle *bold = mstyle_new ();
+		GnmRange r;
+		GnmStyle *bold = mstyle_new ();
 		mstyle_set_font_bold (bold, TRUE);
 		sheet_style_apply_range	(sheet,
 			range_init (&r, 0, 0, file->fields-1, 0), bold);
