@@ -21,7 +21,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "xml-io-version.h"
 #include "io-context.h"
@@ -59,6 +59,7 @@
 #include <libxml/parserInternals.h>
 #include <stdlib.h>
 #include <string.h>
+#include <locale.h>
 
 GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
@@ -138,7 +139,7 @@ xml_sax_attr_int (xmlChar const * const *attrs, char const *name, int *res)
 }
 
 static gboolean
-xml_sax_attr_cellpos (xmlChar const * const *attrs, char const *name, CellPos *val)
+xml_sax_attr_cellpos (xmlChar const * const *attrs, char const *name, GnmCellPos *val)
 {
 	g_return_val_if_fail (attrs != NULL, FALSE);
 	g_return_val_if_fail (attrs[0] != NULL, FALSE);
@@ -156,7 +157,7 @@ xml_sax_attr_cellpos (xmlChar const * const *attrs, char const *name, CellPos *v
 }
 
 static gboolean
-xml_sax_attr_color (xmlChar const * const *attrs, char const *name, StyleColor **res)
+xml_sax_attr_color (xmlChar const * const *attrs, char const *name, GnmColor **res)
 {
 	int red, green, blue;
 
@@ -177,7 +178,7 @@ xml_sax_attr_color (xmlChar const * const *attrs, char const *name, StyleColor *
 }
 
 static gboolean
-xml_sax_attr_range (xmlChar const * const *attrs, Range *res)
+xml_sax_attr_range (xmlChar const * const *attrs, GnmRange *res)
 {
 	int flags = 0;
 	for (; attrs[0] && attrs[1] ; attrs += 2)
@@ -233,13 +234,13 @@ typedef struct {
 	} validation;
 
 	gboolean  style_range_init;
-	Range	  style_range;
-	MStyle   *style;
+	GnmRange	  style_range;
+	GnmStyle   *style;
 
-	CellPos cell;
+	GnmCellPos cell;
 	int expr_id, array_rows, array_cols;
 	int value_type;
-	StyleFormat *value_fmt;
+	GnmFormat *value_fmt;
 
 	int display_formulas;
 	int hide_zero;
@@ -249,7 +250,7 @@ typedef struct {
 	int display_outlines;
 	int outline_symbols_below;
 	int outline_symbols_right;
-	StyleColor *tab_color;
+	GnmColor *tab_color;
 
 	/* expressions with ref > 1 a map from index -> expr pointer */
 	GHashTable *expr_map;
@@ -341,6 +342,26 @@ xml_sax_wb_view (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	if (width > 0 && height > 0)
 		wb_view_preferred_size (state->wb_view, width, height);
 }
+static void
+xml_sax_calculation (GsfXMLIn *gsf_state, xmlChar const **attrs)
+{
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	gboolean b;
+	int 	 i;
+	double	 d;
+
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
+		if (xml_sax_attr_bool (attrs, "ManualRecalc", &b))
+			workbook_autorecalc_enable (state->wb, !b);
+		else if (xml_sax_attr_bool (attrs, "EnableIteration", &b))
+			workbook_iteration_enabled (state->wb, b);
+		else if (xml_sax_attr_int  (attrs, "MaxIterations", &i))
+			workbook_iteration_max_number (state->wb, i);
+		else if (xml_sax_attr_double (attrs, "IterationTolerance", &d))
+			workbook_iteration_tolerance (state->wb, d);
+		else
+			unknown_attr (state, attrs, "WorkbookView");
+}
 
 static void
 xml_sax_finish_parse_wb_attr (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
@@ -387,7 +408,7 @@ xml_sax_sheet_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
 	gboolean tmp;
-	StyleColor *color = NULL;
+	GnmColor *color = NULL;
 
 	state->hide_col_header = state->hide_row_header =
 	state->display_formulas = state->hide_zero =
@@ -584,7 +605,7 @@ xml_sax_selection_range (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
-	Range r;
+	GnmRange r;
 	if (xml_sax_attr_range (attrs, &r))
 		sv_selection_add_range (
 			sheet_get_view (state->sheet, state->wb_view),
@@ -621,7 +642,7 @@ xml_sax_selection_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
-	CellPos const pos = state->cell;
+	GnmCellPos const pos = state->cell;
 	state->cell.col = state->cell.row = -1;
 	sv_set_edit_pos (sheet_get_view (state->sheet, state->wb_view), &pos);
 }
@@ -631,7 +652,7 @@ xml_sax_sheet_layout (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
-	CellPos tmp;
+	GnmCellPos tmp;
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
 		if (xml_sax_attr_cellpos (attrs, "TopLeft", &tmp))
@@ -647,7 +668,7 @@ xml_sax_sheet_freezepanes (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
-	CellPos frozen_tl, unfrozen_tl;
+	GnmCellPos frozen_tl, unfrozen_tl;
 	int flags = 0;
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
@@ -778,7 +799,7 @@ xml_sax_styleregion_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
 	int val;
-	StyleColor *colour;
+	GnmColor *colour;
 
 	g_return_if_fail (state->style != NULL);
 
@@ -876,7 +897,7 @@ font_component (const char *fontname, int idx)
  * Returns: A valid style font.
  */
 static void
-style_font_read_from_x11 (MStyle *mstyle, const char *fontname)
+style_font_read_from_x11 (GnmStyle *mstyle, const char *fontname)
 {
 	char const *c;
 
@@ -984,7 +1005,7 @@ xml_sax_validation_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob
 
 	int const i = gsf_state->node->user_data.v_int;
 	GnmExpr const *expr;
-	ParsePos pos;
+	GnmParsePos pos;
 
 	g_return_if_fail (state->validation.expr [i] == NULL);
 
@@ -1002,7 +1023,7 @@ xml_sax_style_region_borders (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
 	int pattern = -1;
-	StyleColor *colour = NULL;
+	GnmColor *colour = NULL;
 
 	g_return_if_fail (state->style != NULL);
 
@@ -1016,7 +1037,7 @@ xml_sax_style_region_borders (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	if (pattern >= STYLE_BORDER_NONE) {
 		MStyleElementType const type = gsf_state->node->user_data.v_int;
-		StyleBorder *border =
+		GnmBorder *border =
 			style_border_fetch ((StyleBorderType)pattern, colour,
 					    style_border_get_orientation (type));
 		mstyle_set_border (state->style, type, border);
@@ -1031,7 +1052,7 @@ xml_sax_cell (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	int row = -1, col = -1;
 	int rows = -1, cols = -1;
 	int value_type = -1;
-	StyleFormat *value_fmt = NULL;
+	GnmFormat *value_fmt = NULL;
 	int expr_id = -1;
 
 	g_return_if_fail (state->cell.row == -1);
@@ -1083,10 +1104,10 @@ xml_sax_cell (GsfXMLIn *gsf_state, xmlChar const **attrs)
  * @cols : The number of columns.
  */
 static void
-xml_cell_set_array_expr (Cell *cell, char const *text,
+xml_cell_set_array_expr (GnmCell *cell, char const *text,
 			 int const cols, int const rows)
 {
-	ParsePos pp;
+	GnmParsePos pp;
 	GnmExpr const *expr = gnm_expr_parse_str_simple (text,
 		parse_pos_init_cell (&pp, cell));
 
@@ -1105,7 +1126,7 @@ xml_cell_set_array_expr (Cell *cell, char const *text,
  *     If it is not a member of an array return TRUE.
  */
 static gboolean
-xml_not_used_old_array_spec (Cell *cell, char const *content)
+xml_not_used_old_array_spec (GnmCell *cell, char const *content)
 {
 	int rows, cols, row, col;
 
@@ -1150,7 +1171,7 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
 	gboolean is_new_cell, is_post_52_array = FALSE;
-	Cell *cell;
+	GnmCell *cell;
 
 	int const col = state->cell.col;
 	int const row = state->cell.row;
@@ -1158,7 +1179,7 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	int const array_rows = state->array_rows;
 	int const expr_id = state->expr_id;
 	int const value_type = state->value_type;
-	StyleFormat *value_fmt = state->value_fmt;
+	GnmFormat *value_fmt = state->value_fmt;
 	gpointer const id = GINT_TO_POINTER (expr_id);
 	gpointer expr = NULL;
 
@@ -1195,7 +1216,7 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 		} else if (state->version >= GNUM_XML_V3 ||
 			   xml_not_used_old_array_spec (cell, content)) {
 			if (value_type > 0) {
-				Value *v = value_new_from_string (value_type, content, value_fmt, FALSE);
+				GnmValue *v = value_new_from_string (value_type, content, value_fmt, FALSE);
 				cell_set_value (cell, v);
 			} else
 				cell_set_text (cell, content);
@@ -1239,12 +1260,12 @@ xml_sax_merge (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
-	Range r;
+	GnmRange r;
 	g_return_if_fail (state->base.content->len > 0);
 
 	if (parse_range (state->base.content->str, &r))
 		sheet_merge_add (state->sheet, &r, FALSE,
-			COMMAND_CONTEXT (state->context));
+			GNM_CMD_CONTEXT (state->context));
 }
 
 static void
@@ -1257,16 +1278,16 @@ xml_sax_named_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
 
-	ParseError  perr;
-	ParsePos    pos;
+	GnmParseError  perr;
+	GnmParsePos    pos;
 	GnmExpr const	*expr;
 
 	g_return_if_fail (state->name.name != NULL);
 	g_return_if_fail (state->name.value != NULL);
 
-	parse_pos_init_sheet (&pos, state->sheet);
+	parse_pos_init (&pos, state->wb, state->sheet, 0, 0);
 	if (state->name.position) {
-		CellRef tmp;
+		GnmCellRef tmp;
 		char const *res = cellref_parse (&tmp, state->name.position, &pos.eval);
 		if (res != NULL && *res == '\0') {
 			pos.eval.col = tmp.col;
@@ -1464,6 +1485,7 @@ GSF_XML_IN_NODE_FULL (START, WB, GNM, "Workbook", FALSE, TRUE, FALSE, &xml_sax_w
 
   GSF_XML_IN_NODE (WB, WB_GEOMETRY, GNM, "Geometry", FALSE, &xml_sax_wb_view, NULL),
   GSF_XML_IN_NODE (WB, WB_VIEW, GNM, "UIData", FALSE, &xml_sax_wb_view, NULL),
+  GSF_XML_IN_NODE (WB, WB_CALC, GNM, "Calculation", FALSE, &xml_sax_calculation, NULL),
   { NULL }
 };
 static GsfXMLInDoc *doc;
@@ -1557,6 +1579,7 @@ xml_sax_file_open (GnmFileOpener const *fo, IOContext *io_context,
 		   WorkbookView *wb_view, GsfInput *input)
 {
 	XMLSaxParseState state;
+	char *old_num_locale, *old_monetary_locale;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wb_view));
 	g_return_if_fail (GSF_IS_INPUT (input));
@@ -1588,11 +1611,23 @@ xml_sax_file_open (GnmFileOpener const *fo, IOContext *io_context,
 	input = maybe_convert (input, FALSE);
 	gsf_input_seek (input, 0, G_SEEK_SET);
 
+	old_num_locale = g_strdup (gnm_setlocale (LC_NUMERIC, NULL));
+	gnm_setlocale (LC_NUMERIC, "C");
+	old_monetary_locale = g_strdup (gnm_setlocale (LC_MONETARY, NULL));
+	gnm_setlocale (LC_MONETARY, "C");
+	gnm_set_untranslated_bools ();
+
 	if (!gsf_xml_in_parse (&state.base, input))
 		gnumeric_io_error_string (io_context, _("XML document not well formed!"));
 	else
 		workbook_queue_all_recalc (state.wb);
 
+	/* gnm_setlocale restores bools to locale translation */
+	gnm_setlocale (LC_MONETARY, old_monetary_locale);
+	g_free (old_monetary_locale);
+	gnm_setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+
 	g_object_unref (input);
 
 	/* cleanup */
