@@ -59,7 +59,7 @@ gnm_persist_stream_load (PortableServer_Servant  servant,
 
 	object        = bonobo_object_from_servant (servant);
 	wbc = GNM_PERSIST_STREAM (object)->wbc;
-	ioc = gnumeric_io_context_new (COMMAND_CONTEXT (wbc));
+	ioc = gnumeric_io_context_new (GNM_CMD_CONTEXT (wbc));
 	input = GSF_INPUT (gsf_input_bonobo_new (stream, NULL));
 	wb_view = wb_view_new_from_input  (input, NULL, ioc, NULL);
 	if (gnumeric_io_error_occurred (ioc) || wb_view == NULL) {
@@ -83,8 +83,8 @@ gnm_persist_stream_load (PortableServer_Servant  servant,
 	}
 	g_object_ref (G_OBJECT (wbc));
 	workbook_unref (old_wb);
-	workbook_control_set_view (wbc, wb_view, NULL);
-	workbook_control_init_state (wbc);
+	wb_control_set_view (wbc, wb_view, NULL);
+	wb_control_init_state (wbc);
 }
 
 static void
