@@ -6,7 +6,7 @@
  *  Miguel de Icaza (miguel@gnu.org)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "clipboard.h"
 
@@ -33,7 +33,7 @@
 #include <string.h>
 
 static gboolean
-cell_has_expr_or_number_or_blank (Cell const * cell)
+cell_has_expr_or_number_or_blank (GnmCell const * cell)
 {
 	return (cell_is_empty (cell) ||
 		(cell != NULL && cell_is_number (cell)) ||
@@ -41,7 +41,7 @@ cell_has_expr_or_number_or_blank (Cell const * cell)
 }
 
 static GnmExpr const *
-cell_get_contents_as_expr (Cell const * cell)
+cell_get_contents_as_expr (GnmCell const * cell)
 {
 	GnmExpr const *expr = NULL;
 
@@ -53,7 +53,7 @@ cell_get_contents_as_expr (Cell const * cell)
 		expr = cell->base.expression;
 		gnm_expr_ref (expr);
 	} else if (cell_is_number (cell))
-		expr = gnm_expr_new_constant (value_duplicate (cell->value));
+		expr = gnm_expr_new_constant (value_dup (cell->value));
 	else
 		g_assert_not_reached ();
 
@@ -81,10 +81,10 @@ paste_op_to_expr_op (int paste_flags)
 
 #warning handle formating
 static GnmValue *
-apply_paste_oper_to_values (Cell const *old_cell, Cell const *copied_cell,
-			    Cell const *new_cell, int paste_flags)
+apply_paste_oper_to_values (GnmCell const *old_cell, GnmCell const *copied_cell,
+			    GnmCell const *new_cell, int paste_flags)
 {
-	EvalPos pos;
+	GnmEvalPos pos;
 	GnmExpr expr, arg_a, arg_b;
 	GnmExprOp op = paste_op_to_expr_op (paste_flags);
 
@@ -108,7 +108,7 @@ paste_cell_with_operation (Sheet *dest_sheet,
 			   GnmExprRewriteInfo const *rwinfo,
 			   CellCopy *c_copy, int paste_flags)
 {
-	Cell *new_cell;
+	GnmCell *new_cell;
 
 	g_return_if_fail (paste_flags & PASTE_OPER_MASK);
 
@@ -144,10 +144,10 @@ paste_cell_with_operation (Sheet *dest_sheet,
  * this is called.
  */
 static void
-paste_link (PasteTarget const *pt, int top, int left,
-	    CellRegion const *content)
+paste_link (GnmPasteTarget const *pt, int top, int left,
+	    GnmCellRegion const *content)
 {
-	Cell *cell;
+	GnmCell *cell;
 	GnmCellPos pos;
 	GnmExpr const *expr;
 	GnmCellRef source_cell_ref;
@@ -199,7 +199,7 @@ paste_cell (Sheet *dest_sheet,
 {
 #warning we need to dup the author too
 	if ((paste_flags & PASTE_COMMENTS) && c_copy->comment) {
-		CellComment   *cell_comment;
+		GnmComment   *cell_comment;
 		GnmCellPos       pos;
 		pos.col = target_col;
 		pos.row = target_row;
@@ -220,8 +220,8 @@ paste_cell (Sheet *dest_sheet,
 	}
 
 	if (c_copy->type == CELL_COPY_TYPE_CELL) {
-		Cell *new_cell = sheet_cell_fetch (dest_sheet, target_col, target_row);
-		Cell *src_cell = c_copy->u.cell;
+		GnmCell *new_cell = sheet_cell_fetch (dest_sheet, target_col, target_row);
+		GnmCell *src_cell = c_copy->u.cell;
 
 		if (!src_cell) {
 			g_warning ("Cell copy type set but no cell found (this is bad!)");
@@ -230,14 +230,14 @@ paste_cell (Sheet *dest_sheet,
 
 		if (cell_has_expr (src_cell)) {
 			cell_set_expr_and_value (new_cell, src_cell->base.expression,
-						 value_duplicate (src_cell->value), FALSE);
+						 value_dup (src_cell->value), FALSE);
 
 			if (paste_flags & PASTE_CONTENT)
 				cell_relocate (new_cell, rwinfo);
 			else
 				cell_convert_expr_to_value (new_cell);
 		} else
-				cell_set_value (new_cell, value_duplicate (src_cell->value));
+				cell_set_value (new_cell, value_dup (src_cell->value));
 
 	} else if (c_copy->u.text)
 		cell_set_text (sheet_cell_fetch (dest_sheet, target_col, target_row),
@@ -246,24 +246,25 @@ paste_cell (Sheet *dest_sheet,
 
 /**
  * clipboard_paste_region:
- * @content : The CellRegion to paste.
+ * @content : The GnmCellRegion to paste.
  * @pt : Where to paste the values.
  * @cc : The context for error handling.
  *
- * Pastes the supplied CellRegion (@content) into the supplied
- * PasteTarget (@pt).  This operation is not undoable.  It does not auto grow
+ * Pastes the supplied GnmCellRegion (@content) into the supplied
+ * GnmPasteTarget (@pt).  This operation is not undoable.  It does not auto grow
  * the destination if the target is a singleton.  This is a simple interface to
  * paste a region.
  *
  * returns : TRUE if there was a problem.
  */
 gboolean
-clipboard_paste_region (CellRegion const *content,
-			PasteTarget const *pt,
-			CommandContext *cc)
+clipboard_paste_region (GnmCellRegion const *content,
+			GnmPasteTarget const *pt,
+			GnmCmdContext *cc)
 {
 	int repeat_horizontal, repeat_vertical, clearFlags;
-	int dst_cols, dst_rows, src_cols, src_rows, tmp;
+	int dst_cols, dst_rows, src_cols, src_rows;
+	int i, j;
 	GnmRange const *r;
 	gboolean has_content, adjust_merges = TRUE;
 
@@ -297,7 +298,7 @@ clipboard_paste_region (CellRegion const *content,
 	}
 
 	if (content->not_as_content && (pt->paste_flags & PASTE_CONTENT)) {
-		gnumeric_error_invalid (cc,
+		gnm_cmd_context_error_invalid (cc,
 					_("Unable to paste"),
 					_("Content can only be pasted by value or by link."));
 		return TRUE;
@@ -310,7 +311,7 @@ clipboard_paste_region (CellRegion const *content,
 			_("destination does not have an even multiple of source columns (%d vs %d)\n\n"
 			  "Try selecting a single cell or an area of the same shape and size."),
 			dst_cols, src_cols);
-		gnumeric_error_invalid (cc, _("Unable to paste"), msg);
+		gnm_cmd_context_error_invalid (cc, _("Unable to paste"), msg);
 		g_free (msg);
 		return TRUE;
 	}
@@ -321,14 +322,14 @@ clipboard_paste_region (CellRegion const *content,
 			_("destination does not have an even multiple of source rows (%d vs %d)\n\n"
 			  "Try selecting a single cell or an area of the same shape and size."),
 			dst_rows, src_rows);
-		gnumeric_error_invalid (cc, _("Unable to paste"), msg);
+		gnm_cmd_context_error_invalid (cc, _("Unable to paste"), msg);
 		g_free (msg);
 		return TRUE;
 	}
 
 	if ((pt->range.start.col + dst_cols) > SHEET_MAX_COLS ||
 	    (pt->range.start.row + dst_rows) > SHEET_MAX_ROWS) {
-		gnumeric_error_invalid (cc,
+		gnm_cmd_context_error_invalid (cc,
 					_("Unable to paste"),
 					_("result passes the sheet boundary"));
 		return TRUE;
@@ -362,11 +363,11 @@ clipboard_paste_region (CellRegion const *content,
 				    clearFlags, cc);
 	}
 
-	for (tmp = repeat_vertical; repeat_horizontal-- > 0 ; repeat_vertical = tmp)
-		while (repeat_vertical-- > 0) {
-			int const left = repeat_horizontal * src_cols + pt->range.start.col;
-			int const top = repeat_vertical * src_rows + pt->range.start.row;
-			CellCopyList *l;
+	for (i = 0; i < repeat_horizontal ; i++)
+		for (j = 0; j < repeat_vertical ; j++) {
+			int const left = i * src_cols + pt->range.start.col;
+			int const top = j * src_rows + pt->range.start.row;
+			CellCopyList *ptr;
 			GnmExprRewriteInfo   rwinfo;
 			GnmExprRelocateInfo *rinfo;
 
@@ -416,8 +417,8 @@ clipboard_paste_region (CellRegion const *content,
 			}
 
 
-			for (l = content->content; l; l = l->next) {
-				CellCopy *c_copy = l->data;
+			for (ptr = content->content; ptr; ptr = ptr->next) {
+				CellCopy *c_copy = ptr->data;
 				int target_col = left;
 				int target_row = top;
 
@@ -459,12 +460,12 @@ clipboard_paste_region (CellRegion const *content,
 }
 
 static GnmValue *
-clipboard_prepend_cell (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
+clipboard_prepend_cell (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
 {
-	CellRegion *cr = user_data;
+	GnmCellRegion *cr = user_data;
 	GnmExprArray const *a;
 	CellCopy *copy;
-	CellComment   *comment;
+	GnmComment   *comment;
 
 	copy = g_new (CellCopy, 1);
 	copy->type       = CELL_COPY_TYPE_CELL;
@@ -477,7 +478,7 @@ clipboard_prepend_cell (Sheet *sheet, int col, int row, Cell *cell, void *user_d
 	else
 		copy->comment = NULL;
 
-	cr->content = g_list_prepend (cr->content, copy);
+	cr->content = g_slist_prepend (cr->content, copy);
 
 	/* Check for array division */
 	if (!cr->not_as_content && NULL != (a = cell_is_array (cell))) {
@@ -499,8 +500,8 @@ static void
 clipboard_prepend_comment (SheetObject const *so, void *user_data)
 {
 	GnmRange const *r = sheet_object_range_get (so);
-	Sheet       *sheet = sheet_object_get_sheet (so);
-	Cell        *the_cell = sheet_cell_get (sheet, r->start.col, r->start.row);
+	Sheet   *sheet = sheet_object_get_sheet (so);
+	GnmCell *the_cell = sheet_cell_get (sheet, r->start.col, r->start.row);
 
 	if (the_cell == NULL)
 		clipboard_prepend_cell (sheet,  r->start.col, r->start.row,
@@ -513,10 +514,10 @@ clipboard_prepend_comment (SheetObject const *so, void *user_data)
  *
  * Entry point to the clipboard copy code
  */
-CellRegion *
+GnmCellRegion *
 clipboard_copy_range (Sheet *sheet, GnmRange const *r)
 {
-	CellRegion *cr;
+	GnmCellRegion *cr;
 	GSList *merged, *ptr;
 	GSList *comments;
 
@@ -549,8 +550,8 @@ clipboard_copy_range (Sheet *sheet, GnmRange const *r)
 	return cr;
 }
 
-PasteTarget*
-paste_target_init (PasteTarget *pt, Sheet *sheet, GnmRange const *r, int flags)
+GnmPasteTarget*
+paste_target_init (GnmPasteTarget *pt, Sheet *sheet, GnmRange const *r, int flags)
 {
 	pt->sheet = sheet;
 	pt->range = *r;
@@ -564,10 +565,10 @@ paste_target_init (PasteTarget *pt, Sheet *sheet, GnmRange const *r, int flags)
  *
  * A convenience routine to create CellRegions and init the flags nicely.
  */
-CellRegion *
+GnmCellRegion *
 cellregion_new (Sheet *origin_sheet)
 {
-	CellRegion *cr = g_new0 (CellRegion, 1);
+	GnmCellRegion *cr = g_new0 (GnmCellRegion, 1);
 	cr->origin_sheet	= origin_sheet;
 	cr->cols = cr->rows	= -1;
 	cr->not_as_content	= FALSE;
@@ -582,17 +583,17 @@ cellregion_new (Sheet *origin_sheet)
  * cellregion_free :
  * @content :
  *
- * A convenience routine to free the memory associated with a CellRegion.
+ * A convenience routine to free the memory associated with a GnmCellRegion.
  */
 void
-cellregion_free (CellRegion *cr)
+cellregion_free (GnmCellRegion *cr)
 {
-	CellCopyList *l;
+	CellCopyList *ptr;
 
 	g_return_if_fail (cr != NULL);
 
-	for (l = cr->content; l; l = l->next) {
-		CellCopy *this_cell = l->data;
+	for (ptr = cr->content; ptr; ptr = ptr->next) {
+		CellCopy *this_cell = ptr->data;
 
 		if (this_cell->type == CELL_COPY_TYPE_CELL) {
 			/* The cell is not really in the rows or columns */
@@ -606,7 +607,7 @@ cellregion_free (CellRegion *cr)
 			g_free (this_cell->comment);
 		g_free (this_cell);
 	}
-	g_list_free (cr->content);
+	g_slist_free (cr->content);
 	cr->content = NULL;
 
 	if (cr->styles != NULL) {
@@ -628,13 +629,13 @@ cellregion_free (CellRegion *cr)
  * cellregion_to_string
  * @cr :
  *
- * Renders a CellRegion as a sequence of strings.
+ * Renders a GnmCellRegion as a sequence of strings.
  */
 char *
-cellregion_to_string (PangoContext *context, CellRegion const *cr)
+cellregion_to_string (PangoContext *context, GnmCellRegion const *cr)
 {
 	GString *all, *line;
-	GList *l;
+	CellCopyList *ptr;
 	char ***data, *return_val;
 	int col, row;
 
@@ -645,13 +646,13 @@ cellregion_to_string (PangoContext *context, CellRegion const *cr)
 	for (row = 0; row < cr->rows; row++)
 		data[row] = g_new0 (char *, cr->cols);
 
-	for (l = cr->content; l; l = l->next) {
-		CellCopy *c_copy = l->data;
+	for (ptr = cr->content; ptr; ptr = ptr->next) {
+		CellCopy *c_copy = ptr->data;
 		char *v;
 
 		if (c_copy->type != CELL_COPY_TYPE_TEXT) {
 			/* FIXME: feels bogus to use rendered value.  */
-			MStyle const *mstyle = style_list_get_style (cr->styles,
+			GnmStyle const *mstyle = style_list_get_style (cr->styles,
 				&c_copy->u.cell->pos);
 			RenderedValue *rv;
 
