@@ -2,7 +2,7 @@
 /*
  * gog-axis.c :
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,7 +19,7 @@
  * USA
  */
 
-#include <gnumeric-config.h>
+#include <goffice/goffice-config.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-styled-object.h>
 #include <goffice/graph/gog-style.h>
@@ -35,7 +35,7 @@
 #include <goffice/utils/go-format.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <src/gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <src/gui-util.h>
 #include <src/mathfunc.h>
 #include <src/format.h>
@@ -309,7 +309,7 @@ gog_axis_get_property (GObject *obj, guint param_id,
 		break;
 	case AXIS_PROP_ASSIGNED_FORMAT_STR_XL :
 		if (axis->assigned_format != NULL)
-			g_value_set_string_take_ownership (value,
+			g_value_take_string (value,
 				go_format_as_XL	(axis->assigned_format, FALSE));
 		else
 			g_value_set_static_string (value, NULL);
@@ -344,8 +344,7 @@ gog_axis_finalize (GObject *obj)
 	}
 
 	gog_dataset_finalize (GOG_DATASET (axis));
-	if (parent_klass != NULL && parent_klass->finalize != NULL)
-		(parent_klass->finalize) (obj);
+	(parent_klass->finalize) (obj);
 }
 
 static double
@@ -647,7 +646,7 @@ gog_axis_editor (GogObject *gobj, GogDataAllocator *dalloc, GnmCmdContext *cc)
 	gtk_notebook_prepend_page (GTK_NOTEBOOK (notebook),
 		glade_xml_get_widget (gui, "axis_pref_table"),
 		gtk_label_new (_("Details")));
-	gog_style_editor (GOG_STYLED_OBJECT (gobj), cc, notebook);
+	gog_styled_object_editor (GOG_STYLED_OBJECT (gobj), cc, notebook);
 
 	w = glade_xml_get_widget (gui, "axis_low");
 	if (axis->pos == GOG_AXIS_AT_LOW)
@@ -1080,7 +1079,7 @@ gog_axis_get_marker (GogAxis *axis, unsigned i)
 	if (axis->is_discrete) {
 		if (axis->labels != NULL) {
 			if ((int)i < go_data_vector_get_len (axis->labels))
-				return go_data_vector_get_str (axis->labels, i);
+			return go_data_vector_get_str (axis->labels, i);
 			return g_strdup ("");
 		}
 		return g_strdup_printf ("%d", i+1);
@@ -1322,6 +1321,8 @@ gog_axis_view_render (GogView *v, GogViewAllocation const *bbox)
 				bound = label_pos.x + label_result.w;
 			}
 		}
+		if (line_width > 0)
+			gog_renderer_draw_path (v->renderer, axis_path, NULL);
 		break;
 
 	case GOG_AXIS_Y:
@@ -1410,13 +1411,41 @@ gog_axis_view_render (GogView *v, GogViewAllocation const *bbox)
 				bound = label_pos.y - label_result.h;
 			}
 		}
+		if (line_width > 0)
+			gog_renderer_draw_path (v->renderer, axis_path, NULL);
+		break;
+	case GOG_AXIS_CIRCULAR:
+		{
+			ArtVpath  path[3];
+			double center_x, center_y, radius;
+			unsigned i;
+
+			path[0].code = ART_MOVETO;
+			path[1].code = ART_LINETO;
+			path[2].code = ART_END;
+
+			center_x = area->x + (area->w/2);
+			center_y = area->y + (area->h/2);
+			radius = v->allocation.h > v->allocation.w 
+				? v->allocation.w / 2.0 
+				: v->allocation.h / 2.0;
+			path[0].x = center_x;
+			path[0].y = center_y;
+
+			for (i = 0; i < n-1; i++) {
+				double angle_rad = i * 2.0 * M_PI/(n-1);
+
+				path[1].x = center_x + radius * sin(angle_rad);
+				path[1].y = center_y - radius * cos(angle_rad);
+				gog_renderer_draw_path (v->renderer, path, 
+							NULL);
+			}
+		}
 		break;
 	default :
 		break;
 	}
 
-	if (line_width > 0)
-		gog_renderer_draw_path (v->renderer, axis_path, NULL);
 	gog_renderer_pop_style (v->renderer);
 }
 
