@@ -3,7 +3,7 @@
 /*
  * go-graph-guru.c:  The Graph guru
  *
- * Copyright (C) 2000-2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  */
-#include <gnumeric-config.h>
+#include <goffice/goffice-config.h>
 
 #include <goffice/graph/gog-guru.h>
 #include <goffice/graph/gog-object.h>
@@ -33,7 +33,7 @@
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-control-foocanvas.h>
 
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gui-util.h>
 
 #include <libxml/parser.h>
@@ -41,6 +41,21 @@
 #include <libfoocanvas/foo-canvas-pixbuf.h>
 #include <libfoocanvas/foo-canvas-rect-ellipse.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreestore.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkviewport.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcellrendererpixbuf.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkliststore.h>
 #include <string.h>
 
 typedef struct _GraphGuruState		GraphGuruState;
@@ -51,10 +66,9 @@ struct _GraphGuruState {
 	GogChart    *chart;
 	GogPlot	    *plot;
 
-	CommandContext	 *cc;
+	GnmCmdContext	 *cc;
 	GogDataAllocator *dalloc;
-	GogGuruRegister   register_graph;
-	gpointer	  register_graph_data;
+	GClosure         *register_closure;
 
 	/* GUI accessors */
 	GladeXML    *gui;
@@ -67,7 +81,7 @@ struct _GraphGuruState {
 
 	FooCanvasItem	  *sample_graph_item;
 
-	GtkNotebook	  *prop_notebook;
+	GtkContainer  	  *prop_frame;
 	GtkTreeSelection  *prop_selection;
 	GtkTreeView	  *prop_view;
 	GtkTreeStore	  *prop_model;
@@ -76,6 +90,9 @@ struct _GraphGuruState {
 
 	GraphGuruTypeSelector *type_selector;
 
+	struct {
+		GtkWidget *inc, *dec, *first, *last, *menu;
+	} prec;
 	/* internal state */
 	int current_page, initial_page;
 	gboolean valid;
@@ -87,7 +104,6 @@ struct _GraphGuruState {
 };
 
 struct _GraphGuruTypeSelector {
-	GtkWidget	*notebook;
 	GtkWidget	*canvas;
 	GtkWidget	*sample_button;
 	GtkLabel	*label;
@@ -127,7 +143,6 @@ enum {
 #define PLOT_TYPE_KEY		"plot_type"
 #define FIRST_MINOR_TYPE	"first_minor_type"
 #define ROLE_KEY		"role"
-#define GRAPH_OBJECT_KEY	"graph_obj"
 #define STATE_KEY		"plot_type"
 
 static GdkPixbuf *
@@ -199,8 +214,9 @@ graph_typeselect_minor (GraphGuruTypeSelector *typesel, FooCanvasItem *item)
 	g_return_if_fail (plot != NULL);
 
 	if (s->plot != NULL) {
-		gog_object_clear_parent (GOG_OBJECT (s->plot));
-		g_object_unref (s->plot);
+		GogObject *obj = GOG_OBJECT (s->plot);
+		gog_object_clear_parent (obj);
+		g_object_unref (obj);
 	}
 
 	gog_object_add_by_name (GOG_OBJECT (s->chart),
@@ -225,8 +241,10 @@ graph_typeselect_minor_x_y (GraphGuruTypeSelector *typesel,
 	FooCanvasItem *item = foo_canvas_get_item_at (
 		FOO_CANVAS (typesel->canvas), x, y);
 
-	if (item != NULL && item != typesel->selector) {
-		graph_typeselect_minor (typesel, item);
+	if (item != NULL) {
+		if(item != typesel->selector)
+			graph_typeselect_minor (typesel, item);
+		foo_canvas_item_grab_focus (item);
 		return TRUE;
 	}
 
@@ -444,7 +462,7 @@ cb_plot_families_init (char const *id, GogPlotFamily *family,
 	gtk_list_store_append (typesel->model, &iter);
 	gtk_list_store_set (typesel->model, &iter,
 		PLOT_FAMILY_TYPE_IMAGE,		get_pixbuf (family->sample_image_file),
-		PLOT_FAMILY_TYPE_NAME,		family->name,
+		PLOT_FAMILY_TYPE_NAME,		_(family->name),
 		PLOT_FAMILY_TYPE_CANVAS_GROUP,	group,
 		-1);
 
@@ -461,10 +479,9 @@ cb_plot_families_init (char const *id, GogPlotFamily *family,
 }
 
 static void
-cb_canvas_realized (G_GNUC_UNUSED GtkWidget *widget,
-		    G_GNUC_UNUSED gpointer data)
+cb_canvas_realized (GtkLayout *widget)
 {
-	gdk_window_set_back_pixmap (GTK_LAYOUT (widget)->bin_window, NULL, FALSE);
+	gdk_window_set_back_pixmap (widget->bin_window, NULL, FALSE);
 }
 
 static void
@@ -482,6 +499,11 @@ graph_guru_state_destroy (GraphGuruState *state)
 		state->gui = NULL;
 	}
 
+	if (state->register_closure != NULL) {
+		g_closure_unref (state->register_closure);
+		state->register_closure = NULL;
+	}
+
 	state->dialog = NULL;
 	g_free (state);
 }
@@ -498,7 +520,7 @@ cb_graph_guru_add_item (GtkWidget *w, GraphGuruState *s)
 }
 
 static void
-cb_graph_guru_delete_item (GtkWidget *w, GraphGuruState *s)
+cb_graph_guru_delete_item (GraphGuruState *s)
 {
 	if (s->prop_object != NULL) {
 		GtkTreeIter iter;
@@ -514,20 +536,52 @@ cb_graph_guru_delete_item (GtkWidget *w, GraphGuruState *s)
 }
 
 static void
-cb_select_prop_page (GtkWidget *page, GogObject **target)
+update_prec_menu (GraphGuruState *s, gboolean inc_ok, gboolean dec_ok)
+{
+	gtk_widget_set_sensitive (s->prec.first,    inc_ok);
+	gtk_widget_set_sensitive (s->prec.inc,	    inc_ok);
+	gtk_widget_set_sensitive (s->prec.dec,	    dec_ok);
+	gtk_widget_set_sensitive (s->prec.last,	    dec_ok);
+	gtk_widget_set_sensitive (s->prec.menu,	    dec_ok | inc_ok);
+}
+
+static gboolean
+cb_reordered_find (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+		   GraphGuruState *s)
 {
 	GogObject *obj;
-
-	if (*target == NULL)
-		return;
-	obj = g_object_get_data (G_OBJECT (page), GRAPH_OBJECT_KEY);
-	if (obj == *target) {
-		GtkWidget *notebook = gtk_widget_get_parent (page);
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
-			gtk_notebook_page_num (GTK_NOTEBOOK (notebook), page));
-		*target = NULL;
+	gtk_tree_model_get (model, iter, PLOT_ATTR_OBJECT, &obj, -1);
+	if (obj == s->search_target) {
+		gtk_tree_store_move_after (s->prop_model, &s->prop_iter, iter);
+		return TRUE;
 	}
+
+	return FALSE;
 }
+static void
+reorder (GraphGuruState *s, gboolean inc, gboolean goto_max)
+{
+	gboolean inc_ok, dec_ok;
+	GogObject *after;
+
+	g_return_if_fail (s->search_target == NULL);
+
+	after = gog_object_reorder (s->prop_object, inc, goto_max);
+	if (after != NULL) {
+		s->search_target = after;
+		gtk_tree_model_foreach (GTK_TREE_MODEL (s->prop_model),
+			(GtkTreeModelForeachFunc) cb_reordered_find, s);
+		s->search_target = NULL;
+	} else
+		gtk_tree_store_move_after (s->prop_model, &s->prop_iter, NULL);
+
+	gog_object_can_reorder (s->prop_object, &inc_ok, &dec_ok);
+	update_prec_menu (s, inc_ok, dec_ok);
+}
+static void cb_prec_first (GraphGuruState *s) { reorder (s, TRUE,  TRUE); }
+static void cb_prec_inc	  (GraphGuruState *s) { reorder (s, TRUE,  FALSE); }
+static void cb_prec_dec   (GraphGuruState *s) { reorder (s, FALSE, FALSE); }
+static void cb_prec_last  (GraphGuruState *s) { reorder (s, FALSE, TRUE); }
 
 struct type_menu_create {
 	GraphGuruState *state;
@@ -558,6 +612,8 @@ cb_graph_guru_add_plot (GtkWidget *w, GraphGuruState *s)
 	GogPlot *plot = gog_plot_new_by_type (type);
 	gog_object_add_by_name (GOG_OBJECT (s->prop_object),
 		"Plot", GOG_OBJECT (plot));
+	/* as a convenience add a series to the newly created plot */
+	gog_object_add_by_name (GOG_OBJECT (plot), "Series", NULL);
 }
 
 static void
@@ -571,7 +627,7 @@ cb_plot_family_menu_create (char const *id, GogPlotFamily *family,
 	if (g_hash_table_size (family->types) <= 0)
 		return;
 
-	menu = gtk_image_menu_item_new_with_label (family->name);
+	menu = gtk_image_menu_item_new_with_label (_(family->name));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu),
 		gtk_image_new_from_pixbuf (
 			get_pixbuf (family->sample_image_file)));
@@ -622,15 +678,27 @@ cb_attr_tree_selection_change (GraphGuruState *s)
 {
 	gboolean add_ok = FALSE;
 	gboolean delete_ok = FALSE;
+	gboolean inc_ok = FALSE;
+	gboolean dec_ok = FALSE;
 	GtkTreeModel *model;
-	GogObject  *obj;
+	GogObject  *obj = NULL;
+	GtkWidget *w, *editor;
+	GtkShadowType shadow = GTK_SHADOW_OUT;
 
-	s->prop_object = NULL;
 	if (gtk_tree_selection_get_selected (s->prop_selection, &model, &s->prop_iter))
 		gtk_tree_model_get (model, &s->prop_iter,
-				    PLOT_ATTR_OBJECT, &s->prop_object,
+				    PLOT_ATTR_OBJECT, &obj,
 				    -1);
 
+	if (s->prop_object == obj)
+		return;
+
+	/* remove the old prop page */
+	s->prop_object = obj;
+	w = gtk_bin_get_child (GTK_BIN (s->prop_frame));
+	if (w != NULL)
+		gtk_container_remove (s->prop_frame, w);
+
 	if (s->prop_object != NULL) {
 		/* Setup up the additions menu */
 		GSList *additions = gog_object_possible_additions (s->prop_object);
@@ -670,8 +738,6 @@ cb_attr_tree_selection_change (GraphGuruState *s)
 			gtk_widget_show_all (s->add_menu);
 		}
 
-		obj = s->prop_object;
-
 		/* if we ever go back to the typeselector be sure to 
 		 * add the plot to the last selected chart */
 		s->chart = (GogChart *)
@@ -681,25 +747,22 @@ cb_attr_tree_selection_change (GraphGuruState *s)
 		gtk_widget_set_sensitive (s->button_navigate, s->chart != NULL);
 
 		delete_ok = gog_object_is_deletable (s->prop_object);
+		gog_object_can_reorder (obj, &inc_ok, &dec_ok);
 
-		/* open or create a prefs page for the graph obj */
-		gtk_container_foreach (GTK_CONTAINER (s->prop_notebook),
-			(GtkCallback) cb_select_prop_page, &obj);
-		if (obj != NULL) {
-			int item_page = 0;
-			GtkWidget *editor = gog_object_get_editor (obj, s->dalloc, s->cc);
-			if (editor != NULL) {
-				item_page = gtk_notebook_get_n_pages (s->prop_notebook);
-				gtk_notebook_append_page (s->prop_notebook, editor, NULL);
-				gtk_widget_show (editor);
-				g_object_set_data (G_OBJECT (editor), GRAPH_OBJECT_KEY, obj);
-			}
-			gtk_notebook_set_current_page (s->prop_notebook, item_page);
+		/* create a prefs page for the graph obj */
+		editor = gog_object_get_editor (obj, s->dalloc, s->cc);
+		if (editor != NULL) {
+			if (GTK_IS_NOTEBOOK (editor))
+				shadow = GTK_SHADOW_NONE;
+			gtk_container_add (s->prop_frame, editor);
+			gtk_widget_show (editor);
 		}
 	}
+	gtk_frame_set_shadow_type (GTK_FRAME (s->prop_frame), shadow);
 
 	gtk_widget_set_sensitive (s->delete_button, delete_ok);
-	gtk_widget_set_sensitive (s->add_menu, add_ok);
+	gtk_widget_set_sensitive (s->add_menu,	    add_ok);
+	update_prec_menu (s, inc_ok, dec_ok);
 }
 
 static gboolean
@@ -754,28 +817,21 @@ cb_obj_child_added (GogObject *parent, GogObject *child, GraphGuruState *s)
 	s->new_child = NULL;
 }
 
-static void
-cb_remove_prop_page (GtkWidget *page, GogObject *target)
-{
-	if (target != g_object_get_data (G_OBJECT (page), GRAPH_OBJECT_KEY))
-		return;
-	gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent (page)),
-			      page);
-}
-
 static gboolean
 cb_find_child_removed (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
 		       GraphGuruState *s)
 {
 	GogObject *obj;
+	GtkWidget *w;
 
 	gtk_tree_model_get (model, iter, PLOT_ATTR_OBJECT, &obj, -1);
 	if (obj == s->search_target) {
 		s->search_target = NULL;
 		/* remove the tree element and the prop page */
 		gtk_tree_store_remove (s->prop_model, iter);
-		gtk_container_foreach (GTK_CONTAINER (s->prop_notebook),
-			(GtkCallback) cb_remove_prop_page, &obj);
+		w = gtk_bin_get_child (GTK_BIN (s->prop_frame));
+		if (w != NULL)
+			gtk_container_remove (s->prop_frame, w);
 		return TRUE;
 	}
 
@@ -828,20 +884,20 @@ populate_graph_item_list (GogObject *obj, GogObject *select, GraphGuruState *s,
 	closure = g_cclosure_new (G_CALLBACK (cb_obj_name_changed), s, NULL);
 	g_object_watch_closure (G_OBJECT (s->prop_view), closure);
 	g_signal_connect_closure (G_OBJECT (obj),
-		"name_changed",
+		"name-changed",
 		closure, FALSE);
 	closure = g_cclosure_new (G_CALLBACK (cb_obj_child_added), s, NULL);
 	g_object_watch_closure (G_OBJECT (s->prop_view), closure);
 	g_signal_connect_closure (G_OBJECT (obj),
-		"child_added",
+		"child-added",
 		closure, FALSE);
 	closure = g_cclosure_new (G_CALLBACK (cb_obj_child_removed), s, NULL);
 	g_object_watch_closure (G_OBJECT (s->prop_view), closure);
 	g_signal_connect_closure (G_OBJECT (obj),
-		"child_removed",
+		"child-removed",
 		closure, FALSE);
 
-	children = gog_object_get_children (obj);
+	children = gog_object_get_children (obj, NULL);
 	for (ptr = children ; ptr != NULL ; ptr = ptr->next)
 		populate_graph_item_list (ptr->data, select, s, &iter, FALSE);
 	g_slist_free (children);
@@ -925,10 +981,27 @@ graph_guru_init_format_page (GraphGuruState *s)
 	s->fmt_page_initialized = TRUE;
 	s->add_menu	 = glade_xml_get_widget (s->gui, "add_menu");
 	s->delete_button = glade_xml_get_widget (s->gui, "delete");
+	s->prec.menu  = glade_xml_get_widget (s->gui, "precedence_menu");
+	s->prec.inc   = glade_xml_get_widget (s->gui, "inc_precedence");
+	s->prec.dec   = glade_xml_get_widget (s->gui, "dec_precedence");
+	s->prec.first = glade_xml_get_widget (s->gui, "first_precedence");
+	s->prec.last  = glade_xml_get_widget (s->gui, "last_precedence");
 
-	g_signal_connect (G_OBJECT (s->delete_button),
+	g_signal_connect_swapped (G_OBJECT (s->delete_button),
 		"activate",
 		G_CALLBACK (cb_graph_guru_delete_item), s);
+	g_signal_connect_swapped (G_OBJECT (s->prec.first),
+		"activate",
+		G_CALLBACK (cb_prec_first), s);
+	g_signal_connect_swapped (G_OBJECT (s->prec.inc),
+		"activate",
+		G_CALLBACK (cb_prec_inc), s);
+	g_signal_connect_swapped (G_OBJECT (s->prec.dec),
+		"activate",
+		G_CALLBACK (cb_prec_dec), s);
+	g_signal_connect_swapped (G_OBJECT (s->prec.last),
+		"activate",
+		G_CALLBACK (cb_prec_last), s);
 
 	/* Load up the sample view and make it fill the entire canvas */
 	w = glade_xml_get_widget (s->gui, "sample_canvas");
@@ -946,8 +1019,8 @@ graph_guru_init_format_page (GraphGuruState *s)
 		G_CALLBACK (cb_canvas_select_item), s);
 	gtk_widget_show (w);
 
-	w = glade_xml_get_widget (s->gui, "prop_notebook");
-	s->prop_notebook = GTK_NOTEBOOK (w);
+	w = glade_xml_get_widget (s->gui, "prop_frame");
+	s->prop_frame = GTK_CONTAINER (w);
 	s->prop_model = gtk_tree_store_new (PLOT_ATTR_NUM_COLUMNS,
 				    G_TYPE_STRING, G_TYPE_POINTER);
 	s->prop_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (
@@ -984,7 +1057,7 @@ graph_guru_set_page (GraphGuruState *s, int page)
 		return;
 
 	switch (page) {
-	case 0: name = _("Step 1 of 2: Select graph type");
+	case 0: name = _("Step 1 of 2: Select Chart Type");
 		gtk_widget_set_sensitive (s->button_navigate, s->plot != NULL);
 		gtk_button_set_label (GTK_BUTTON (s->button_navigate),
 				      GTK_STOCK_GO_FORWARD);
@@ -992,19 +1065,19 @@ graph_guru_set_page (GraphGuruState *s, int page)
 
 	case 1:
 		if (s->initial_page == 0) {
-			name = _("Step 2 of 2: Configure Graph");
+			name = _("Step 2 of 2: Customize Chart");
 			gtk_widget_set_sensitive (s->button_navigate, s->chart != NULL);
 			gtk_button_set_label (GTK_BUTTON (s->button_navigate),
 					      GTK_STOCK_GO_BACK);
 		} else {
-			name = _("Configure Graph");
+			name = _("Customize Chart");
 			gtk_widget_hide	(s->button_navigate);
 		}
 		graph_guru_init_format_page (s);
 		break;
 
 	default:
-		g_warning ("Invalid Graph Guru page");
+		g_warning ("Invalid Chart Guru page");
 		return;
 	}
 
@@ -1023,8 +1096,30 @@ cb_graph_guru_clicked (GtkWidget *button, GraphGuruState *s)
 		graph_guru_set_page (s, (s->current_page + 1) % 2);
 		return;
 	} else if (button == s->button_ok &&
-		   s->register_graph != NULL && s->graph != NULL)
-		(s->register_graph) (s->graph, s->register_graph_data);
+		   s->register_closure != NULL && s->graph != NULL) {
+		/* Invoking closure */
+		GValue instance_and_params[2];
+		gpointer data = s->register_closure->is_invalid ? 
+			NULL : s->register_closure->data;
+		
+		instance_and_params[0].g_type = 0;
+		g_value_init (&instance_and_params[0], GOG_GRAPH_TYPE);
+		g_value_set_instance (&instance_and_params[0], s->graph);
+
+		instance_and_params[1].g_type = 0;
+		g_value_init (&instance_and_params[1], G_TYPE_POINTER);
+		g_value_set_pointer (&instance_and_params[1], data);
+
+		g_closure_set_marshal (s->register_closure,
+				       g_cclosure_marshal_VOID__POINTER);
+
+		g_closure_invoke (s->register_closure,
+				  NULL,
+				  2,
+				  instance_and_params,
+				  NULL);
+		g_value_unset (instance_and_params + 0);
+	}
 
 	gtk_widget_destroy (GTK_WIDGET (s->dialog));
 }
@@ -1039,13 +1134,30 @@ graph_guru_init_button (GraphGuruState *s, char const *widget_name)
 	return button;
 }
 
+static void
+typesel_set_selection_color (GraphGuruTypeSelector *typesel)
+{
+	GtkWidget *w = gtk_entry_new ();
+	GdkColor  *color = &w->style->base [GTK_WIDGET_HAS_FOCUS (typesel->canvas)
+		? GTK_STATE_SELECTED : GTK_STATE_ACTIVE];
+	guint32    select_color = 0;
+
+	select_color |= ((color->red >> 8) & 0xff)   << 24;
+	select_color |= ((color->green >> 8) & 0xff) << 16;
+	select_color |= ((color->blue >> 8) & 0xff)  << 8;
+	select_color |= 0x40; /* alpha of 25% */
+	foo_canvas_item_set (typesel->selector,
+		"fill_color_rgba",	select_color,
+		NULL);
+	gtk_object_destroy (GTK_OBJECT (w));
+}
+
 static GtkWidget *
 graph_guru_type_selector_new (GraphGuruState *s)
 {
 	GtkTreeSelection *selection;
 	GraphGuruTypeSelector *typesel;
 	GtkWidget *tmp, *vbox, *hbox;
-	guint32 select_color;
 
 	typesel = g_new0 (GraphGuruTypeSelector, 1);
 	typesel->state = s;
@@ -1070,7 +1182,7 @@ graph_guru_type_selector_new (GraphGuruState *s)
 			"pixbuf", PLOT_FAMILY_TYPE_IMAGE,
 			NULL));
 	gtk_tree_view_append_column (typesel->list_view,
-		gtk_tree_view_column_new_with_attributes (_("Plot Type"),
+		gtk_tree_view_column_new_with_attributes (_("_Plot Type"),
 			gtk_cell_renderer_text_new (),
 			"text", PLOT_FAMILY_TYPE_NAME,
 			NULL));
@@ -1085,32 +1197,26 @@ graph_guru_type_selector_new (GraphGuruState *s)
 
 	/* Setup an canvas to display the sample image & the sample plot. */
 	typesel->canvas = foo_canvas_new ();
-	g_signal_connect (G_OBJECT (typesel->canvas),
-		"realize",
-		G_CALLBACK (cb_canvas_realized), typesel);
-
 	typesel->graph_group = FOO_CANVAS_GROUP (foo_canvas_item_new (
 		foo_canvas_root (FOO_CANVAS (typesel->canvas)),
 		foo_canvas_group_get_type (),
 		"x", 0.0,
 		"y", 0.0,
 		NULL));
-	g_signal_connect (G_OBJECT (typesel->canvas),
-		"size_allocate",
-		G_CALLBACK (cb_typesel_sample_plot_resize), typesel);
+	g_object_connect (typesel->canvas,
+		"signal::realize", G_CALLBACK (cb_canvas_realized), typesel,
+		"signal::size_allocate", G_CALLBACK (cb_typesel_sample_plot_resize), typesel,
+		"signal_after::key_press_event", G_CALLBACK (cb_key_press_event), typesel,
+		"signal::button_press_event", G_CALLBACK (cb_button_press_event), typesel,
+		"swapped_signal::focus_in_event", G_CALLBACK (typesel_set_selection_color), typesel,
+		"swapped_signal::focus_out_event", G_CALLBACK (typesel_set_selection_color), typesel,
+		NULL);
 
 	gtk_widget_set_size_request (typesel->canvas,
 		MINOR_PIXMAP_WIDTH*3 + BORDER*5,
 		MINOR_PIXMAP_HEIGHT*3 + BORDER*5);
 	foo_canvas_scroll_to (FOO_CANVAS (typesel->canvas), 0, 0);
 
-	g_signal_connect_after (G_OBJECT (typesel->canvas),
-		"key_press_event",
-		G_CALLBACK (cb_key_press_event), typesel);
-	g_signal_connect (GTK_OBJECT (typesel->canvas),
-		"button_press_event",
-		G_CALLBACK (cb_button_press_event), typesel);
-
 	tmp = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (tmp), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (tmp), typesel->canvas);
@@ -1120,28 +1226,14 @@ graph_guru_type_selector_new (GraphGuruState *s)
 	g_hash_table_foreach ((GHashTable *)gog_plot_families (),
 		(GHFunc) cb_plot_families_init, typesel);
 
-#if 1
-	/* hard code for now until I figure out where to get a decent colour */
-	select_color = 0xe090f840;
-#else
-	{
-		GdkColor *color = typesel->canvas->style->base + GTK_STATE_SELECTED;
-
-		select_color |= ((color->red >> 8) & 0xff)   << 24;
-		select_color |= ((color->green >> 8) & 0xff) << 16;
-		select_color |= ((color->blue >> 8) & 0xff)  << 8;
-		select_color = 0x40; /* alpha of 25% */
-	}
-#endif
-
 	/* The alpha blended selection box */
 	typesel->selector = foo_canvas_item_new (
 		foo_canvas_root (FOO_CANVAS (typesel->canvas)),
 		foo_canvas_rect_get_type (),
-		"fill_color_rgba",	select_color,
 		"outline_color_rgba",	0x000000ff,	/* black */
 		"width_pixels", 1,
 		NULL);
+	typesel_set_selection_color (typesel);
 
 	/* Setup the description label */
 	typesel->label = GTK_LABEL (gtk_label_new (""));
@@ -1174,14 +1266,10 @@ graph_guru_type_selector_new (GraphGuruState *s)
 	gtk_box_pack_start (GTK_BOX (hbox), tmp, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), typesel->sample_button, FALSE, TRUE, 0);
 
-	typesel->notebook = gtk_notebook_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (typesel->notebook),
-		vbox, gtk_label_new (_("Basic Types")));
-
-	g_object_set_data_full (G_OBJECT (typesel->notebook),
+	g_object_set_data_full (G_OBJECT (vbox),
 		"state", typesel, (GDestroyNotify) g_free);
 
-	return typesel->notebook;
+	return vbox;
 }
 
 static gboolean
@@ -1199,10 +1287,9 @@ graph_guru_init (GraphGuruState *s)
 	s->button_navigate = graph_guru_init_button (s, "button_navigate");
 	s->button_ok	   = graph_guru_init_button (s, "button_ok");
 
-/* FIXME: Add correct helpfile address */
 	gnumeric_init_help_button (
 		glade_xml_get_widget (s->gui, "help_button"),
-		"graph-guru.html");
+		"sect-graphics-plots");
 
 	return FALSE;
 }
@@ -1217,8 +1304,8 @@ graph_guru_init (GraphGuruState *s)
  */
 GtkWidget *
 gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
-	  CommandContext *cc, GtkWindow *toplevel,
-	  GogGuruRegister handler, gpointer handler_data)
+	  GnmCmdContext *cc, GtkWindow *toplevel,
+	  GClosure *closure)
 {
 	int page = (graph != NULL) ? 1 : 0;
 	GraphGuruState *state;
@@ -1230,16 +1317,14 @@ gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
 	state->gui	= NULL;
 	state->cc       = cc;
 	state->dalloc   = dalloc;
-	state->register_graph	   = handler;
-	state->register_graph_data = handler_data;
 	state->current_page	= -1;
+	state->register_closure	= closure;
+	g_closure_ref (closure);
 
 	if (graph != NULL) {
 		g_return_val_if_fail (IS_GOG_GRAPH (graph), NULL);
 
-		state->graph = graph;
-		g_object_ref (graph);
-#warning TODO should dup not ref
+		state->graph = gog_graph_dup (graph);
 		state->chart = NULL;
 		state->plot  = NULL;
 	} else {
