@@ -2,7 +2,7 @@
 /*
  * gog-label.c
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,10 +19,11 @@
  * USA
  */
 
-#include <gnumeric-config.h>
+#include <goffice/goffice-config.h>
 #include <goffice/graph/gog-label.h>
-#include <goffice/graph/gog-styled-object.h>
+#include <goffice/graph/gog-outlined-object.h>
 #include <goffice/graph/gog-style.h>
+#include <goffice/graph/gog-theme.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-data-set.h>
@@ -30,15 +31,15 @@
 #include <goffice/graph/go-data.h>
 
 #include <src/gui-util.h>
-#include <src/gnumeric-i18n.h>
+#include <glib/gi18n.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtklabel.h>
-#include <gtk/gtkbox.h>
+#include <gtk/gtkhbox.h>
 
 struct _GogLabel {
-	GogStyledObject	base;
+	GogOutlinedObject	base;
 
 	GogDatasetElement text;
 	gboolean	  allow_markup;
@@ -54,9 +55,6 @@ static GType gog_label_view_get_type (void);
 static GObjectClass *label_parent_klass;
 static GogViewClass *lview_parent_klass;
 
-/* a property ? */
-#define PAD_HACK	4	/* pts */
-
 static void
 gog_label_set_property (GObject *obj, guint param_id,
 			     GValue const *value, GParamSpec *pspec)
@@ -94,13 +92,13 @@ static void
 gog_label_finalize (GObject *obj)
 {
 	gog_dataset_finalize (GOG_DATASET (obj));
-	if (label_parent_klass->finalize != NULL)
-		(label_parent_klass->finalize) (obj);
+	(*label_parent_klass->finalize) (obj);
 }
 
 static gpointer
-gog_label_editor (GogObject *gobj, GogDataAllocator *dalloc, CommandContext *cc)
+gog_label_editor (GogObject *gobj, GogDataAllocator *dalloc, GnmCmdContext *cc)
 {
+	static guint label_pref_page = 0;
 	GtkWidget *notebook = gtk_notebook_new ();
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 5);
 
@@ -112,15 +110,17 @@ gog_label_editor (GogObject *gobj, GogDataAllocator *dalloc, CommandContext *cc)
 	gtk_widget_show_all (hbox);
 	gtk_notebook_prepend_page (GTK_NOTEBOOK (notebook), hbox,
 		gtk_label_new (_("Data")));
-	gog_style_editor (gobj, cc, notebook,
-		GOG_STYLE_OUTLINE | GOG_STYLE_FILL | GOG_STYLE_FONT);
+	gog_styled_object_editor (GOG_STYLED_OBJECT (gobj), cc, notebook);
+	gog_style_handle_notebook (notebook, &label_pref_page);
 	return notebook;
 }
 
-static unsigned
-gog_label_interesting_fields (GogStyledObject *obj)
+static void
+gog_label_init_style (GogStyledObject *gso, GogStyle *style)
 {
-	return GOG_STYLE_OUTLINE | GOG_STYLE_FILL | GOG_STYLE_FONT;
+	style->interesting_fields = GOG_STYLE_OUTLINE | GOG_STYLE_FILL | GOG_STYLE_FONT;
+	gog_theme_init_style (gog_object_get_theme (GOG_OBJECT (gso)),
+		style, GOG_OBJECT (gso), 0);
 }
 
 static void
@@ -141,7 +141,7 @@ gog_label_class_init (GogLabelClass *klass)
 
 	gog_klass->editor	= gog_label_editor;
 	gog_klass->view_type	= gog_label_view_get_type ();
-	style_klass->interesting_fields = gog_label_interesting_fields;
+	style_klass->init_style = gog_label_init_style;
 }
 
 static void
@@ -173,13 +173,13 @@ gog_label_dataset_init (GogDatasetClass *iface)
 
 GSF_CLASS_FULL (GogLabel, gog_label,
 		gog_label_class_init, NULL,
-		GOG_STYLED_OBJECT_TYPE, 0,
+		GOG_OUTLINED_OBJECT_TYPE, 0,
 		GSF_INTERFACE (gog_label_dataset_init, GOG_DATASET_TYPE))
 
 /************************************************************************/
 
-typedef GogView		GogLabelView;
-typedef GogViewClass	GogLabelViewClass;
+typedef GogOutlinedView		GogLabelView;
+typedef GogOutlinedViewClass	GogLabelViewClass;
 
 #define GOG_LABEL_VIEW_TYPE	(gog_label_view_get_type ())
 #define GOG_LABEL_VIEW(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_LABEL_VIEW_TYPE, GogLabelView))
@@ -189,45 +189,17 @@ static void
 gog_label_view_size_request (GogView *v, GogViewRequisition *req)
 {
 	GogLabel *l = GOG_LABEL (v->model);
-	double outline = gog_renderer_line_size (
-		v->renderer, l->base.style->outline.width);
 
 	req->w = req->h = 0.;
 	if (l->text.data != NULL) {
 		char const *text = go_data_scalar_get_str (GO_DATA_SCALAR (l->text.data));
 		if (text != NULL) {
-			gog_renderer_push_style (v->renderer, l->base.style);
+			gog_renderer_push_style (v->renderer, l->base.base.style);
 			gog_renderer_measure_text (v->renderer, text, req);
 			gog_renderer_pop_style (v->renderer);
 		}
 	}
-	if (outline > 0) {
-		double pad_y = gog_renderer_pt2r_y (v->renderer, PAD_HACK);
-		double pad_x = gog_renderer_pt2r_y (v->renderer, PAD_HACK);
-		req->w += outline * 2 + pad_x;
-		req->h += outline * 2 + pad_y;
-	}
-}
-
-static void
-gog_label_view_size_allocate (GogView *v, GogViewAllocation const *a)
-{
-	GogLabel *l = GOG_LABEL (v->model);
-	GogViewAllocation res = *a;
-	double outline = gog_renderer_line_size (
-		v->renderer, l->base.style->outline.width);
-
-	/* We only need internal padding if there is an outline */
-	if (outline > 0) {
-		double pad_x = gog_renderer_pt2r_x (v->renderer, PAD_HACK);
-		double pad_y = gog_renderer_pt2r_y (v->renderer, PAD_HACK);
-
-		res.x += outline + pad_x/2;
-		res.y += outline + pad_y/2;
-		res.w -= outline * 2. + pad_x;
-		res.h -= outline * 2. + pad_y;
-	}
-	(lview_parent_klass->size_allocate) (v, &res);
+	lview_parent_klass->size_request (v, req);
 }
 
 static void
@@ -235,9 +207,9 @@ gog_label_view_render (GogView *view, GogViewAllocation const *bbox)
 {
 	GogLabel *l = GOG_LABEL (view->model);
 
-	gog_renderer_push_style (view->renderer, l->base.style);
-	gog_renderer_draw_rectangle (view->renderer, &view->allocation);
+	(lview_parent_klass->render) (view, bbox);
 
+	gog_renderer_push_style (view->renderer, l->base.base.style);
 	if (l->text.data != NULL) {
 		char const *text = go_data_scalar_get_str (GO_DATA_SCALAR (l->text.data));
 		if (text != NULL)
@@ -245,7 +217,6 @@ gog_label_view_render (GogView *view, GogViewAllocation const *bbox)
 				&view->residual, GTK_ANCHOR_NW, NULL);
 	}
 	gog_renderer_pop_style (view->renderer);
-	(lview_parent_klass->render) (view, bbox);
 }
 
 static void
@@ -255,10 +226,9 @@ gog_label_view_class_init (GogLabelViewClass *gview_klass)
 
 	lview_parent_klass = g_type_class_peek_parent (gview_klass);
 	view_klass->size_request    = gog_label_view_size_request;
-	view_klass->size_allocate   = gog_label_view_size_allocate;
 	view_klass->render	    = gog_label_view_render;
 }
 
 static GSF_CLASS (GogLabelView, gog_label_view,
 	   gog_label_view_class_init, NULL,
-	   GOG_VIEW_TYPE)
+	   GOG_OUTLINED_VIEW_TYPE)
