@@ -2,7 +2,7 @@
 /*
  * gog-object.c :
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,14 +19,14 @@
  * USA
  */
 
-#include <gnumeric-config.h>
+#include <goffice/goffice-config.h>
 #include <goffice/graph/gog-object.h>
 #include <goffice/graph/gog-graph-impl.h> /* for gog_graph_request_update */
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/go-data.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <src/gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -54,8 +54,7 @@ gog_object_finalize (GObject *gobj)
 	g_slist_free (obj->children);
 	obj->children = NULL;
 
-	if (parent_klass != NULL && parent_klass->finalize != NULL)
-		(parent_klass->finalize) (gobj);
+	(parent_klass->finalize) (gobj);
 }
 
 static void
@@ -668,11 +667,17 @@ gog_object_request_update (GogObject *obj)
 void
 gog_object_emit_changed (GogObject *obj, gboolean resize)
 {
+	GogObjectClass *gog_klass;
+	
 	g_return_if_fail (GOG_OBJECT (obj));
 
-	if (obj->use_parent_as_proxy) {
+	gog_klass = GOG_OBJECT_GET_CLASS (obj);
+
+	if (gog_klass->use_parent_as_proxy) {
 		obj = obj->parent;
-		g_return_if_fail (GOG_OBJECT (obj));
+		g_return_if_fail (IS_GOG_OBJECT (obj));
+		gog_object_emit_changed (obj, resize);
+		return;
 	}
 	g_signal_emit (G_OBJECT (obj),
 		gog_object_signals [CHANGED], 0, resize);
