@@ -2,7 +2,7 @@
 /*
  * gog-plot.c :
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,7 +19,7 @@
  * USA
  */
 
-#include <gnumeric-config.h>
+#include <goffice/goffice-config.h>
 #include <goffice/graph/gog-plot-impl.h>
 #include <goffice/graph/gog-plot-engine.h>
 #include <goffice/graph/gog-series-impl.h>
@@ -29,7 +29,7 @@
 #include <goffice/graph/gog-graph.h>
 #include <goffice/graph/gog-object-xml.h>
 #include <goffice/graph/go-data.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <src/mathfunc.h>
 
 #include <gsf/gsf-impl-utils.h>
@@ -52,8 +52,7 @@ gog_plot_finalize (GObject *obj)
 
 	gog_plot_axis_clear (plot, GOG_AXIS_SET_ALL); /* just in case */
 
-	if (plot_parent_klass != NULL && plot_parent_klass->finalize != NULL)
-		(plot_parent_klass->finalize) (obj);
+	(*plot_parent_klass->finalize) (obj);
 }
 
 static gboolean
@@ -195,7 +194,7 @@ gog_plot_class_init (GogObjectClass *gog_klass)
 		g_param_spec_boolean ("vary_style_by_element", "vary_style_by_element",
 			"Use a different style for each segments",
 			FALSE,
-			G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+			G_PARAM_READWRITE|GOG_PARAM_PERSISTENT|GOG_PARAM_FORCE_SAVE));
 
 	gog_klass->children_reordered = gog_plot_children_reordered;
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
@@ -347,24 +346,35 @@ gog_plot_foreach_elem (GogPlot *plot, GogEnumFunc func, gpointer data)
 	}
 }
 
+/**
+ * gog_plot_get_axis_bounds :
+ * @plot : #GogPlot
+ * @axis : #GogAxisType
+ * @bounds : #GogPlotBoundInfo
+ *
+ * Queries @plot for its axis preferences for @axis and stores the results in
+ * @bounds.  All elements of @bounds are initialized to sane values before the
+ * query _ACCEPT_ ::fmt.  The caller is responsible for initializing it.  This
+ * is done so that once on plot has selected a format the others need not do
+ * the lookup too if so desired.
+ *
+ * Caller is responsible for unrefing ::fmt.
+ **/
 GOData *
 gog_plot_get_axis_bounds (GogPlot *plot, GogAxisType axis,
-			  double *minima, double *maxima,
-			  double *logical_minima, double *logical_maxima,
-			  gboolean *is_discrete)
+			  GogPlotBoundInfo *bounds)
 {
 	GogPlotClass *klass = GOG_PLOT_GET_CLASS (plot);
 
 	g_return_val_if_fail (klass != NULL, NULL);
+	g_return_val_if_fail (bounds != NULL, NULL);
 
-	*logical_maxima = gnm_nan;
-	*logical_minima = gnm_nan;
-	*minima =  DBL_MAX;
-	*maxima = -DBL_MAX;
-	*is_discrete = FALSE;
-	return (klass->axis_bounds) (plot, axis, minima, maxima,
-				     logical_minima, logical_maxima,
-				     is_discrete);
+	bounds->val.minima =  DBL_MAX;
+	bounds->val.maxima = -DBL_MAX;
+	bounds->logical.maxima = gnm_nan;
+	bounds->logical.minima = gnm_nan;
+	bounds->is_discrete = FALSE;
+	return (klass->axis_get_bounds) (plot, axis, bounds);
 }
 
 gboolean
