@@ -2,7 +2,7 @@
 /*
  * gog-renderer-gnome-print.c :
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,9 +19,9 @@
  * USA
  */
 
-#include <gnumeric-config.h>
+#include <goffice/goffice-config.h>
 /* <style.h> is only needed for gnm_font_find_closest_from_weight_slant */
-#include <style.h>  
+#include <style.h>
 #include <goffice/graph/gog-renderer-gnome-print.h>
 #include <goffice/graph/gog-renderer-impl.h>
 #include <goffice/graph/gog-style.h>
@@ -81,8 +81,33 @@ gog_renderer_gnome_print_finalize (GObject *obj)
 		prend->fonts = NULL;
 	}
 
-	if (parent_klass != NULL && parent_klass->finalize != NULL)
-		(parent_klass->finalize) (obj);
+	(*parent_klass->finalize) (obj);
+}
+
+/*
+ * print_make_rectangle_path
+ * @pc      print context
+ * @left    left side x coordinate
+ * @bottom  bottom side y coordinate
+ * @right   right side x coordinate
+ * @top     top side y coordinate
+ *
+ * Make a rectangular path.
+ * */
+
+static void
+print_make_rectangle_path (GnomePrintContext *pc,
+                           double left, double bottom,
+                           double right, double top)
+{
+        g_return_if_fail (pc != NULL);
+
+        gnome_print_newpath   (pc);
+        gnome_print_moveto    (pc, left, bottom);
+        gnome_print_lineto    (pc, left, top);
+        gnome_print_lineto    (pc, right, top);
+        gnome_print_lineto    (pc, right, bottom);
+        gnome_print_closepath (pc);
 }
 
 static GnomeFont *
@@ -119,6 +144,27 @@ set_color (GogRendererGnomePrint *prend, GOColor color)
 	gnome_print_setopacity (prend->gp_context, a);
 }
 
+static void
+gog_renderer_gnome_print_start_clipping (GogRenderer *rend)
+{
+	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
+
+	gnome_print_gsave (prend->gp_context);
+	print_make_rectangle_path (prend->gp_context,
+				   rend->clip_rectangle.x, -rend->clip_rectangle.y,
+				   rend->clip_rectangle.w + rend->clip_rectangle.x,
+				   -rend->clip_rectangle.h - rend->clip_rectangle.y);
+	gnome_print_clip (prend->gp_context);
+}
+
+static void
+gog_renderer_gnome_print_stop_clipping (GogRenderer *rend)
+{
+	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (rend);
+
+	gnome_print_grestore (prend->gp_context);
+}
+
 static void
 draw_path (GogRendererGnomePrint *prend, ArtVpath const *path)
 {
@@ -139,16 +185,32 @@ draw_path (GogRendererGnomePrint *prend, ArtVpath const *path)
 }
 
 static void
-gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path)
+setup_clip (GogRendererGnomePrint *prend, GogViewAllocation const *bound)
+{
+	gnome_print_gsave (prend->gp_context);
+	gnome_print_moveto (prend->gp_context, bound->x, -bound->y);
+	gnome_print_lineto (prend->gp_context, bound->x + bound->w, -bound->y);
+	gnome_print_lineto (prend->gp_context, bound->x + bound->w, -bound->y - bound->h);
+	gnome_print_lineto (prend->gp_context, bound->x, -bound->y - bound->h);
+	gnome_print_clip (prend->gp_context);
+}
+
+static void
+gog_renderer_gnome_print_draw_path (GogRenderer *renderer, ArtVpath const *path,
+				    GogViewAllocation const *bound)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
 	GogStyle const *style = renderer->cur_style;
 
 	set_color (prend, style->line.color);
-	gnome_print_setlinewidth (prend->gp_context, 
+	gnome_print_setlinewidth (prend->gp_context,
 		gog_renderer_line_size (renderer, style->line.width));
+	if (bound != NULL)
+		setup_clip (prend, bound);
 	draw_path (prend, path);
 	gnome_print_stroke (prend->gp_context);
+	if (bound != NULL)
+		gnome_print_grestore (prend->gp_context);
 }
 
 static void
@@ -166,7 +228,8 @@ print_image (GogRendererGnomePrint *prend, GdkPixbuf *image, int w, int h)
 
 #define PIXBUF_SIZE 1024
 static void
-gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *path, gboolean narrow)
+gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *path,
+				       gboolean narrow, GogViewAllocation const *bound)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
 	GogStyle const *style = renderer->cur_style;
@@ -179,6 +242,9 @@ gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *pa
 	ArtGradientLinear gradient;
 	ArtGradientStop stops[2];
 
+	if (bound != NULL)
+		setup_clip (prend, bound);
+
 	if (style->fill.type != GOG_FILL_STYLE_NONE || with_outline) {
 		draw_path (prend, path);
 		gnome_print_closepath (prend->gp_context);
@@ -222,7 +288,7 @@ gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *pa
 			gnome_print_gsave (prend->gp_context);
 			gnome_print_clip (prend->gp_context);
 			render = art_render_new (0, 0, PIXBUF_SIZE, PIXBUF_SIZE,
-				gdk_pixbuf_get_pixels (image), 
+				gdk_pixbuf_get_pixels (image),
 				gdk_pixbuf_get_rowstride (image),
 				gdk_pixbuf_get_n_channels (image) - 1,
 				8, ART_ALPHA_SEPARATE, NULL);
@@ -253,6 +319,18 @@ gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *pa
 			gnome_print_gsave (prend->gp_context);
 			gnome_print_clip (prend->gp_context);
 			switch (style->fill.u.image.type) {
+			case GOG_IMAGE_CENTERED:
+				w = (bbox.x1 - bbox.x0) - gdk_pixbuf_get_width (image);
+				if (w > 0) w /= 2.; else w = 0.;
+				h = (bbox.y1 - bbox.y0) - gdk_pixbuf_get_height (image);
+				if (h > 0) h /= 2.; else h = 0.;
+
+				gnome_print_translate (prend->gp_context,
+					bbox.x0 + w, - bbox.y1 - h);
+				print_image (prend, image,
+					gdk_pixbuf_get_width (image),
+					gdk_pixbuf_get_height (image));
+				break;
 			case GOG_IMAGE_STRETCHED:
 				gnome_print_translate (prend->gp_context, bbox.x0, - bbox.y1);
 				gnome_print_scale (prend->gp_context, bbox.x1 - bbox.x0, bbox.y1 - bbox.y0);
@@ -312,10 +390,12 @@ gog_renderer_gnome_print_draw_polygon (GogRenderer *renderer, ArtVpath const *pa
 
 	if (with_outline) {
 		set_color (prend, style->outline.color);
-		gnome_print_setlinewidth (prend->gp_context, 
+		gnome_print_setlinewidth (prend->gp_context,
 			gog_renderer_line_size (renderer, style->outline.width));
 		gnome_print_stroke (prend->gp_context);
 	}
+	if (bound != NULL)
+		gnome_print_grestore (prend->gp_context);
 }
 
 static void
@@ -385,7 +465,7 @@ static void
 gog_renderer_gnome_print_draw_marker (GogRenderer *renderer, double x, double y)
 {
 	GogRendererGnomePrint *prend = GOG_RENDERER_GNOME_PRINT (renderer);
-	GOMarker *marker = renderer->cur_style->marker;
+	GOMarker *marker = renderer->cur_style->marker.mark;
 	ArtVpath const *outline_path_raw, *fill_path_raw;
 	ArtVpath *outline_path, *fill_path;
 	double scaling[6], translation[6], affine[6];
@@ -408,7 +488,7 @@ gog_renderer_gnome_print_draw_marker (GogRenderer *renderer, double x, double y)
 
 	outline_path = art_vpath_affine_transform (outline_path_raw, affine);
 	fill_path = art_vpath_affine_transform (fill_path_raw, affine);
-	
+
 	gnome_print_setlinecap (prend->gp_context, ART_PATH_STROKE_CAP_ROUND);
 	set_color (prend, marker->fill_color);
 	draw_path (prend, fill_path);
@@ -416,13 +496,13 @@ gog_renderer_gnome_print_draw_marker (GogRenderer *renderer, double x, double y)
 	gnome_print_fill (prend->gp_context);
 
 	set_color (prend, marker->outline_color);
-	gnome_print_setlinewidth (prend->gp_context, 
-		gog_renderer_line_size (renderer, 
+	gnome_print_setlinewidth (prend->gp_context,
+		gog_renderer_line_size (renderer,
 					go_marker_get_outline_width (marker)));
 	draw_path (prend, outline_path);
 	gnome_print_stroke (prend->gp_context);
 	gnome_print_newpath (prend->gp_context);
-	
+
 	gnome_print_grestore (prend->gp_context);
 
 	g_free (outline_path);
@@ -435,12 +515,14 @@ gog_renderer_gnome_print_class_init (GogRendererClass *rend_klass)
 	GObjectClass *gobject_klass   = (GObjectClass *) rend_klass;
 
 	parent_klass = g_type_class_peek_parent (rend_klass);
-	gobject_klass->finalize	  = gog_renderer_gnome_print_finalize;
-	rend_klass->draw_path	  = gog_renderer_gnome_print_draw_path;
-	rend_klass->draw_polygon  = gog_renderer_gnome_print_draw_polygon;
-	rend_klass->draw_text	  = gog_renderer_gnome_print_draw_text;
-	rend_klass->draw_marker	  = gog_renderer_gnome_print_draw_marker;
-	rend_klass->measure_text  = gog_renderer_gnome_print_measure_text;
+	gobject_klass->finalize	  	= gog_renderer_gnome_print_finalize;
+	rend_klass->start_clipping  	= gog_renderer_gnome_print_start_clipping;
+	rend_klass->stop_clipping 	= gog_renderer_gnome_print_stop_clipping;
+	rend_klass->draw_path	  	= gog_renderer_gnome_print_draw_path;
+	rend_klass->draw_polygon  	= gog_renderer_gnome_print_draw_polygon;
+	rend_klass->draw_text	  	= gog_renderer_gnome_print_draw_text;
+	rend_klass->draw_marker	  	= gog_renderer_gnome_print_draw_marker;
+	rend_klass->measure_text  	= gog_renderer_gnome_print_measure_text;
 }
 
 static void
@@ -462,7 +544,9 @@ gog_graph_print_to_gnome_print (GogGraph *graph,
 	GogViewAllocation allocation;
 	GogRendererGnomePrint *prend =
 		g_object_new (GOG_RENDERER_GNOME_PRINT_TYPE,
-			      "model", graph, NULL);
+			      "model", graph,
+			      "zoom", 1.,
+			      NULL);
 	g_object_ref (gp_context);
 	prend->gp_context = gp_context;
 	allocation.x = 0.;
