@@ -2,7 +2,7 @@
 /*
  * gog-series-impl.h :  
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -26,14 +26,32 @@
 #include <goffice/graph/gog-styled-object.h>
 #include <goffice/graph/gog-series.h>
 #include <goffice/graph/gog-data-set.h>
+#include <goffice/graph/gog-style.h>
 #include <glib-object.h>
+#include <gtk/gtknotebook.h>
 
 G_BEGIN_DECLS
 
+typedef struct {
+	GogStyledObject base;
+
+	unsigned index;
+} GogSeriesElement;
+
+typedef struct {
+	GogStyledObjectClass base;
+	
+	/* Virtuals */
+	gpointer (*gse_editor) (GogObject *gobj, GnmCmdContext *cc);
+} GogSeriesElementClass;
+
+#define GOG_SERIES_ELEMENT_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), GOG_SERIES_ELEMENT_TYPE, GogSeriesElementClass))
+
 typedef enum {
 	GOG_SERIES_REQUIRED,  /* it must be there */
 	GOG_SERIES_SUGGESTED, /* allocator will fill it in, but use need not */
-	GOG_SERIES_OPTIONAL
+	GOG_SERIES_OPTIONAL,
+	GOG_SERIES_ERRORS
 } GogSeriesPriority;
 
 struct _GogSeriesDimDesc {
@@ -53,7 +71,7 @@ struct _GogSeriesDesc {
 struct _GogSeries {
 	GogStyledObject base;
 
-	unsigned index;
+	int index;
 	unsigned manual_index : 1;
 	unsigned is_valid     : 1;
 	unsigned needs_recalc : 1;
@@ -68,8 +86,11 @@ struct _GogSeries {
 typedef struct {
 	GogStyledObjectClass base;
 
+	GType		series_element_type;
+
 	/* Virtuals */
 	void (*dim_changed) (GogSeries *series, int dim_i);
+	void (*populate_editor) (GogSeries *series, GtkNotebook* book, GogDataAllocator *dalloc, GnmCmdContext *cc);
 } GogSeriesClass;
 
 #define GOG_SERIES_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), GOG_SERIES_TYPE, GogSeriesClass))
