@@ -2,7 +2,7 @@
 /*
  * gog-font.c :
  *
- * Copyright (C) 2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,8 +19,8 @@
  * USA
  */
 
-#include <gnumeric-config.h>
-#include <goffice/utils/go-font.h>
+#include <goffice/goffice-config.h>
+#include "go-font.h"
 
 static GHashTable   *font_hash;
 static GPtrArray    *font_array;
