@@ -18,10 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED for GtkOptionMenu"
+
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <gui-util.h>
 #include <commands.h>
@@ -34,6 +38,10 @@
 #include <number-match.h>
 
 #include <glade/glade.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <string.h>
 
 typedef struct {
 	GladeXML           *gui;
@@ -60,14 +68,14 @@ cb_autofilter_destroy (AutoFilterState *state)
 	g_free (state);
 }
 
-static Value *
+static GnmValue *
 map_op (AutoFilterState *state, GnmFilterOp *op,
 	char const *op_widget, char const *val_widget)
 {
 	int i;
 	GtkWidget *w = glade_xml_get_widget (state->gui, val_widget);
 	char const *txt = gtk_entry_get_text (GTK_ENTRY (w));
-	Value *v = NULL;
+	GnmValue *v = NULL;
 
 	*op = GNM_FILTER_UNUSED;
 	if (txt == NULL || *txt == '\0')
@@ -122,11 +130,11 @@ cb_autofilter_ok (G_GNUC_UNUSED GtkWidget *button,
 
 	if (state->is_expr) {
 		GnmFilterOp op0;
-		Value *v0 = map_op (state, &op0, "op0", "value0");
+		GnmValue *v0 = map_op (state, &op0, "op0", "value0");
 
 		if (op0 != GNM_FILTER_UNUSED) {
 			GnmFilterOp op1;
-			Value *v1 = map_op (state, &op1, "op1", "value1");
+			GnmValue *v1 = map_op (state, &op1, "op1", "value1");
 			if (op1 != GNM_FILTER_UNUSED) {
 				w = glade_xml_get_widget (state->gui, "and_button");
 				cond = gnm_filter_condition_new_double (op0, v0,
@@ -177,7 +185,7 @@ cb_top10_type_changed (GtkOptionMenu *menu,
 }
 
 static void
-init_operator (AutoFilterState *state, GnmFilterOp op, Value const *v,
+init_operator (AutoFilterState *state, GnmFilterOp op, GnmValue const *v,
 	       char const *op_widget, char const *val_widget)
 {
 	GtkWidget *w = glade_xml_get_widget (state->gui, op_widget);
@@ -202,9 +210,9 @@ init_operator (AutoFilterState *state, GnmFilterOp op, Value const *v,
 		gboolean ends   = (len > 1 && str[len-1] == '*' && str[len-2] != '~');
 
 		if (starts)
-			i += (ends ? 10 : 8);
+			i += (ends ? 10 : 6);
 		else if (ends)
-			i += 6;
+			i += 8;
 	}
 	gtk_option_menu_set_history (GTK_OPTION_MENU (w), i);
 
@@ -227,7 +235,7 @@ dialog_auto_filter (WorkbookControlGUI *wbcg,
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, DIALOG_KEY))
 		return;
-	gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
 			(is_expr ? "autofilter-expression.glade" : "autofilter-top10.glade"),
 			NULL, NULL);
 	if (gui == NULL)
@@ -287,7 +295,8 @@ dialog_auto_filter (WorkbookControlGUI *wbcg,
 	/* a candidate for merging into attach guru */
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
-		"autofilter.html");
+		is_expr ? GNUMERIC_HELP_LINK_AUTOFILTER_CUSTOM :
+		GNUMERIC_HELP_LINK_AUTOFILTER_TOP_TEN);
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify)cb_autofilter_destroy);
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
