@@ -24,8 +24,11 @@
  * remove the disabling of new/edit/remove buttons!
  */
 
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED for GtkOptionMenu"
+
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -133,7 +136,7 @@ typedef struct {
 } AutoFormatGrid;
 typedef PreviewGridClass AutoFormatGridClass;
 
-static MStyle *
+static GnmStyle *
 afg_get_cell_style (PreviewGrid *pg, int col, int row)
 {
 	/* If this happens to be NULL the default style
@@ -224,7 +227,7 @@ templates_load (AutoFormatState *state)
 		return FALSE;
 
 	state->templates = category_group_get_templates_list (
-		state->current_category_group, COMMAND_CONTEXT (state->wbcg));
+		state->current_category_group, GNM_CMD_CONTEXT (state->wbcg));
 	for (l = state->templates; l != NULL; l = l->next) {
 		FormatTemplate *ft = l->data;
 		range_init (&ft->dimension,
@@ -553,7 +556,7 @@ dialog_autoformat (WorkbookControlGUI *wbcg)
 	AutoFormatState *state;
 	int i;
 
-	gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
 		"autoformat.glade", NULL, NULL);
 	if (gui == NULL)
 		return;
