@@ -23,7 +23,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -77,7 +77,7 @@ typedef struct {
 	WorkbookControlGUI	*wbcg;
 	GList			*expr_names;
 	GnmNamedExpr		*cur_name;
-	ParsePos		 pp;
+	GnmParsePos		 pp;
 	gboolean	 	 updating;
 } NameGuruState;
 
@@ -327,8 +327,8 @@ static gboolean
 name_guru_add (NameGuruState *state)
 {
 	GnmExpr	const *expr;
-	ParsePos	pp;
-	ParseError	perr;
+	GnmParsePos	pp;
+	GnmParseError	perr;
 	char const *name;
 	gboolean	res;
 
@@ -461,7 +461,7 @@ name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 	state->wb   = wb;
 	state->sv = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sv_sheet (state->sv);
-	state->gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+	state->gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
 		"names.glade", NULL, NULL);
         if (state->gui == NULL)
                 return TRUE;
@@ -567,7 +567,7 @@ dialog_define_names (WorkbookControlGUI *wbcg)
 	g_return_if_fail (wbcg != NULL);
 
 	/* Only one guru per workbook. */
-	if (wbcg_edit_has_guru (wbcg))
+	if (wbcg_edit_get_guru (wbcg))
 		return;
 
 	/* Only pop up one copy per workbook */
