@@ -20,9 +20,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <gui-util.h>
 #include <selection.h>
@@ -36,8 +37,8 @@
 #include <command-context.h>
 
 #include <glade/glade.h>
+#include <gtk/gtkradiobutton.h>
 
-#define GLADE_FILE "delete-cells.glade"
 #define DELETE_CELL_DIALOG_KEY "delete-cells-dialog"
 
 typedef struct {
@@ -45,7 +46,7 @@ typedef struct {
 	GtkWidget          *dialog;
 	GtkWidget          *ok_button;
 	GtkWidget          *cancel_button;
-	Range const        *sel;
+	GnmRange const        *sel;
 	Sheet              *sheet;
 	GladeXML           *gui;
 } DeleteCellState;
@@ -70,7 +71,8 @@ delete_cell_destroy (GtkObject *w, DeleteCellState *state)
 }
 
 static void
-cb_delete_cell_ok_clicked (GtkWidget *button, DeleteCellState *state)
+cb_delete_cell_ok_clicked (G_GNUC_UNUSED GtkWidget *button,
+			   DeleteCellState *state)
 {
 	WorkbookControl *wbc = WORKBOOK_CONTROL (state->wbcg);
 	GtkWidget *radio_0;
@@ -82,7 +84,7 @@ cb_delete_cell_ok_clicked (GtkWidget *button, DeleteCellState *state)
 
 	i = gtk_radio_group_get_selected
 		(GTK_RADIO_BUTTON (radio_0)->group);
-	
+
 	cols = state->sel->end.col - state->sel->start.col + 1;
 	rows = state->sel->end.row - state->sel->start.row + 1;
 
@@ -112,7 +114,8 @@ cb_delete_cell_ok_clicked (GtkWidget *button, DeleteCellState *state)
 }
 
 static void
-cb_delete_cell_cancel_clicked (GtkWidget *button, DeleteCellState *state)
+cb_delete_cell_cancel_clicked (G_GNUC_UNUSED GtkWidget *button,
+			       DeleteCellState *state)
 {
 	gtk_widget_destroy (state->dialog);
 	return;
@@ -124,12 +127,13 @@ dialog_delete_cells (WorkbookControlGUI *wbcg)
 	DeleteCellState *state;
 	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
 	SheetView	*sv  = wb_control_cur_sheet_view (wbc);
-	Range const *sel;
+	GnmRange const *sel;
+	GladeXML    *gui;
 	int  cols, rows;
 
 	g_return_if_fail (wbcg != NULL);
 
-	if (!(sel = selection_first_range (sv, COMMAND_CONTEXT (wbc), _("Delete"))))
+	if (!(sel = selection_first_range (sv, GNM_CMD_CONTEXT (wbc), _("Delete"))))
 		return;
 	cols = sel->end.col - sel->start.col + 1;
 	rows = sel->end.row - sel->start.row + 1;
@@ -145,15 +149,17 @@ dialog_delete_cells (WorkbookControlGUI *wbcg)
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, DELETE_CELL_DIALOG_KEY))
 		return;
+	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+		"delete-cells.glade", NULL, NULL);
+	if (gui == NULL)
+		return;
 
 	state = g_new (DeleteCellState, 1);
 	state->wbcg  = wbcg;
 	state->sel   = sel;
+	state->gui   = gui;
 	state->sheet = sv_sheet (sv);
 
-	state->gui = gnumeric_glade_xml_new (wbcg, GLADE_FILE);
-	g_return_if_fail (state->gui != NULL);
-
 	state->dialog = glade_xml_get_widget (state->gui, "Delete_cells");
 	if (state->dialog == NULL) {
 		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
@@ -171,15 +177,21 @@ dialog_delete_cells (WorkbookControlGUI *wbcg)
 	g_signal_connect (G_OBJECT (state->cancel_button),
 		"clicked",
 		G_CALLBACK (cb_delete_cell_cancel_clicked), state);
-/* FIXME: Add correct helpfile address */
+
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "helpbutton"),
-		"delete-cells.html");
+		GNUMERIC_HELP_LINK_DELETE_CELLS);
 
 	g_signal_connect (G_OBJECT (state->dialog),
 		"destroy",
 		G_CALLBACK (delete_cell_destroy), state);
 
+	gtk_toggle_button_set_active 
+		(GTK_TOGGLE_BUTTON (glade_xml_get_widget 
+				    (state->gui, cols < rows 
+				     ? "radio_0" : "radio_1")), 
+		 TRUE);
+
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 	gnumeric_keyed_dialog (wbcg, GTK_WINDOW (state->dialog),
 			       DELETE_CELL_DIALOG_KEY);
