@@ -20,8 +20,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED for GtkOptionMenu"
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -53,6 +55,7 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkmenuitem.h>
 #include <stdio.h>
 
 /* FIXME: do not hardcode pixel counts.  */
@@ -1666,16 +1669,6 @@ do_fetch_page (PrinterSetupState *state)
 	GtkWidget *w;
 	GladeXML *gui = state->gui;
 
-	double height;
-	double width;
-
-	if (gnome_print_job_get_page_size_from_config (state->pi->print_config,
-							  &width, &height) &&
-	    height > width) {
-		state->pi->orientation = PRINT_ORIENT_VERTICAL;
-	} else
-		state->pi->orientation = PRINT_ORIENT_HORIZONTAL;
-
 	w = glade_xml_get_widget (gui, "scale-percent-radio");
 	if (GTK_TOGGLE_BUTTON (w)->active)
 		state->pi->scaling.type = PERCENTAGE;
@@ -1787,7 +1780,7 @@ dialog_printer_setup (WorkbookControlGUI *wbcg, Sheet *sheet)
 	PrinterSetupState *state;
 
 	/* Only one guru per workbook. */
-	if (wbcg_edit_has_guru (wbcg))
+	if (wbcg_edit_get_guru (wbcg))
 		return;
 
 	/* Only pop up one copy per workbook */
@@ -1798,7 +1791,6 @@ dialog_printer_setup (WorkbookControlGUI *wbcg, Sheet *sheet)
 	if (!state)
 		return;
 
-/* FIXME: Add correct helpfile address */
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
 		GNUMERIC_HELP_LINK_PRINTER_SETUP_GENERAL);
