@@ -22,10 +22,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED"
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <gui-util.h>
 #include <func.h>
@@ -38,9 +41,15 @@
 #include <workbook.h>
 #include <parse-util.h>
 #include <ranges.h>
+#include <commands.h>
 #include <widgets/gnumeric-expr-entry.h>
 
 #include <glade/glade.h>
+#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtktable.h>
 #include <string.h>
 #include <scenarios.h>
 
@@ -73,6 +82,8 @@ typedef struct {
 	GSList              *ov_cell_stack;
 	GtkWidget           *warning_dialog;
 
+	gboolean             cancelled;
+
 	Sheet	            *sheet;
 	Workbook            *wb;
 	WorkbookControlGUI  *wbcg;
@@ -86,7 +97,7 @@ typedef struct {
 } algorithm_def_t;
 
 static algorithm_def_t algorithm_defs [] = {
-	{ N_("Revised Simplex (GLPK 3.2)"), GLPKSimplex, SolverLPModel },
+	{ N_("Revised Simplex (GLPK 4.2)"), GLPKSimplex, SolverLPModel },
 	{ N_("Simplex (LP Solve 3.2)"), LPSolve, SolverLPModel },
 	{ N_("< Not available >"), QPDummy, SolverQPModel },
 	{ NULL, 0, 0 }
@@ -100,8 +111,8 @@ typedef struct {
 
 
 typedef struct {
-	Value                *lhs_value;
-	Value                *rhs_value;
+	GnmValue                *lhs_value;
+	GnmValue                *rhs_value;
 	SolverConstraintType type;
 } constraint_t;
 
@@ -132,8 +143,8 @@ static gboolean
 is_hom_row_or_col_ref (GnmExprEntry *entry_1, GnmExprEntry *entry_2,
 		       Sheet *sheet)
 {
-        Value    *input_range_1;
-        Value    *input_range_2;
+        GnmValue    *input_range_1;
+        GnmValue    *input_range_2;
 	gboolean res;
 
 	input_range_1 = gnm_expr_entry_parse_as_value (entry_1, sheet);
@@ -210,8 +221,27 @@ constraint_select_click (G_GNUC_UNUSED GtkWidget      *clist,
 			 G_GNUC_UNUSED GdkEventButton *event,
 			 SolverState    *state)
 {
+	constraint_t const *constr =
+		gtk_clist_get_row_data (state->constraint_list, row);
+	GnmRange range;
+
         state->selected_row = row;
 	dialog_set_sec_button_sensitivity (NULL, state);
+
+	if (constr == NULL)
+		return; /* Fail? */
+
+	range_init_value (&range, constr->lhs_value);
+	gnm_expr_entry_load_from_range (state->lhs_entry, state->sheet,&range);
+
+	if (constr->type != SolverINT && constr->type != SolverBOOL) {
+		range_init_value (&range, constr->rhs_value);
+		gnm_expr_entry_load_from_range (state->rhs_entry,
+						state->sheet, &range);
+	} else
+		gnm_expr_entry_load_from_text (state->rhs_entry, "");
+
+	gtk_option_menu_set_history (state->type_combo, constr->type);
 }
 
 /**
@@ -456,6 +486,15 @@ dialog_destroy (GtkObject *w, SolverState  *state)
 	g_return_val_if_fail (w != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
 
+	if (state->ov_cell_stack != NULL &&
+	    !state->cancelled &&
+	    !cmd_solver (WORKBOOK_CONTROL(state->wbcg), state->ov_cell_stack, 
+			 state->ov_stack, NULL))
+	{
+		state->ov_cell_stack = NULL;
+		state->ov_stack = NULL;
+	}
+
 	if (state->ov_stack != NULL) {
 		g_slist_foreach (state->ov_stack, (GFunc)free_original_values,
 				 NULL);
@@ -472,7 +511,7 @@ dialog_destroy (GtkObject *w, SolverState  *state)
 		state->gui = NULL;
 	}
 
-	wbcg_edit_finish (state->wbcg, FALSE);
+	wbcg_edit_finish (state->wbcg, FALSE, NULL);
 
 	state->dialog = NULL;
 
@@ -491,8 +530,8 @@ static void
 restore_original_values (GSList *input_cells, GSList *ov)
 {
         while (ov != NULL) {
-	        const char *str = ov->data;
-	        Cell *cell = input_cells->data;
+	        char const *str = ov->data;
+	        GnmCell *cell = input_cells->data;
 
 		sheet_cell_set_text (cell, str);
 		ov = ov->next;
@@ -528,6 +567,7 @@ cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button, SolverState *state)
 		state->ov_stack = NULL;
 		workbook_recalc (state->sheet->workbook);
 	}
+	state->cancelled = TRUE;
 
 	gtk_widget_destroy (state->dialog);
 }
@@ -543,6 +583,7 @@ static void
 cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
 			 SolverState *state)
 {
+	state->cancelled = FALSE;
 	gtk_widget_destroy (state->dialog);
 }
 
@@ -551,8 +592,8 @@ cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
  *  @cell:
  *  @data: pointer to a data_set_t
  */
-static Value *
-grab_cells (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
+static GnmValue *
+grab_cells (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
 {
 	GList **the_list = user_data;
 
@@ -568,7 +609,7 @@ grab_cells (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
  *  @data: pointer to a data_set_t
  */
 static gint
-check_int_constraints (Value *input_range, GtkCList *constraint_list)
+check_int_constraints (GnmValue *input_range, GtkCList *constraint_list)
 {
 	gint i;
 
@@ -654,7 +695,7 @@ revert_constraint_format (constraint_conversion_t * conv)
 	while (engine_constraint_list != NULL) {
 		const SolverConstraint *engine_constraint =
 			engine_constraint_list->data;
-		Range r;
+		GnmRange r;
 		constraint_t *a_constraint = g_new (constraint_t, 1);
 
 		r.start.col = engine_constraint->lhs.col;
@@ -695,7 +736,7 @@ save_original_values (GSList *input_cells)
         GSList *ov = NULL;
 
 	while (input_cells != NULL) {
-	        Cell *cell = input_cells->data;
+	        GnmCell *cell = input_cells->data;
 		char *str;
 
 		str = value_get_as_string (cell->value);
@@ -742,10 +783,10 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				((GtkWindow *) state->dialog,
 				 &(state->warning_dialog),
 				 GTK_MESSAGE_INFO,
-				 _("Sensitivity nor limits report are "
-				   "not meaningful if the program has "
-				   "integer constraints. These reports "
-				   "will thus not be created."));
+				 _("Neither sensitivity nor limits report are "
+				   "meaningful if the program has "
+				   "integer constraints.  These reports "
+				   "will not be created."));
 		err = solver_reports (WORKBOOK_CONTROL(state->wbcg),
 				      state->sheet, res,
 				      opt->answer_report,
@@ -828,7 +869,7 @@ static void
 solver_add_scenario (SolverState *state, SolverResults *res, gchar *name)
 {
 	SolverParameters *param = res->param;
-	Value            *input_range;
+	GnmValue            *input_range;
 	const gchar      *comment = _("Optimal solution created by solver.\n");
 	scenario_t       *scenario;
 
@@ -855,11 +896,11 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 {
 	constraint_conversion_t conv = {NULL, NULL, NULL};
 	SolverResults           *res;
-	Value                   *target_range;
-	Value                   *input_range;
+	GnmValue                   *target_range;
+	GnmValue                   *input_range;
         GSList			*input_cells = NULL;
-	Value                   *result;
-	EvalPos                 pos;
+	GnmValue                   *result;
+	GnmEvalPos                 pos;
 	gint                    i;
 	gboolean                answer, sensitivity, limits, performance;
 	gboolean                program, dual_program;
@@ -1019,8 +1060,10 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 	workbook_recalc (state->sheet->workbook);
 
 	if (res != NULL) {
+		state->cancelled = FALSE;
 		solver_reporting (state, res, errmsg);
-		if (param->options.add_scenario)
+		if (res->status == SolverOptimal && 
+		    param->options.add_scenario)
 			solver_add_scenario (state, res,
 					     param->options.scenario_name);
 
@@ -1076,7 +1119,7 @@ dialog_init (SolverState *state)
 			}
 	}
 
-	state->gui = gnm_glade_xml_new (COMMAND_CONTEXT (state->wbcg),
+	state->gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (state->wbcg),
 		"solver.glade", NULL, NULL);
         if (state->gui == NULL)
                 return TRUE;
@@ -1102,7 +1145,7 @@ dialog_init (SolverState *state)
 
 	gnumeric_init_help_button (glade_xml_get_widget (state->gui,
 							 "helpbutton"),
-				   "solver.html");
+				   GNUMERIC_HELP_LINK_SOLVER);
 
 	state->add_button  = glade_xml_get_widget (state->gui, "addbutton");
 	g_signal_connect (G_OBJECT (state->add_button), "clicked",
@@ -1346,6 +1389,7 @@ dialog_solver (WorkbookControlGUI *wbcg, Sheet *sheet)
 	state->ov_stack       = NULL;
 	state->ov_cell_stack  = NULL;
 	state->warning_dialog = NULL;
+	state->cancelled      = TRUE;
 
 	if (dialog_init (state)) {
 		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
