@@ -7,7 +7,7 @@
  *   Zbigniew Chyla (cyba@gnome.pl)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "file.h"
 
@@ -24,7 +24,12 @@
 
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-output.h>
+#include <gsf/gsf-output-stdio.h>
 #include <gsf/gsf-impl-utils.h>
+#include <gsf/gsf-input-memory.h>
+#include <gsf/gsf-input-stdio.h>
+#include <gsf/gsf-output-stdio.h>
+#include <gsf/gsf-utils.h>
 #include <string.h>
 
 static void
@@ -50,6 +55,12 @@ gnm_file_opener_finalize (GObject *obj)
 	G_OBJECT_CLASS (g_type_class_peek (G_TYPE_OBJECT))->finalize (obj);
 }
 
+static gboolean
+gnm_file_opener_can_probe_real (GnmFileOpener const *fo, FileProbeLevel pl)
+{
+	return fo->probe_func != NULL;
+}
+
 static gboolean
 gnm_file_opener_probe_real (GnmFileOpener const *fo, GsfInput *input,
                              FileProbeLevel pl)
@@ -83,6 +94,7 @@ gnm_file_opener_class_init (GnmFileOpenerClass *klass)
 {
 	G_OBJECT_CLASS (klass)->finalize = gnm_file_opener_finalize;
 
+	klass->can_probe = gnm_file_opener_can_probe_real;
 	klass->probe = gnm_file_opener_probe_real;
 	klass->open = gnm_file_opener_open_real;
 }
@@ -201,6 +213,13 @@ gnm_file_opener_is_encoding_dependent (GnmFileOpener const *fo)
 	return fo->encoding_dependent;
 }
 
+gboolean
+gnm_file_opener_can_probe (GnmFileOpener const *fo, FileProbeLevel pl)
+{
+	g_return_val_if_fail (IS_GNM_FILE_OPENER (fo), FALSE);
+
+	return GNM_FILE_OPENER_METHOD (fo, can_probe) (fo, pl);
+}
 
 
 
@@ -295,92 +314,6 @@ gnm_file_saver_save_real (GnmFileSaver const *fs, IOContext *io_context,
 	fs->save_func (fs, io_context, wbv, output);
 }
 
-#if defined(GNOME2_CONVERSION_COMPLETE) && defined(WITH_BONOBO)
-#include <bonobo.h>
-#include <bonobo/bonobo-stream.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#define FILE_COPY_CHUNK_SIZE  0x10000
-
-static void
-gnm_file_saver_save_to_stream_real (GnmFileSaver const *fs,
-				    IOContext *io_context,
-				    WorkbookView *wbv,
-				    BonoboStream *stream,
-				    CORBA_Environment *ev)
-{
-	gchar *tmp_name;
-	gint old_umask;
-	gint fd;
-
-	tmp_name = g_build_filename (g_get_tmp_dir (), "gnumeric-file-XXXXXX", NULL);
-	old_umask = umask (0077);
-	fd = mkstemp (tmp_name);
-	if (fd == -1) {
-		gnumeric_io_error_info_set (io_context, error_info_new_str_with_details (
-		                            _("Cannot create temporary file."),
-		                            error_info_new_from_errno ()));
-	} else {
-		struct stat sbuf;
-
-		io_progress_range_push (io_context, 0.0, 0.5);
-		gnm_file_saver_save (fs, io_context, wbv, tmp_name);
-		io_progress_range_pop (io_context);
-		(void) unlink (tmp_name);
-
-		if (gnumeric_io_error_occurred (io_context)) {
-			gnumeric_io_error_push (io_context, error_info_new_str (
-			                        _("Error saving to temporary file.")));
-		} else if (fstat (fd, &sbuf) == -1) {
-			gnumeric_io_error_info_set (io_context, error_info_new_str_with_details (
-			                            _("Cannot get temporary file size."),
-			                            error_info_new_from_errno ()));
-		} else {
-			Bonobo_Stream_iobuf *iobuf;
-			gint offset;
-
-			io_progress_message (io_context, _("Saving file to stream."));
-			io_progress_range_push (io_context, 0.5, 1.0);
-			iobuf = Bonobo_Stream_iobuf__alloc ();
-			value_io_progress_set (io_context, sbuf.st_size, FILE_COPY_CHUNK_SIZE);
-			offset = 0;
-			while (1) {
-				guchar buf[FILE_COPY_CHUNK_SIZE];
-				gint read_bytes;
-
-				value_io_progress_update (io_context, offset);
-				read_bytes = read (fd, buf, FILE_COPY_CHUNK_SIZE);
-				if (read_bytes == -1 && errno != EINTR) {
-					gnumeric_io_error_info_set (io_context, error_info_new_str_with_details (
-					                            _("Error reading temporary file."),
-					                            error_info_new_from_errno ()));
-					break;
-				} else if (read_bytes == -1) {
-					continue;
-				} else if (read_bytes == 0) {
-					break;
-				}
-				iobuf->_buffer = buf;
-				iobuf->_length = read_bytes;
-				Bonobo_Stream_write (BONOBO_OBJREF (stream), iobuf, ev);
-				if (BONOBO_EX (ev)) {
-					gnumeric_io_error_string (
-					io_context, _("Exception occurred while saving to stream."));
-					break;
-				}
-				offset += read_bytes;
-			}
-			io_progress_unset (io_context);
-			CORBA_free (iobuf);
-			io_progress_range_pop (io_context);
-		}
-		(void) close (fd);
-	}
-	g_free (tmp_name);
-	(void) umask (old_umask);
-}
-#endif
-
 static void
 gnm_file_saver_class_init (GnmFileSaverClass *klass)
 {
@@ -470,7 +403,7 @@ gnm_file_saver_set_save_scope (GnmFileSaver *fs, FileSaveScope scope)
 }
 
 FileSaveScope
-gnm_file_saver_get_save_scope (GnmFileSaver *fs)
+gnm_file_saver_get_save_scope (GnmFileSaver const *fs)
 {
 	g_return_val_if_fail (IS_GNM_FILE_SAVER (fs), FILE_SAVE_WORKBOOK);
 
@@ -539,18 +472,21 @@ gnm_file_saver_save (GnmFileSaver const *fs, IOContext *io_context,
 	g_return_if_fail (IS_GNM_FILE_SAVER (fs));
 	g_return_if_fail (GSF_IS_OUTPUT (output));
 
-	file_name = (char *) gsf_output_name (output);
-	g_return_if_fail (file_name != NULL);
+	if (GSF_IS_OUTPUT_STDIO (output)) {
+		file_name = (char *) gsf_output_name (output);
+		g_return_if_fail (file_name != NULL);
+		
+		if (!fs->overwrite_files &&
+		    g_file_test ((file_name), G_FILE_TEST_EXISTS)) {
+			ErrorInfo *save_error;
 
-	if (!fs->overwrite_files && g_file_test ((file_name), G_FILE_TEST_EXISTS)) {
-		ErrorInfo *save_error;
-
-		save_error = error_info_new_str_with_details (
-		_("Saving over old files of this type is disabled for safety."),
-		error_info_new_str (
-		_("You can turn this safety feature off by editing appropriate plugin.xml file.")));
-		gnumeric_io_error_info_set (io_context, save_error);
-		return;
+			save_error = error_info_new_str_with_details (
+				_("Saving over old files of this type is disabled for safety."),
+				error_info_new_str (
+					_("You can turn this safety feature off by editing appropriate plugin.xml file.")));
+			gnumeric_io_error_info_set (io_context, save_error);
+			return;
+		}
 	}
 
 	GNM_FILE_SAVER_METHOD (fs, save) (fs, io_context, wbv, output);
@@ -865,6 +801,27 @@ gnm_file_saver_for_mime_type (gchar const *mime_type)
 	return (NULL);
 }
 
+/**
+ * gnm_file_saver_for_file_name :
+ * @file_name :
+ *
+ * Searches for file opener with given @filename, registered using
+ * gnm_file_opener_register
+ *
+ * Return value: GnmFileOpener object or NULL if opener cannot be found.
+ **/
+GnmFileSaver *
+gnm_file_saver_for_file_name (char const *file_name)
+{
+	GList *l;
+	char const *extension = gsf_extension_pointer (file_name);
+
+	for (l = file_saver_list; l != NULL; l = l->next)
+		if (!strcmp (gnm_file_saver_get_extension (l->data), extension))
+			return l->data;
+	return NULL;
+}
+
 /**
  * gnm_file_opener_for_id:
  * @id          : File opener's ID
@@ -929,3 +886,32 @@ get_file_openers (void)
 {
 	return file_opener_list;
 }
+
+/**
+ * go_file_open :
+ * @path :
+ * @err : #GError
+ *
+ * Try all available methods to open a file or return an error
+ **/
+#warning this belongs in gsf
+GsfInput *
+go_file_open (char const *path, GError **err)
+{
+	GsfInputMemory *in_mem;
+	GsfInputStdio  *in_stdio;
+
+	if (err != NULL)
+		*err = NULL;
+
+	in_mem = gsf_input_mmap_new (path, NULL);
+	if (in_mem != NULL)
+		return GSF_INPUT (in_mem);
+
+	/* Only report error if stdio fails too */
+	in_stdio = gsf_input_stdio_new (path, err);
+	if (in_stdio != NULL)
+		return GSF_INPUT (in_stdio);
+
+	return NULL; 
+}
