@@ -22,7 +22,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "format.h"
 
@@ -32,6 +32,7 @@
 #include "datetime.h"
 #include "mathfunc.h"
 #include "str.h"
+#include "gutils.h"
 #include "number-match.h"
 
 #include <locale.h>
@@ -43,6 +44,8 @@
 #    include <langinfo.h>
 #endif
 
+#undef DEBUG_REF_COUNT
+
 /***************************************************************************/
 
 static GnmFormat *default_percentage_fmt;
@@ -52,13 +55,16 @@ static GnmFormat *default_time_fmt;
 static GnmFormat *default_general_fmt;
 
 
-/* Points to the locale information for number display */
+/*
+ * Points to the locale information for number display.  All strings are
+ * in UTF-8 encoding.
+ */
 static gboolean locale_info_cached = FALSE;
-static char *lc_decimal = NULL;
-static char *lc_thousand = NULL;
+static GString *lc_decimal = NULL;
+static GString *lc_thousand = NULL;
 static gboolean lc_precedes;
 static gboolean lc_space_sep;
-static char *lc_currency = NULL; /* in UTF-8 */
+static GString *lc_currency = NULL;
 
 static gboolean date_order_cached = FALSE;
 
@@ -75,20 +81,25 @@ gnm_setlocale (int category, char const *val)
 	return setlocale (category, val);
 }
 
-static char *
-convert1 (const char *lstr, const char *name, const char *def)
+static void
+convert1 (GString *res, const char *lstr, const char *name, const char *def)
 {
-	char *res;
+	char *tmp;
 
-	if (lstr == NULL || lstr[0] == 0)
-		return g_strdup (def);
+	if (lstr == NULL || lstr[0] == 0) {
+		g_string_assign (res, def);
+		return;
+	}
 
-	res = g_locale_to_utf8 (lstr, -1, NULL, NULL, NULL);
-	if (res)
-		return res;
+	tmp = g_locale_to_utf8 (lstr, -1, NULL, NULL, NULL);
+	if (tmp) {
+		g_string_assign (res, tmp);
+		g_free (tmp);
+		return;
+	}
 
 	g_warning ("Failed to convert locale's %s \"%s\" to UTF-8.", name, lstr);
-	return g_strdup (def);
+	g_string_assign (res, def);
 }
 
 static void
@@ -101,30 +112,34 @@ update_lc (void)
 	 * valid after next localeconv call which could be anywhere.
 	 */
 
-	g_free (lc_decimal);
-	lc_decimal = convert1 (lc->decimal_point, "decimal separator", ".");
-	if (g_utf8_strlen (lc_decimal, -1) != 1)
+	convert1 (lc_decimal, lc->decimal_point, "decimal separator", ".");
+	if (g_utf8_strlen (lc_decimal->str, -1) != 1)
 		g_warning ("Decimal separator is not a single character.");
 
-	g_free (lc_thousand);
-	lc_thousand = convert1 (lc->mon_thousands_sep, "monetary thousands separator",
-				(lc_decimal[0] == ',' ? "." : ","));
-	if (g_utf8_strlen (lc_thousand, -1) != 1)
+	convert1 (lc_thousand, lc->mon_thousands_sep, "monetary thousands separator",
+		  (lc_decimal->str[0] == ',' ? "." : ","));
+	if (g_utf8_strlen (lc_thousand->str, -1) != 1)
 		g_warning ("Monetary thousands separator is not a single character.");
 
+	if (g_string_equal (lc_thousand, lc_decimal)) {
+		g_string_assign (lc_thousand,
+				 (lc_decimal->str[0] == ',') ? "." : ",");
+		g_warning ("Monetary thousands separator is the same as the decimal seperator; converting '%s' to '%s'",
+			   lc_decimal->str, lc_thousand->str);
+	}
+
 	/* Use != 0 rather than == 1 so that CHAR_MAX (undefined) is true */
 	lc_precedes = (lc->p_cs_precedes != 0);
 
 	/* Use == 1 rather than != 0 so that CHAR_MAX (undefined) is false */
 	lc_space_sep = (lc->p_sep_by_space == 1);
 
-	g_free (lc_currency);
-	lc_currency = convert1 (lc->currency_symbol, "currency symbol",	"$");
+	convert1 (lc_currency, lc->currency_symbol, "currency symbol",	"$");
 
 	locale_info_cached = TRUE;
 }
 
-char const *
+GString const *
 format_get_decimal (void)
 {
 	if (!locale_info_cached)
@@ -133,7 +148,7 @@ format_get_decimal (void)
 	return lc_decimal;
 }
 
-char const *
+GString const *
 format_get_thousand (void)
 {
 	if (!locale_info_cached)
@@ -152,7 +167,7 @@ format_get_thousand (void)
  * Play with the default logic so that things come out nicely for the default
  * case.
  */
-char const *
+GString const *
 format_get_currency (gboolean *precedes, gboolean *space_sep)
 {
 	if (!locale_info_cached)
@@ -212,7 +227,7 @@ format_month_before_day (void)
 char
 format_get_arg_sep (void)
 {
-	if (*format_get_decimal () == ',')
+	if (format_get_decimal ()->str[0] == ',')
 		return ';';
 	return ',';
 }
@@ -220,7 +235,7 @@ format_get_arg_sep (void)
 char
 format_get_col_sep (void)
 {
-	if (*format_get_decimal () == ',')
+	if (format_get_decimal ()->str[0] == ',')
 		return '\\';
 	return ',';
 }
@@ -260,8 +275,10 @@ typedef struct {
 	gboolean    want_am_pm;
 	gboolean    has_fraction;
         char        restriction_type;
-        gnm_float  restriction_value;
-	GnmColor *color;
+	gboolean    suppress_minus;
+	gboolean    elapsed_time;
+        gnm_float   restriction_value;
+	GnmColor   *color;
 } StyleFormatEntry;
 
 /*
@@ -326,11 +343,15 @@ append_month (GString *string, int n, struct tm const *time_split)
 		return 2;
 	}
 
-	if (n == 3){
+	if (n == 3) {
 		g_string_append (string, _(month_short[month - 1]) + 1);
 		return 3;
 	}
 
+	if (n == 5) {
+		g_string_append_c (string, _(month_short[month - 1])[1]);
+		return 5;
+	}
 	g_string_append (string, _(month_long[month - 1]));
 	return 4;
 }
@@ -383,7 +404,7 @@ append_hour_elapsed (GString *string, struct tm *tm, gnm_float number)
 
 	/* ick.  round assuming no more than 100th of a second, we really need
 	 * to know the precision earlier */
-	res = gnumeric_add_epsilon (number) * 24. * 60. * 60. * 100. + .5;
+	res = gnumeric_fake_round (number * (24 * 60 * 60 * 100));
 	res = modf (res / (60. * 60. * 100.), &int_part);
 	tm->tm_hour = int_part;
 	is_neg = (res < 0);
@@ -391,8 +412,6 @@ append_hour_elapsed (GString *string, struct tm *tm, gnm_float number)
 	tm->tm_min = res / 60.;
 	tm->tm_sec = res - tm->tm_min * 60;
 
-	if (tm->tm_hour == 0 && is_neg)
-		g_string_append_c (string, '-');
 	g_string_append_printf (string, "%d", tm->tm_hour);
 }
 
@@ -440,6 +459,8 @@ format_entry_ctor (void)
 	entry = g_new (StyleFormatEntry, 1);
 	entry->restriction_type = '*';
 	entry->restriction_value = 0.;
+	entry->suppress_minus = FALSE;
+	entry->elapsed_time = FALSE;
 	entry->want_am_pm = entry->has_fraction = FALSE;
 	entry->color = NULL;
 	return entry;
@@ -529,17 +550,20 @@ format_compile (GnmFormat *format)
 				}
 			} else if (*begin == '=') {
 				entry->restriction_type = '=';
-			} else if (*begin != '$' && entry->color == NULL) {
-				entry->color = lookup_color (begin, end);
-				/* Only the first colour counts */
-				if (NULL == entry->color) {
-					if (NULL == real_start)
-						real_start = fmt;
-					continue;
-				}
-				fmt = end;
-				continue;
 			} else {
+				if (begin[1] == ']' &&
+				    (*begin == 'h' || *begin == 'H' ||
+				     *begin == 'm' || *begin == 'M' ||
+				     *begin == 's' || *begin == 'S'))
+					entry->elapsed_time = TRUE;
+				else if (*begin != '$' && entry->color == NULL) {
+					entry->color = lookup_color (begin, end);
+					/* Only the first colour counts */
+					if (NULL != entry->color) {
+						fmt = end;
+						continue;
+					}
+				}
 				if (NULL == real_start)
 					real_start = fmt;
 				continue;
@@ -551,6 +575,17 @@ format_compile (GnmFormat *format)
 			entry->restriction_value = strtognum (begin, (char **)&end);
 			if (errno == ERANGE || begin == end)
 				entry->restriction_value = 0.;
+
+			/* this is a guess based on checking the results of
+			 * 0.00;[<0]0.00
+			 * 0.00;[<=0]0.00
+			 *
+			 * for -1.2.3
+			 **/
+			else if (entry->restriction_type == '<')
+				entry->suppress_minus = (entry->restriction_value <= 0.);
+			else if (entry->restriction_type == ',')
+				entry->suppress_minus = (entry->restriction_value < 0.);
 			break;
 		}
 
@@ -582,6 +617,15 @@ format_compile (GnmFormat *format)
 				entry->want_am_pm = TRUE;
 			break;
 
+		case 'M': case 'm':
+		case 'D': case 'd':
+		case 'Y': case 'y':
+		case 'S': case 's':
+		case 'H': case 'h':
+			if (!entry->suppress_minus && !entry->elapsed_time)
+				entry->suppress_minus = TRUE;
+			break;
+
 		case ';':
 			format_entry_set_fmt (entry, real_start, fmt);
 			format->entries = g_slist_append (format->entries, entry);
@@ -710,7 +754,7 @@ render_number (GString *result,
 	       gnm_float number,
 	       format_info_t const *info)
 {
-	const char *thousands_sep = format_get_thousand ();
+	const GString *thousands_sep = format_get_thousand ();
 	char num_buf[(GNUM_MANT_DIG + GNUM_MAX_EXP) * 2 + 1];
 	gchar *num = num_buf + sizeof (num_buf) - 1;
 	gnm_float frac_part, int_part;
@@ -736,8 +780,8 @@ render_number (GString *result,
 		if (group-- == 0) {
 			int i;
 			group = 2;
-			for (i = strlen (thousands_sep) - 1; i >= 0; i--)
-				*(--num) = thousands_sep[i];
+			for (i = thousands_sep->len - 1; i >= 0; i--)
+				*(--num) = thousands_sep->str[i];
 		}
 		*(--num) = '0';
 		sigdig++;
@@ -750,16 +794,13 @@ render_number (GString *result,
 		if (group-- == 0) {
 			int i;
 			group = 2;
-			for (i = strlen (thousands_sep) - 1; i >= 0; i--)
-				*(--num) = thousands_sep[i];
+			for (i = thousands_sep->len - 1; i >= 0; i--)
+				*(--num) = thousands_sep->str[i];
 		}
 		*(--num) = digit + '0';
 		sigdig++;
 	}
 
-	/* TODO : What ifthe only visible digits are zeros ? -0.00 looks bad */
-	if (info->negative && !info->supress_minus)
-		g_string_append_c (result, '-');
 	if (left_req > digit_count) {
 		for (left_spaces -= left_req ; left_spaces-- > 0 ;)
 			g_string_append_c (result, ' ');
@@ -778,7 +819,7 @@ render_number (GString *result,
 	     number < 1.0 &&
 	     info->right_allowed == 0 &&
 	     info->right_optional > 0))
-		g_string_append (result, format_get_decimal ());
+		gnm_string_append_gstring (result, format_get_decimal ());
 
 	/* TODO : clip this a DBL_DIG */
 	/* TODO : What if is a fraction ? */
@@ -829,7 +870,6 @@ do_render_number (gnm_float number, format_info_t *info, GString *result)
 		"left_spaces: %d\n"
 		"right_spaces:%d\n"
 		"right_allow: %d\n"
-		"negative:    %d\n"
 		"supress:     %d\n"
 		"decimalseen: %d\n"
 		"decimalp:    %s\n",
@@ -838,8 +878,6 @@ do_render_number (gnm_float number, format_info_t *info, GString *result)
 		info->left_spaces,
 		info->right_spaces,
 		info->right_allowed + info->right_optional,
-		info->negative,
-		info->supress_minus,
 		info->decimal_separator_seen,
 		decimal_point);
 #endif
@@ -1031,6 +1069,7 @@ style_format_account (FormatCharacteristics const *fmt)
 	GString *str, *sym, *num;
 	GnmFormat *sf;
 	int symbol = fmt->currency_symbol_index;
+	gboolean quote_currency;
 
 	g_return_val_if_fail (fmt->num_decimals >= 0, NULL);
 	g_return_val_if_fail (fmt->num_decimals <= NUM_ZEROS, NULL);
@@ -1045,8 +1084,13 @@ style_format_account (FormatCharacteristics const *fmt)
 
 	/* The currency symbols with space after or before */
 	sym = g_string_new (NULL);
+	quote_currency = (currency_symbols[symbol].symbol[0] != '[');
 	if (currency_symbols[symbol].precedes) {
+		if (quote_currency)
+			g_string_append_c (sym, '\"');
 		g_string_append (sym, currency_symbols[symbol].symbol);
+		if (quote_currency)
+			g_string_append_c (sym, '\"');
 		g_string_append (sym, "* ");
 		if (currency_symbols[symbol].has_space)
 			g_string_append_c (sym, ' ');
@@ -1054,7 +1098,11 @@ style_format_account (FormatCharacteristics const *fmt)
 		g_string_append (sym, "* ");
 		if (currency_symbols[symbol].has_space)
 			g_string_append_c (sym, ' ');
+		if (quote_currency)
+			g_string_append_c (sym, '\"');
 		g_string_append (sym, currency_symbols[symbol].symbol);
+		if (quote_currency)
+			g_string_append_c (sym, '\"');
 	}
 
 	/* Finally build the correct string */
@@ -1401,7 +1449,6 @@ format_number (GString *result,
 	gboolean hour_seen = FALSE;
 	gboolean time_display_elapsed = FALSE;
 	gboolean ignore_further_elapsed = FALSE;
-	size_t prelen = result->len;
 
 	gunichar fill_char = 0;
 	int fill_start = -1;
@@ -1412,9 +1459,10 @@ format_number (GString *result,
 
 	memset (&info, 0, sizeof (info));
 	signed_number = number;
-	if (number < 0.0){
-		info.negative = TRUE;
+	if (number < 0.) {
 		number = -number;
+		if (!entry->suppress_minus)
+			g_string_append_c (result, '-');
 	}
 	info.has_fraction = entry->has_fraction;
 	info.scale = 1;
@@ -1490,38 +1538,38 @@ format_number (GString *result,
 				format = tmp;
 				continue;
 			} else
-				g_string_append (result, format_get_thousand ());
+				gnm_string_append_gstring (result, format_get_thousand ());
 			break;
 
 		/* FIXME: this is a gross hack */
+		/* FIXME: Missing support for scientific notation.
+		 * #00.00e###  that keeps axponent to a multiple of 3
+		 * XL seems to just special case that.
+		 **/
 		case 'E': case 'e': {
-			gboolean const is_lower = (*format == 'e');
+			gboolean const is_lower = (*format++ == 'e');
 			gboolean shows_plus = FALSE;
 			int prec = info.right_optional + info.right_req;
 
 			can_render_number = TRUE;
-			while (*(++format))
-				if (*format == '+')
-					shows_plus = TRUE;
-				else if (*format == '-' || *format == '0')
-					;
-				else
-					break;
+			if (*format == '+') {
+				shows_plus = TRUE;
+				format++;
+			} else if (*format == '-')
+				format++;
+
+			while (*format == '0' || *format == '#')
+				format++;
 
 			g_string_append_printf (result,
-						is_lower ? "%s%.*" GNUM_FORMAT_e : "%s%.*" GNUM_FORMAT_E,
-						info.negative ? "-" :
-						shows_plus ? "+" : "",
+						is_lower ? "%.*" GNUM_FORMAT_e : "%.*" GNUM_FORMAT_E,
 						prec, number);
 			return;
 		}
 
 		case '\\':
 			if (format[1] != '\0') {
-				/* TODO : Other chars here ?? ('+' or ':') ? */
-				if (format[1] == '-' || format[1] == '(')
-					info.supress_minus = TRUE;
-				else if (can_render_number && !info.rendered)
+				if (can_render_number && !info.rendered)
 					do_render_number (number, &info, result);
 
 				format++;
@@ -1581,8 +1629,6 @@ format_number (GString *result,
 					if (!info.rendered) {
 						info.rendered = TRUE;
 						numerator += ((int)number) * denominator;
-						if (info.negative && !info.supress_minus)
-							g_string_insert_c (result, prelen, '-');
 					}
 
 					/*
@@ -1608,9 +1654,6 @@ format_number (GString *result,
 		case '(':
 		case '+':
 		case ':':
-			info.supress_minus = TRUE;
-			/* fall down */
-
 		case ' ': /* eg # ?/? */
 		case '$':
 		case 0x00A3 : /* pound */
@@ -1674,7 +1717,7 @@ format_number (GString *result,
 			if (time_display_elapsed) {
 				need_time_split = time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
-				append_minute_elapsed (result, &tm, signed_number);
+				append_minute_elapsed (result, &tm, number);
 				break;
 			}
 
@@ -1714,7 +1757,7 @@ format_number (GString *result,
 			if (time_display_elapsed) {
 				need_time_split = time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
-				append_second_elapsed (result, signed_number);
+				append_second_elapsed (result, number);
 			} else {
 				if (need_time_split)
 					need_time_split = split_time (&tm, signed_number, date_conv);
@@ -1734,7 +1777,7 @@ format_number (GString *result,
 			if (time_display_elapsed) {
 				need_time_split = time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
-				append_hour_elapsed (result, &tm, signed_number);
+				append_hour_elapsed (result, &tm, number);
 			} else {
 				/* h == hour optionally in 24 hour mode
 				 * h followed by am/pm puts it in 12 hour mode
@@ -1874,7 +1917,7 @@ fmt_general_float (GString *result, gnm_float val, double col_width)
 
 	/* leave space for the decimal */
 	/* FIXME : idealy we would use the width of a decimal point */
-	prec = (int) floor (col_width - 1.);
+	prec = (int) floor (col_width - .4);
 	if (prec < 0)
 		prec = 0;
 
@@ -2076,6 +2119,10 @@ number_format_init (void)
 {
 	style_format_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	beyond_precision = gpow10 (GNUM_DIG + 1);
+
+	lc_decimal = g_string_new (NULL);
+	lc_thousand = g_string_new (NULL);
+	lc_currency = g_string_new (NULL);
 }
 
 static void
@@ -2090,6 +2137,15 @@ cb_format_leak (gpointer key, gpointer value, gpointer user_data)
 void
 number_format_shutdown (void)
 {
+	g_string_free (lc_decimal, TRUE);
+	lc_decimal = NULL;
+
+	g_string_free (lc_thousand, TRUE);
+	lc_thousand = NULL;
+
+	g_string_free (lc_currency, TRUE);
+	lc_currency = NULL;
+
 	if (default_percentage_fmt) {
 		style_format_unref (default_percentage_fmt);
 		default_percentage_fmt = NULL;
@@ -2161,17 +2217,17 @@ style_format_delocalize (char const *descriptor_string)
 		return g_strdup ("");
 
 	if (strcmp (descriptor_string, _("General"))) {
-		char const *thousands_sep = format_get_thousand ();
-		char const *decimal = format_get_decimal ();
+		GString const *thousands_sep = format_get_thousand ();
+		GString const *decimal = format_get_decimal ();
 		char const *ptr = descriptor_string;
 		GString *res = g_string_sized_new (strlen (ptr));
 
 		for ( ; *ptr ; ++ptr) {
-			if (strncmp (ptr, decimal, strlen (decimal)) == 0) {
-				ptr += strlen (decimal) - 1;
+			if (strncmp (ptr, decimal->str, decimal->len) == 0) {
+				ptr += decimal->len - 1;
 				g_string_append_c (res, '.');
-			} else if (strncmp (ptr, thousands_sep, strlen (thousands_sep)) == 0) {
-				ptr += strlen (thousands_sep) - 1;
+			} else if (strncmp (ptr, thousands_sep->str, thousands_sep->len) == 0) {
+				ptr += thousands_sep->len - 1;
 				g_string_append_c (res, ',');
 			} else if (*ptr == '\"') {
 				do {
@@ -2187,8 +2243,8 @@ style_format_delocalize (char const *descriptor_string)
 				if (*ptr == '\\' && ptr[1] != '\0') {
 					ptr++;
 					/* Ignore '\' if we probably added it */
-					if (strncmp (ptr, decimal, strlen (decimal)) != 0 &&
-					    strncmp (ptr, thousands_sep, strlen (thousands_sep)) != 0)
+					if (strncmp (ptr, decimal->str, decimal->len) != 0 &&
+					    strncmp (ptr, thousands_sep->str, thousands_sep->len) != 0)
 						g_string_append_c (res, '\\');
 				}
 				g_string_append_c (res, *ptr);
@@ -2235,6 +2291,10 @@ style_format_new_XL (char const *descriptor_string, gboolean delocalize)
 		g_hash_table_insert (style_format_hash, format->format, format);
 	}
 	format->ref_count++;
+#ifdef DEBUG_REF_COUNT
+	g_message (__FUNCTION__ " format=%p '%s' ref_count=%d",
+		   format, format->format, format->ref_count);
+#endif
 
 	g_free (desc_copy);
 	return format;
@@ -2284,8 +2344,7 @@ style_format_build (FormatFamily family, const FormatCharacteristics *info)
 char *
 style_format_str_as_XL (char const *ptr, gboolean localized)
 {
-	char const *thousands_sep, *decimal;
-	size_t thousands_sep_len, decimal_len;
+	GString const *thousands_sep, *decimal;
 	GString *res;
 
 	g_return_val_if_fail (ptr != NULL,
@@ -2298,10 +2357,7 @@ style_format_str_as_XL (char const *ptr, gboolean localized)
 		return g_strdup (_("General"));
 
 	thousands_sep = format_get_thousand ();
-	thousands_sep_len = strlen (thousands_sep);
-
 	decimal = format_get_decimal ();
-	decimal_len = strlen (decimal);
 
 	res = g_string_sized_new (strlen (ptr));
 
@@ -2321,10 +2377,10 @@ style_format_str_as_XL (char const *ptr, gboolean localized)
 	for ( ; *ptr ; ++ptr)
 		switch (*ptr) {
 		case '.':
-			g_string_append_len (res, decimal, decimal_len);
+			gnm_string_append_gstring (res, decimal);
 			break;
 		case ',':
-			g_string_append_len (res, thousands_sep, thousands_sep_len);
+			gnm_string_append_gstring (res, thousands_sep);
 			break;
 
 		case '\"':
@@ -2351,8 +2407,8 @@ style_format_str_as_XL (char const *ptr, gboolean localized)
 		}
 
 		default:
-			if (strncmp (ptr, decimal, decimal_len) == 0 ||
-			    strncmp (ptr, thousands_sep, thousands_sep_len) == 0)
+			if (strncmp (ptr, decimal->str, decimal->len) == 0 ||
+			    strncmp (ptr, thousands_sep->str, thousands_sep->len) == 0)
 				g_string_append_c (res, '\\');
 			g_string_append_c (res, *ptr);
 		}
@@ -2401,6 +2457,10 @@ style_format_ref (GnmFormat *sf)
 	g_return_if_fail (sf != NULL);
 
 	sf->ref_count++;
+#ifdef DEBUG_REF_COUNT
+	g_message (__FUNCTION__ " format=%p '%s' ref_count=%d",
+		   sf, sf->format, sf->ref_count);
+#endif
 }
 
 /**
@@ -2418,6 +2478,10 @@ style_format_unref (GnmFormat *sf)
 	g_return_if_fail (sf->ref_count > 0);
 
 	sf->ref_count--;
+#ifdef DEBUG_REF_COUNT
+	g_message (__FUNCTION__ " format=%p '%s' ref_count=%d",
+		   sf, sf->format, sf->ref_count);
+#endif
 	if (sf->ref_count != 0)
 		return;
 
