@@ -15,6 +15,12 @@
 #define FUNCTION_SELECT_GCONF_RECENT "/apps/gnumeric/functionselector/recentfunctions"
 #define FUNCTION_SELECT_GCONF_NUM_OF_RECENT "/apps/gnumeric/functionselector/num-of-recent"
 
+#define CONF_DEFAULT_FONT_DIR	"/apps/gnumeric/core/defaultfont/"
+#define CONF_DEFAULT_FONT_NAME	  CONF_DEFAULT_FONT_DIR "name"
+#define CONF_DEFAULT_FONT_SIZE	  CONF_DEFAULT_FONT_DIR "size"
+#define CONF_DEFAULT_FONT_BOLD	  CONF_DEFAULT_FONT_DIR "bold"
+#define CONF_DEFAULT_FONT_ITALIC  CONF_DEFAULT_FONT_DIR "italic"
+
 #define PLUGIN_GCONF_DIRECTORY "/apps/gnumeric/plugins"
 #define PLUGIN_GCONF_ACTIVATE_NEW PLUGIN_GCONF_DIRECTORY "/activate-new"
 #define PLUGIN_GCONF_ACTIVE PLUGIN_GCONF_DIRECTORY "/active"
@@ -28,9 +34,31 @@
 
 #define PRINTSETUP_GCONF_DIRECTORY "/apps/gnumeric/printsetup"
 #define PRINTSETUP_GCONF_ALL_SHEETS PRINTSETUP_GCONF_DIRECTORY "/all-sheets"
-
-#define PRINTING_GCONF_DIRECTORY "/apps/gnumeric/printing"
-#define PRINTING_GCONF_PRINTER_CONFIG PRINTING_GCONF_DIRECTORY "/printer-config"
+#define PRINTSETUP_GCONF_PRINTER_CONFIG PRINTSETUP_GCONF_DIRECTORY "/printer-config"
+#define PRINTSETUP_GCONF_HEADER PRINTSETUP_GCONF_DIRECTORY "/header"
+#define PRINTSETUP_GCONF_FOOTER PRINTSETUP_GCONF_DIRECTORY "/footer"
+#define PRINTSETUP_GCONF_HF_FONT_NAME PRINTSETUP_GCONF_DIRECTORY "/hf-font-name"
+#define PRINTSETUP_GCONF_HF_FONT_SIZE PRINTSETUP_GCONF_DIRECTORY "/hf-font-size"
+#define PRINTSETUP_GCONF_HF_FONT_BOLD PRINTSETUP_GCONF_DIRECTORY "/hf-font-bold"
+#define PRINTSETUP_GCONF_HF_FONT_ITALIC PRINTSETUP_GCONF_DIRECTORY "/hf-font-italic"
+#define PRINTSETUP_GCONF_CENTER_HORIZONTALLY PRINTSETUP_GCONF_DIRECTORY "/center-horizontally"
+#define PRINTSETUP_GCONF_CENTER_VERTICALLY PRINTSETUP_GCONF_DIRECTORY "/center-vertically"
+#define PRINTSETUP_GCONF_PRINT_GRID_LINES PRINTSETUP_GCONF_DIRECTORY "/print-grid-lines"
+#define PRINTSETUP_GCONF_EVEN_IF_ONLY_STYLES PRINTSETUP_GCONF_DIRECTORY "/print-even-if-only-styles"
+#define PRINTSETUP_GCONF_PRINT_BLACK_AND_WHITE PRINTSETUP_GCONF_DIRECTORY "/print-black-n-white"
+#define PRINTSETUP_GCONF_PRINT_TITLES PRINTSETUP_GCONF_DIRECTORY "/print-titles"
+#define PRINTSETUP_GCONF_RIGHT_THEN_DOWN PRINTSETUP_GCONF_DIRECTORY "/right-then-down"
+#define PRINTSETUP_GCONF_SCALE_PERCENTAGE PRINTSETUP_GCONF_DIRECTORY "/scale-percentage"
+#define PRINTSETUP_GCONF_SCALE_PERCENTAGE_VALUE PRINTSETUP_GCONF_DIRECTORY "/scale-percentage-value"
+#define PRINTSETUP_GCONF_SCALE_WIDTH PRINTSETUP_GCONF_DIRECTORY "/scale-width"
+#define PRINTSETUP_GCONF_SCALE_HEIGHT PRINTSETUP_GCONF_DIRECTORY "/scale-height"
+#define PRINTSETUP_GCONF_REPEAT_TOP PRINTSETUP_GCONF_DIRECTORY "/repeat-top"
+#define PRINTSETUP_GCONF_REPEAT_LEFT PRINTSETUP_GCONF_DIRECTORY "/repeat-left"
+#define PRINTSETUP_GCONF_MARGIN_TOP PRINTSETUP_GCONF_DIRECTORY "/margin-top"
+#define PRINTSETUP_GCONF_MARGIN_BOTTOM PRINTSETUP_GCONF_DIRECTORY "/margin-bottom"
+#define PRINTSETUP_GCONF_HEADER_FORMAT_LEFT PRINTSETUP_GCONF_DIRECTORY "/hf-left"
+#define PRINTSETUP_GCONF_HEADER_FORMAT_MIDDLE PRINTSETUP_GCONF_DIRECTORY "/hf-middle"
+#define PRINTSETUP_GCONF_HEADER_FORMAT_RIGHT PRINTSETUP_GCONF_DIRECTORY "/hf-right"
 
 #define DIALOGS_GCONF_DIRECTORY "/apps/gnumeric/dialogs"
 #define DIALOGS_GCONF_UNFOCUSED_RS DIALOGS_GCONF_DIRECTORY "/rs/unfocused"
@@ -54,7 +82,6 @@
 #define GNUMERIC_GCONF_FILE_DIRECTORY "/apps/gnumeric/core/file"
 #define GNUMERIC_GCONF_FILE_HISTORY_N GNUMERIC_GCONF_FILE_DIRECTORY "/history/n"
 #define GNUMERIC_GCONF_FILE_HISTORY_FILES GNUMERIC_GCONF_FILE_DIRECTORY "/history/files"
-#define GNUMERIC_GCONF_FILE_IMPORT_USES_ALL_OP GNUMERIC_GCONF_FILE_DIRECTORY "/import/all-openers"
 #define GNUMERIC_GCONF_FILE_OVERWRITE_DEFAULT GNUMERIC_GCONF_FILE_DIRECTORY "/save/def-overwrite"
 #define GNUMERIC_GCONF_FILE_SINGLE_SHEET_SAVE GNUMERIC_GCONF_FILE_DIRECTORY "/save/single_sheet"
 
@@ -64,6 +91,7 @@
 #define GNUMERIC_GCONF_GUI_RES_H GNUMERIC_GCONF_GUI_DIRECTORY "/screen/horizontaldpi"
 #define GNUMERIC_GCONF_GUI_RES_V GNUMERIC_GCONF_GUI_DIRECTORY "/screen/verticaldpi"
 #define GNUMERIC_GCONF_GUI_ED_AUTOCOMPLETE GNUMERIC_GCONF_GUI_DIRECTORY "/editing/autocomplete"
+#define GNUMERIC_GCONF_GUI_ED_TRANSITION_KEYS GNUMERIC_GCONF_GUI_DIRECTORY "/editing/transitionkeys"
 #define GNUMERIC_GCONF_GUI_ED_LIVESCROLLING GNUMERIC_GCONF_GUI_DIRECTORY "/editing/livescrolling"
 #define GNUMERIC_GCONF_GUI_ED_RECALC_LAG GNUMERIC_GCONF_GUI_DIRECTORY "/editing/recalclag"
 #define GNUMERIC_GCONF_GUI_WINDOW_X GNUMERIC_GCONF_GUI_DIRECTORY "/window/x"
@@ -76,9 +104,16 @@
 #define GNUMERIC_GCONF_SORT_DEFAULT_BY_CASE GNUMERIC_GCONF_SORT_DIRECTORY "/default/by-case"
 #define GNUMERIC_GCONF_SORT_DEFAULT_RETAIN_FORM GNUMERIC_GCONF_SORT_DIRECTORY "/default/retain-formats"
 #define GNUMERIC_GCONF_SORT_DEFAULT_ASCENDING GNUMERIC_GCONF_SORT_DIRECTORY "/default/ascending"
-#define GNUMERIC_GCONF_SORT_DIALOG_MAX_INITIAL GNUMERIC_GCONF_SORT_DIRECTORY "/dialog/max-intial-clauses"
+#define GNUMERIC_GCONF_SORT_DIALOG_MAX_INITIAL GNUMERIC_GCONF_SORT_DIRECTORY "/dialog/max-initial-clauses"
 
 #define GNUMERIC_GCONF_CUTANDPASTE_DIRECTORY "/apps/gnumeric/cut-and-paste"
 #define GNUMERIC_GCONF_CUTANDPASTE_PREFER_CLIPBOARD GNUMERIC_GCONF_CUTANDPASTE_DIRECTORY "/prefer-clipboard"
 
+/*
+ *  schemas/gnumeric-plugins.schemas
+ */
+
+#define PLUGIN_GCONF_LATEX "/apps/gnumeric/plugin/latex"
+#define PLUGIN_GCONF_LATEX_USE_UTF8 PLUGIN_GCONF_LATEX "/use-utf8"
+
 #endif /* GNUMERIC_GCONF_PRIV_H */
