@@ -7,7 +7,7 @@
  *  Jody Goldberg (jody@gnome.org)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "gui-clipboard.h"
 
