@@ -5,11 +5,11 @@
  * Author:
  *  Miguel de Icaza (miguel@gnu.org)
  */
-
 #undef GTK_DISABLE_DEPRECATED
-#warning "This file uses GTK_DISABLE_DEPRECATED"
+#warning "This file uses GTK_DISABLE_DEPRECATED for GtkCombo and some signal handling"
+
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "gui-util.h"
 
@@ -26,11 +26,10 @@
 #include "workbook.h"
 #include "gnumeric-paths.h"
 
-#include <string.h>
-#include <widgets/widget-color-combo.h>
-#include <widgets/preview-file-selection.h>
+#include <goffice/gui-utils/go-combo-color.h>
 #include <glade/glade.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkfilechooser.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkeventbox.h>
@@ -41,17 +40,12 @@
 #include <gtk/gtkframe.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkbbox.h>
+#include <gtk/gtkhbox.h>
 #include <atk/atkrelation.h>
 #include <atk/atkrelationset.h>
-
-#ifdef WITH_BONOBO
-#	include <bonobo.h>
-#endif
-
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-help.h>
-#include <libgnomeui/gnome-app.h>
-#include <libgnomeui/gnome-app-helper.h>
+
+#include <string.h>
 
 gboolean
 gnumeric_dialog_question_yes_no (WorkbookControlGUI *wbcg,
@@ -107,135 +101,62 @@ gnumeric_notice_nonmodal (GtkWindow *parent, GtkWidget **ref, GtkMessageType typ
 	return;
 }
 
-
 static void
-fsel_dialog_finish (GtkWidget *widget)
+fsel_response_cb (GtkFileChooser *dialog,
+		  gint response_id,
+		  gboolean *result)
 {
-	gtk_widget_hide (widget);
-	gtk_main_quit ();
-}
+  if (response_id == GTK_RESPONSE_OK) {
+	  char *uri = gtk_file_chooser_get_uri (dialog);
 
-static void
-fsel_dir_handle_ok (GtkWidget *widget, gboolean *result)
-{
-	GtkFileSelection *fsel;
+	  if (uri) {
+		  g_free (uri);
+		  *result = TRUE;
+	  }
+  }
 
-	fsel = GTK_FILE_SELECTION (gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
-	fsel_dialog_finish (GTK_WIDGET (fsel));
-	*result = TRUE;
-}
-
-static void
-fsel_handle_ok (GtkWidget *widget, gboolean *result)
-{
-	GtkFileSelection *fsel;
-	gchar const *file_name;
-
-	fsel = GTK_FILE_SELECTION (gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
-	file_name = gtk_file_selection_get_filename (fsel);
-
-	/* Change into directory if that's what user selected */
-	if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
-		gint name_len;
-		gchar *dir_name;
-
-		name_len = strlen (file_name);
-		if (name_len < 1 || file_name [name_len - 1] != '/') {
-			/* The file selector needs a '/' at the end of a directory name */
-			dir_name = g_strconcat (file_name, "/", NULL);
-		} else {
-			dir_name = g_strdup (file_name);
-		}
-		gtk_file_selection_set_filename (fsel, dir_name);
-		g_free (dir_name);
-	} else {
-		fsel_dialog_finish (GTK_WIDGET (fsel));
-		*result = TRUE;
-	}
-}
-
-static void
-fsel_handle_cancel (GtkWidget *widget, gpointer user_data)
-{
-	GtkWidget *fsel;
-
-	fsel = gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION);
-	fsel_dialog_finish (fsel);
+  gtk_main_quit ();
 }
 
 static gint
-fsel_delete_event (GtkWidget *widget, GdkEventAny *event)
+gu_delete_handler (GtkDialog *dialog,
+		   GdkEventAny *event,
+		   gpointer data)
 {
-	fsel_dialog_finish (widget);
-	return TRUE;
-}
-
-static gint
-fsel_key_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
-{
-	if (event->keyval == GDK_Escape) {
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-		fsel_dialog_finish (widget);
-		return TRUE;
-	}
-
-	return FALSE;
+	gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
+	return TRUE; /* Do not destroy */
 }
 
 gboolean
-gnumeric_dialog_file_selection (WorkbookControlGUI *wbcg, GtkFileSelection *fsel)
+gnumeric_dialog_file_selection (WorkbookControlGUI *wbcg, GtkWidget *w)
 {
 	/* Note: wbcg will be NULL if called (indirectly) from gog-style.c  */
 	gboolean result = FALSE;
+	gulong delete_handler;
 
-	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
+	g_return_val_if_fail (GTK_IS_FILE_CHOOSER (w), FALSE);
+
+	gtk_window_set_modal (GTK_WINDOW (w), TRUE);
 	if (wbcg)
 		gnumeric_set_transient (wbcg_toplevel (wbcg), 
-					GTK_WINDOW (fsel));
-	g_signal_connect (G_OBJECT (fsel->ok_button),
-		"clicked",
-		G_CALLBACK (fsel_handle_ok), &result);
-	g_signal_connect (G_OBJECT (fsel->cancel_button),
-		"clicked",
-		G_CALLBACK (fsel_handle_cancel), NULL);
-	g_signal_connect (G_OBJECT (fsel),
-		"key_press_event",
-		G_CALLBACK (fsel_key_event), NULL);
-	g_signal_connect (G_OBJECT (fsel),
-		"delete_event",
-		G_CALLBACK (fsel_delete_event), NULL);
-	gtk_widget_show_all (GTK_WIDGET (fsel));
-	gtk_grab_add (GTK_WIDGET (fsel));
+					GTK_WINDOW (w));
+	g_signal_connect (w, "response",
+			  G_CALLBACK (fsel_response_cb), &result);
+	delete_handler =
+		g_signal_connect (w,
+				  "delete_event",
+				  G_CALLBACK (gu_delete_handler),
+				  NULL);
+
+	gtk_widget_show_all (w);
+	gtk_grab_add (w);
 	gtk_main ();
 
+	g_signal_handler_disconnect (w, delete_handler);
+
 	return result;
 }
 
-gboolean
-gnumeric_dialog_dir_selection (WorkbookControlGUI *wbcg, GtkFileSelection *fsel)
-{
-	gboolean result = FALSE;
-
-	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
-	gnumeric_set_transient (wbcg_toplevel (wbcg), GTK_WINDOW (fsel));
-	g_signal_connect (G_OBJECT (fsel->ok_button),
-		"clicked",
-		G_CALLBACK (fsel_dir_handle_ok), &result);
-	g_signal_connect (G_OBJECT (fsel->cancel_button),
-		"clicked",
-		G_CALLBACK (fsel_handle_cancel), NULL);
-	g_signal_connect (G_OBJECT (fsel),
-		"key_press_event",
-		G_CALLBACK (fsel_key_event), NULL);
-	g_signal_connect (G_OBJECT (fsel),
-		"delete_event",
-		G_CALLBACK (fsel_delete_event), NULL);
-	gtk_widget_show_all (GTK_WIDGET (fsel));
-	gtk_grab_add (GTK_WIDGET (fsel));
-	gtk_main ();
-
-	return result;
-}
 
 static gint
 cb_modal_dialog_keypress (GtkWidget *w, GdkEventKey *e)
@@ -270,7 +191,8 @@ gnumeric_dialog_run (WorkbookControlGUI *wbcg, GtkDialog *dialog)
 		"key-press-event",
 		G_CALLBACK (cb_modal_dialog_keypress), NULL);
 
-	result = gtk_dialog_run (dialog);
+	while ((result = gtk_dialog_run (dialog)) >= 0)
+	       ;
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	return result;
 }
@@ -574,92 +496,6 @@ gnumeric_combo_enters (GtkWindow *window, GtkWidget *combo)
 	gnumeric_editable_enters (window, GTK_WIDGET (GTK_COMBO (combo)->entry));
 }
 
-/**
- * gnumeric_toolbar_insert_with_eventbox
- * @toolbar               toolbar
- * @widget                widget to insert
- * @tooltip_text          tooltip text
- * @tooltip_private_text  longer tooltip text
- * @position              widget position in toolbar
- *
- * Packs widget in an eventbox and adds the eventbox to toolbar.
- * This lets a windowless widget (e.g. combo box) have tooltips.
- **/
-void
-gnumeric_toolbar_insert_with_eventbox (GtkToolbar *toolbar, GtkWidget  *widget,
-				       const char *tooltip_text,
-				       const char *tooltip_private_text,
-				       gint        position)
-{
-	GtkWidget *eventbox;
-
-	g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (position >= 0);
-	g_return_if_fail (position <= toolbar->num_children);
-
-	/* An event box to receive events - this is a requirement for having
-           tooltips */
-	eventbox = gtk_event_box_new ();
-	gtk_widget_show (widget);
-	gtk_container_add (GTK_CONTAINER (eventbox), widget);
-	gtk_widget_show (eventbox);
-	gtk_toolbar_insert_widget (GTK_TOOLBAR (toolbar), eventbox,
-				   tooltip_text, tooltip_private_text,
-				   position);
-}
-
-/**
- * gnumeric_toolbar_append_with_eventbox
- * @toolbar               toolbar
- * @widget                widget to insert
- * @tooltip_text          tooltip text
- * @tooltip_private_text  longer tooltip text
- *
- * Packs widget in an eventbox and adds the eventbox to toolbar.
- * This lets a windowless widget (e.g. combo box) have tooltips.
- **/
-void
-gnumeric_toolbar_append_with_eventbox (GtkToolbar *toolbar, GtkWidget  *widget,
-				       const char *tooltip_text,
-				       const char *tooltip_private_text)
-{
-	GtkWidget *eventbox;
-
-	g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
-	g_return_if_fail (widget != NULL);
-
-	/* An event box to receive events - this is a requirement for having
-           tooltips */
-	eventbox = gtk_event_box_new ();
-	gtk_widget_show (widget);
-	gtk_container_add (GTK_CONTAINER (eventbox), widget);
-	gtk_widget_show (eventbox);
-	gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), eventbox,
-				   tooltip_text, tooltip_private_text);
-}
-
-/**
- * gtk_button_stock_alignment_set :
- * @button :
- * ...
- *
- * gtk-1.3 hard codes the alignment of stock content to be .5,.5,0,0
- * this makes it hard to align stock icons within buttons.
- */
-void
-gtk_button_stock_alignment_set (GtkButton *button,
-				gfloat     xalign,
-				gfloat     yalign,
-				gfloat     xscale,
-				gfloat     yscale)
-{
-	g_return_if_fail (GTK_IS_BUTTON (button));
-
-	gtk_alignment_set (GTK_ALIGNMENT (gtk_bin_get_child (GTK_BIN (button))),
-		xalign, yalign, xscale, yscale);
-}
-
 int
 gtk_radio_group_get_selected (GSList *radio_group)
 {
@@ -719,7 +555,9 @@ gnumeric_popup_menu (GtkMenu *menu, GdkEventButton *event)
 	 * instead pass 0.  Otherwise bringing up a menu with
 	 * the right button will disable clicking on the menu with the left.
 	 */
-	gtk_menu_popup (menu, NULL, NULL, 0, NULL, 0, event->time);
+	gtk_menu_popup (menu, NULL, NULL, 0, NULL, 0, 
+			(event != NULL) ? event->time 
+			: gtk_get_current_event_time());
 }
 
 
@@ -832,34 +670,6 @@ gnumeric_non_modal_dialog (GtkWindow *toplevel, GtkWindow *dialog)
 		G_CALLBACK (cb_non_modal_dialog_keypress), NULL);
 }
 
-#ifdef WITH_BONOBO
-/*
- * gnumeric_inject_widget_into_bonoboui :
- *
- * A quick utility routine to inject a widget into a menu/toolbar.
- */
-void
-gnumeric_inject_widget_into_bonoboui (WorkbookControlGUI *wbcg, GtkWidget *widget, char const *path)
-{
-	BonoboControl *control;
-	CORBA_Environment ev;
-
-	gtk_widget_show_all (widget);
-	control = bonobo_control_new (widget);
-
-	CORBA_exception_init (&ev);
-	bonobo_ui_component_object_set (
-		wbcg->uic, path,
-		BONOBO_OBJREF (control),
-		&ev);
-	/* if there was a problem injecting the widget then nothing is refing
-	 * the control and the widget will go away under our feet */
-	if (!BONOBO_EX (&ev))
-		bonobo_object_unref (BONOBO_OBJECT (control));
-	CORBA_exception_free (&ev);
-}
-#endif
-
 static void
 popup_item_activate (GtkWidget *item, gpointer *user_data)
 {
@@ -956,92 +766,39 @@ gnumeric_create_popup_menu (GnumericPopupMenuElement const *elements,
 }
 
 /**
- * color_combo_get_style_color :
+ * go_combo_color_get_style_color :
  *
  * A utility wrapper to map between gal's colour combo and gnumeric's StyleColors.
  */
 GnmColor *
-color_combo_get_style_color (GtkWidget *color_combo)
+go_combo_color_get_style_color (GtkWidget *go_combo_color)
 {
 	GnmColor *sc = NULL;
-	GdkColor *gdk = color_combo_get_color (COLOR_COMBO (color_combo), NULL);
-	if (gdk != NULL) {
-		    sc = style_color_new (gdk->red, gdk->green, gdk->blue);
-		    gdk_color_free (gdk);
+	guint16   r, g, b;
+	GOColor color = go_combo_color_get_color (GO_COMBO_COLOR (go_combo_color), NULL);
+	if (UINT_RGBA_A (color) >= 0x80) {
+		r  = UINT_RGBA_R (color); r |= (r << 8);
+		g  = UINT_RGBA_G (color); g |= (g << 8);
+		b  = UINT_RGBA_B (color); b |= (b << 8);
+		sc = style_color_new (r, g, b);
 	}
 	return sc;
 }
 
-GtkWidget *
-gnumeric_toolbar_new (WorkbookControlGUI *wbcg,
-		      gpointer info, /* GnomeUIInfo *info */
-		      char const *name,
-		      gint band_num,
-		      gint band_position,
-		      gint offset)
-{
-	GnomeApp *app = GNOME_APP (wbcg->toplevel);
-	GtkWidget *tbar;
-	BonoboDockItemBehavior behavior;
-
-	g_return_val_if_fail (info != NULL, NULL);
-
-	tbar = gtk_toolbar_new ();
-	gtk_toolbar_set_orientation (GTK_TOOLBAR (tbar), GTK_ORIENTATION_HORIZONTAL);
-	gnome_app_fill_toolbar_with_data (GTK_TOOLBAR (tbar), info,
-		app->accel_group, wbcg);
-
-	behavior = gconf_client_get_bool (gnm_app_get_gconf_client (),
-		"/desktop/gnome/interface/toolbar_detachable", NULL)
-		? BONOBO_DOCK_ITEM_BEH_NORMAL
-		: BONOBO_DOCK_ITEM_BEH_LOCKED;
-
-	gnome_app_add_toolbar (GNOME_APP (wbcg->toplevel), GTK_TOOLBAR (tbar),
-		name, behavior, BONOBO_DOCK_TOP,
-		band_num, band_position, offset);
-
-	return tbar;
-}
-
-GtkWidget *
-gnumeric_toolbar_get_widget (GtkToolbar *toolbar, int pos)
-{
-	GtkToolbarChild *child;
-	GList *children;
-	int i;
-
-	g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), NULL);
-	g_return_val_if_fail (pos > 0, NULL);
-
-	children = GTK_TOOLBAR (toolbar)->children;
-
-	if (!children)
-		return NULL;
-
-	i = 0;
-	do {
-		child = children->data;
-		children = children->next;
-
-		if (child->type == GTK_TOOLBAR_CHILD_SPACE)
-			continue;
-
-		if (i == pos)
-			return child->widget;
-		i++;
-	} while (children);
-
-	return NULL;
-}
-
+#ifdef WITH_GNOME
+#include <libgnome/gnome-help.h>
+#endif
 void
 gnumeric_help_display (char const *link)
 {
         g_return_if_fail (link != NULL);
+#ifdef WITH_GNOME
 	gnome_help_display ("gnumeric", link, NULL);
+#else
+	g_warning ("TODO : launch help browser for %s", link);
+#endif
 }
 
-
 static void
 cb_help (GtkWidget *button, char const *link)
 {
@@ -1219,10 +976,16 @@ int_to_entry (GtkEntry *entry, gint the_int)
 		value_release(val);
 }
 
+char *
+gnumeric_icondir (char const *filename)
+{
+	return g_build_path (G_DIR_SEPARATOR_S, GNUMERIC_ICONDIR, filename, NULL);
+}
+
 GtkWidget *
 gnumeric_load_image (char const *filename)
 {
-	char *path = g_strconcat (GNUMERIC_ICONDIR "/", filename, NULL);
+	char *path = gnumeric_icondir (filename);
 	GtkWidget *image = gtk_image_new_from_file (path);
 	g_free (path);
 
@@ -1239,7 +1002,7 @@ gnumeric_load_image (char const *filename)
 GdkPixbuf *
 gnumeric_load_pixbuf (char const *filename)
 {
-	char *path = g_strconcat (GNUMERIC_ICONDIR "/", filename, NULL);
+	char *path = gnumeric_icondir (filename);
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (path, NULL);
 	g_free (path);
 	return pixbuf;
@@ -1404,77 +1167,6 @@ gnm_widget_set_cursor_type (GtkWidget *w, GdkCursorType ct)
 	gdk_cursor_unref (cursor);
 }
 
-#ifndef HAVE_GDK_CURSOR_NEW_FROM_PIXBUF
-/* See http://bugzilla.gnome.org/showattachment.cgi?attach_id=17695 */
-static GdkCursor *
-gdk_cursor_new_from_pixbuf (GdkDisplay *display, 
-			    GdkPixbuf  *pixbuf,
-			    gint        x,
-			    gint        y)
-{
-  GdkCursor *cursor;
-  GdkPixmap *pixmap, *mask;
-  guint width, height, n_channels, rowstride, i, j;
-  guint8 *data, *mask_data, *pixels;
-  GdkColor fg = { 0, 0, 0, 0 };
-  GdkColor bg = { 0, 0xffff, 0xffff, 0xffff };
-  GdkScreen *screen;
-
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
-
-  width = gdk_pixbuf_get_width (pixbuf);
-  height = gdk_pixbuf_get_height (pixbuf);
-
-  g_return_val_if_fail (0 <= x && (guint)x < width, NULL);
-  g_return_val_if_fail (0 <= y && (guint)y < height, NULL);
-
-  n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  pixels = gdk_pixbuf_get_pixels (pixbuf);
-
-  data = g_new0 (guint8, (width + 7) / 8 * height);
-  mask_data = g_new0 (guint8, (width + 7) / 8 * height);
-
-  for (j = 0; j < height; j++)
-    {
-      guint8 *src = pixels + j * rowstride;
-      guint8 *d = data + (width + 7) / 8 * j;
-      guint8 *md = mask_data + (width + 7) / 8 * j;
-	
-      for (i = 0; i < width; i++)
-	{
-	  if (src[1] < 0x80)
-	    *d |= 1 << (i % 8);
-	  
-	  if (n_channels == 3 || src[3] >= 0x80) {
-	    *md |= 1 << (i % 8);
-	  }
-
-	  src += n_channels;
-	  if (i % 8 == 7)
-		  d++, md++;
-	}
-    }
-      
-  screen = gdk_display_get_default_screen (display);
-  pixmap = gdk_bitmap_create_from_data (gdk_screen_get_root_window (screen), 
-					data, width, height);
-  g_free (data);
-
-  mask = gdk_bitmap_create_from_data (gdk_screen_get_root_window (screen),
-				      mask_data, width, height);
-  g_free (mask_data);
-   
-  cursor = gdk_cursor_new_from_pixmap (pixmap, mask, &fg, &bg, x, y);
-   
-  g_object_unref (pixmap);
-  g_object_unref (mask);
-  
-  return cursor;
-}
-#endif
-
 GdkCursor *
 gnm_fat_cross_cursor (GdkDisplay *display)
 {
@@ -1722,3 +1414,11 @@ gnm_pixbuf_intelligent_scale (GdkPixbuf *buf, guint width, guint height)
 	return scaled;
 }
 
+void
+gnm_widget_disable_focus (GtkWidget *w)
+{
+	if (GTK_IS_CONTAINER (w))
+		gtk_container_foreach (GTK_CONTAINER (w),
+			(GtkCallback) gnm_widget_disable_focus, NULL);
+	GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);
+}
