@@ -8,7 +8,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "item-cursor.h"
 
@@ -31,6 +31,8 @@
 #include "parse-util.h"
 #include "gui-util.h"
 #include <gsf/gsf-impl-utils.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkstock.h>
 #define GNUMERIC_ITEM "CURSOR"
 #include "item-debug.h"
 
@@ -44,7 +46,7 @@ struct _ItemCursor {
 	FooCanvasItem canvas_item;
 
 	SheetControlGUI *scg;
-	Range		 pos;
+	GnmRange		 pos;
 
 	/* Offset of dragging cell from top left of pos */
 	int col_delta, row_delta;
@@ -64,7 +66,7 @@ struct _ItemCursor {
 	 *     started.
 	 */
 	int   base_x, base_y;
-	Range autofill_src;
+	GnmRange autofill_src;
 	int   autofill_hsize, autofill_vsize;
 
 	/* Cached values of the last bounding box information used */
@@ -111,8 +113,7 @@ item_cursor_finalize (GObject *obj)
 		ic->tip = NULL;
 	}
 
-	if (G_OBJECT_CLASS (parent_class)->finalize)
-		(*G_OBJECT_CLASS (parent_class)->finalize) (obj);
+	(*G_OBJECT_CLASS (parent_class)->finalize) (obj);
 }
 
 static void
@@ -247,6 +248,7 @@ static void
 item_cursor_draw (FooCanvasItem *item, GdkDrawable *drawable,
 		  GdkEventExpose *expose)
 {
+	GdkGCValues values;
 	ItemCursor *ic = ITEM_CURSOR (item);
 	int dx0, dy0, dx1, dy1;
 	GdkPoint points [5];
@@ -353,6 +355,8 @@ item_cursor_draw (FooCanvasItem *item, GdkDrawable *drawable,
 		GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
 	gdk_gc_set_rgb_fg_color (ic->gc, &gs_black);
 	gdk_gc_set_rgb_bg_color (ic->gc, &gs_white);
+	values.function   = GDK_EQUIV;
+	gdk_gc_set_values (ic->gc, &values, GDK_GC_FUNCTION);
 
 	if (draw_external) {
 		switch (draw_handle) {
@@ -476,7 +480,7 @@ item_cursor_draw (FooCanvasItem *item, GdkDrawable *drawable,
 }
 
 gboolean
-item_cursor_bound_set (ItemCursor *ic, Range const *new_bound)
+item_cursor_bound_set (ItemCursor *ic, GnmRange const *new_bound)
 {
 	g_return_val_if_fail (IS_ITEM_CURSOR (ic), FALSE);
 	g_return_val_if_fail (range_is_sane (new_bound), FALSE);
@@ -520,7 +524,7 @@ item_cursor_point (FooCanvasItem *item, double x, double y, int cx, int cy,
 	 * 3) while a guru is up
 	 */
 	if (!ic->visible || ic->style == ITEM_CURSOR_ANTED ||
-	    wbcg_edit_has_guru (ic->scg->wbcg))
+	    wbcg_edit_get_guru (ic->scg->wbcg) != NULL)
 		return DBL_MAX;
 
 	*actual_item = NULL;
@@ -593,41 +597,28 @@ item_cursor_in_drag_handle (ItemCursor *ic, int x, int y)
 static void
 item_cursor_set_cursor (FooCanvas *canvas, ItemCursor *ic, int x, int y)
 {
-	int cursor;
+	GdkCursorType cursor;
 
 	if (item_cursor_in_drag_handle (ic, x, y))
-		cursor = GNM_CURSOR_THIN_CROSS;
+		cursor = GDK_CROSSHAIR;
 	else
-		cursor = GNM_CURSOR_ARROW;
+		cursor = GDK_ARROW;
 
-	gnm_cursor_set_widget (GTK_WIDGET (canvas), cursor);
-}
-
-static Value *
-cb_autofill_bound (Sheet *sheet, int col, int row,
-		   Cell *cell, gpointer want_cols)
-{
-	gboolean cols = GPOINTER_TO_INT (want_cols);
-
-	if (!cell_is_blank (cell))
-		return cols ? value_new_int (col) : value_new_int (row);
-	else
-		return NULL;
+	gnm_widget_set_cursor_type (GTK_WIDGET (canvas), cursor);
 }
 
 static gint
 item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 {
-	FooCanvas	*canvas = item->canvas;
-	GnmCanvas   *gcanvas = GNM_CANVAS (canvas);
+	FooCanvas  *canvas = item->canvas;
+	GnmCanvas  *gcanvas = GNM_CANVAS (canvas);
 	ItemCursor *ic = ITEM_CURSOR (item);
 	int x, y;
 
 	switch (event->type) {
 	case GDK_ENTER_NOTIFY:
-		foo_canvas_w2c (
-			canvas, event->crossing.x, event->crossing.y, &x, &y);
-
+		foo_canvas_w2c (canvas,
+			event->crossing.x, event->crossing.y, &x, &y);
 		item_cursor_set_cursor (canvas, ic, x, y);
 		return TRUE;
 
@@ -635,8 +626,8 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 		int style, button;
 		ItemCursor *special_cursor;
 
-		foo_canvas_w2c (
-			canvas, event->motion.x, event->motion.y, &x, &y);
+		foo_canvas_w2c (canvas,
+			event->motion.x, event->motion.y, &x, &y);
 
 		if (ic->drag_button < 0) {
 			item_cursor_set_cursor (canvas, ic, x, y);
@@ -719,7 +710,6 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 		Sheet *sheet = sc->sheet;
 		int final_col = ic->pos.end.col;
 		int final_row = ic->pos.end.row;
-		Value *tmp = NULL;
 
 		if (ic->drag_button != (int)event->button.button)
 			return TRUE;
@@ -730,23 +720,33 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 		if (sheet_is_region_empty (sheet, &ic->pos))
 			return TRUE;
 
-		/* TODO : This is still different from XL.
-		 * We probably do not want to use find_boundary
-		 * what we really want is to walk forward through
-		 * template_{col,row} as long as it has content
-		 * AND the region to fill is empty.
+		/* If the cell(s) immediately below the ones in the
+		 * auto-fill template are not blank then over-write
+		 * them.
+		 *
+		 * Otherwise, only go as far as the next non-blank
+		 * cells.  
+		 *
+		 * The code below uses find_boundary twice.  a. to
+		 * find the boundary of the column/row that acts as a
+		 * template to define the region to file and b. to
+		 * find the boundary of the region being filled.  
 		 */
+
 		if (event->button.state & GDK_MOD1_MASK) {
 			int template_col = ic->pos.end.col + 1;
 			int template_row = ic->pos.start.row - 1;
+			int boundary_col_for_target;
+			int target_row;
 
-			if (template_row < 0 ||
-			    sheet_is_cell_empty (sheet, ic->pos.start.col,
+			if (template_row < 0 || template_col >= SHEET_MAX_COLS ||
+			    sheet_is_cell_empty (sheet, template_col,
 						 template_row)) {
 
 				template_row = ic->pos.end.row + 1;
-				if (template_row >= SHEET_MAX_ROWS ||
-				    sheet_is_cell_empty (sheet, ic->pos.end.col,
+				if (template_row >= SHEET_MAX_ROWS || 
+				    template_col >= SHEET_MAX_COLS ||
+				    sheet_is_cell_empty (sheet, template_col,
 							 template_row))
 					return TRUE;
 			}
@@ -761,35 +761,48 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 			if (final_col <= ic->pos.end.col)
 				return TRUE;
 
-			/* Make sure we don't overwrite the contents of the fill target */
-			for (x = ic->pos.end.col + 1; x <= final_col; x++) {
-				tmp = sheet_foreach_cell_in_range (sheet,
-					CELL_ITER_IGNORE_BLANK,
-					x, ic->pos.start.row, x, ic->pos.end.row,
-					(CellIterFunc) cb_autofill_bound,
-					GINT_TO_POINTER (TRUE));
+			/* 
+			   Find the boundary of the target region.
+			   We don't want to go beyond this boundary.
+			*/
+			for (target_row = ic->pos.start.row; target_row <= ic->pos.end.row; target_row++) {
+				/* find_boundary is designed for Ctrl-arrow movement.  (Ab)using it for
+				 * finding autofill regions works fairly well.  One little gotcha is
+				 * that if the current col is the last row of a block of data Ctrl-arrow
+				 * will take you to then next block.  The workaround for this is to
+				 * start the search at the last col of the selection, rather than
+				 * the first col of the region being filled.
+				 */
+				boundary_col_for_target = sheet_find_boundary_horizontal 
+					(sheet,
+					 ic->pos.end.col, target_row,
+					 target_row, 1, TRUE);
 
-				if (tmp) {
-					int bound = value_get_as_int (tmp);
-					if (final_col >= bound)
-						final_col = bound - 1;
-					value_release (tmp);
-					break;
+				if (sheet_is_cell_empty (sheet, boundary_col_for_target-1, target_row) &&
+				    ! sheet_is_cell_empty (sheet, boundary_col_for_target, target_row)) { 
+					/* target region was empty, we are now one col
+					   beyond where it is safe to autofill. */
+					boundary_col_for_target--;
+				}
+				if (boundary_col_for_target < final_col) {
+					final_col = boundary_col_for_target;
 				}
 			}
 		} else {
 			int template_row = ic->pos.end.row + 1;
 			int template_col = ic->pos.start.col - 1;
+			int boundary_row_for_target;
+			int target_col;
 
-			if (template_col < 0 ||
+			if (template_col < 0 || template_row >= SHEET_MAX_ROWS ||
 			    sheet_is_cell_empty (sheet, template_col,
-						 ic->pos.start.row)) {
+						 template_row)) {
 
 				template_col = ic->pos.end.col + 1;
-				if (template_col >= SHEET_MAX_COLS ||
+				if (template_col >= SHEET_MAX_COLS || 
+				    template_row >= SHEET_MAX_ROWS ||
 				    sheet_is_cell_empty (sheet, template_col,
-
-							 ic->pos.start.row))
+							 template_row))
 					return TRUE;
 			}
 
@@ -803,23 +816,32 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 			if (final_row <= ic->pos.end.row)
 				return TRUE;
 
-			/*
-			 * Make sure we don't overwrite the contents of the fill target
-			 * NOTE : We assume the traversal order is 'do all cols in a row and
-			 *        move on to the next row' here!
-			 */
-			tmp = sheet_foreach_cell_in_range (sheet,
-				CELL_ITER_IGNORE_BLANK,
-				ic->pos.start.col, ic->pos.end.row + 1,
-				ic->pos.end.col, final_row,
-				(CellIterFunc) cb_autofill_bound,
-				GINT_TO_POINTER (FALSE));
+			/* 
+			   Find the boundary of the target region.
+			   We don't want to go beyond this boundary.
+			*/
+			for (target_col = ic->pos.start.col; target_col <= ic->pos.end.col; target_col++) {
+				/* find_boundary is designed for Ctrl-arrow movement.  (Ab)using it for
+				 * finding autofill regions works fairly well.  One little gotcha is
+				 * that if the current row is the last row of a block of data Ctrl-arrow
+				 * will take you to then next block.  The workaround for this is to
+				 * start the search at the last row of the selection, rather than
+				 * the first row of the region being filled.
+				 */
+				boundary_row_for_target = sheet_find_boundary_vertical 
+					(sheet,
+					 target_col, ic->pos.end.row,
+					 target_col, 1, TRUE);
+				if (sheet_is_cell_empty (sheet, target_col, boundary_row_for_target-1) &&
+				    ! sheet_is_cell_empty (sheet, target_col, boundary_row_for_target)) { 
+					/* target region was empty, we are now one row
+					   beyond where it is safe to autofill. */
+					boundary_row_for_target--;
+				}
 
-			if (tmp) {
-				int bound = value_get_as_int (tmp);
-				if (final_row >= bound)
-					final_row = bound - 1;
-				value_release (tmp);
+				if (boundary_row_for_target < final_row) {
+					final_row = boundary_row_for_target;
+				}
 			}
 		}
 
@@ -917,7 +939,7 @@ item_cursor_do_action (ItemCursor *ic, ActionType action)
 	SheetView 	*sv;
 	Sheet		*sheet;
 	WorkbookControl *wbc;
-	PasteTarget pt;
+	GnmPasteTarget pt;
 
 	g_return_if_fail (ic != NULL);
 
@@ -1035,9 +1057,9 @@ item_cursor_do_drop (ItemCursor *ic, GdkEventButton *event)
 {
 	/* Only do the operation if something moved */
 	SheetView const *sv = ((SheetControl *) ic->scg)->view;
-	Range const *target = selection_first_range (sv, NULL, NULL);
+	GnmRange const *target = selection_first_range (sv, NULL, NULL);
 
-	wb_control_gui_set_status_text (ic->scg->wbcg, "");
+	wbcg_set_status_text (ic->scg->wbcg, "");
 	if (range_equal (target, &ic->pos)) {
 		scg_special_cursor_stop	(ic->scg);
 		return;
@@ -1054,12 +1076,12 @@ item_cursor_do_drop (ItemCursor *ic, GdkEventButton *event)
 static void
 item_cursor_set_bounds_visibly (ItemCursor *ic,
 				int visible_col,int visible_row,
-				CellPos const *corner,
+				GnmCellPos const *corner,
 				int end_col, int end_row)
 {
 	FooCanvasItem *item = FOO_CANVAS_ITEM (ic);
 	GnmCanvas  *gcanvas = GNM_CANVAS (item->canvas);
-	Range r;
+	GnmRange r;
 
 	/* Handle visibility here rather than in the slide handler because we
 	 * need to constrain movement some times.  eg no sense scrolling the
@@ -1109,7 +1131,7 @@ cb_move_cursor (GnmCanvas *gcanvas, int col, int row, gpointer user_data)
 	ItemCursor *ic = user_data;
 	int const w = (ic->pos.end.col - ic->pos.start.col);
 	int const h = (ic->pos.end.row - ic->pos.start.row);
-	CellPos corner;
+	GnmCellPos corner;
 
 	corner.col = col - ic->col_delta;
 	if (corner.col < 0)
@@ -1179,7 +1201,7 @@ static gboolean
 cb_autofill_scroll (GnmCanvas *gcanvas, int col, int row, gpointer user_data)
 {
 	ItemCursor *ic = user_data;
-	Range r = ic->autofill_src;
+	GnmRange r = ic->autofill_src;
 
 	/* compass offsets are distances (in cells) from the edges of the
 	 * selected area to the mouse cursor
