@@ -15,9 +15,6 @@ extern gboolean initial_workbook_open_complete;
 extern char *x_geometry;
 
 void init_init (char const* gnumeric_binary);
-poptContext gnumeric_arg_parse    (int argc, char *argv []);
-void	    gnumeric_arg_shutdown (void);
-
 void gnm_common_init	(gboolean fast);
 int  gnm_dump_func_defs (char const* filename, gboolean def_or_state);
 void gnm_shutdown	(void);
