@@ -109,7 +109,7 @@
  *    Almer S. Tigelaar (almer@gnome.org)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "number-match.h"
 #include "expr.h"
@@ -287,7 +287,7 @@ typedef struct {
 	char const *start;	/* start of the expression */
 
 	/* Location where the parsing is taking place */
-	ParsePos const *pos;
+	GnmParsePos const *pos;
 
 	/* Locale info. */
 	gunichar decimal_point;
@@ -304,7 +304,7 @@ typedef struct {
 
 	GnmExprList *result;
 
-	ParseError *error;
+	GnmParseError *error;
 } ParserState;
 
 /* The error returned from the */
@@ -328,12 +328,12 @@ static GnmExpr *
 fold_negative (GnmExpr *expr)
 {
 	if (expr->any.oper == GNM_EXPR_OP_CONSTANT) {
-		Value const *v = expr->constant.value;
+		GnmValue const *v = expr->constant.value;
 
 		if (v->type == VALUE_INTEGER)
-			((Value *)v)->v_int.val   = -v->v_int.val;
+			((GnmValue *)v)->v_int.val   = -v->v_int.val;
 		else if (v->type == VALUE_FLOAT)
-			((Value *)v)->v_float.val = -v->v_float.val;
+			((GnmValue *)v)->v_float.val = -v->v_float.val;
 		else
 			return NULL;
 		return expr;
@@ -386,7 +386,7 @@ build_not (GnmExpr *expr)
 static GnmExpr *
 build_array (GSList *cols)
 {
-	Value *array;
+	GnmValue *array;
 	GSList *row;
 	int x, mx, y;
 
@@ -410,11 +410,11 @@ build_array (GSList *cols)
 		x = 0;
 		while (row && x < mx) {
 			GnmExpr    *expr = row->data;
-			Value const *v = expr->constant.value;
+			GnmValue const *v = expr->constant.value;
 
 			g_assert (expr->any.oper == GNM_EXPR_OP_CONSTANT);
 
-			value_array_set (array, x, y, value_duplicate (v));
+			value_array_set (array, x, y, value_dup (v));
 
 			x++;
 			row = row->next;
@@ -477,13 +477,13 @@ build_set (GnmExprList *list)
  * parse_string_as_value :
  *
  * Try to parse the entered text as a basic value (empty, bool, int,
- * float, err) if this succeeds, we store this as a Value otherwise, we
+ * float, err) if this succeeds, we store this as a GnmValue otherwise, we
  * return a string.
  */
 static GnmExpr *
 parse_string_as_value (GnmExpr *str)
 {
-	Value *v = format_match_simple (str->constant.value->v_str.val->str);
+	GnmValue *v = format_match_simple (str->constant.value->v_str.val->str);
 
 	if (v != NULL) {
 		unregister_allocation (str);
@@ -505,7 +505,7 @@ parser_simple_val_or_name (GnmExpr *str_expr)
 {
 	GnmExpr const	*res;
 	char const	*str = str_expr->constant.value->v_str.val->str;
-	Value		*v   = format_match_simple (str);
+	GnmValue		*v   = format_match_simple (str);
 
 	/* if it is not a simple value see if it is a name */
 	if (v == NULL) {
@@ -514,7 +514,7 @@ parser_simple_val_or_name (GnmExpr *str_expr)
 			if (state->unknown_names_are_strings) {
 				res = gnm_expr_new_constant (value_new_string (str));
 			} else {
-				ParsePos pp = *state->pos;
+				GnmParsePos pp = *state->pos;
 				pp.sheet = NULL;
 				/* Create a place holder */
 				nexpr = expr_name_add (&pp, str, NULL, NULL, TRUE);
@@ -581,14 +581,14 @@ int yyparse (void);
 #line 470 "parser.y"
 typedef union YYSTYPE {
 	GnmExpr		*expr;
-	Value		*value;
-	CellRef		*cell;
+	GnmValue	*value;
+	GnmCellRef	*cell;
 	GnmExprList	*list;
 	Sheet		*sheet;
 	Workbook	*wb;
 } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 592 "y.tab.c"
+#line 592 "parser.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -600,7 +600,7 @@ typedef union YYSTYPE {
 
 
 /* Line 214 of yacc.c.  */
-#line 604 "y.tab.c"
+#line 604 "parser.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -1765,7 +1765,7 @@ yyreduce:
     {
 		GnmNamedExpr *nexpr = NULL;
 		char const *name = yyvsp[0].expr->constant.value->v_str.val->str;
-		ParsePos pos = *state->pos;
+		GnmParsePos pos = *state->pos;
 
 		pos.sheet = yyvsp[-1].sheet;
 		nexpr = expr_name_lookup (&pos, name);
@@ -1787,7 +1787,7 @@ yyreduce:
     {
 		GnmNamedExpr *nexpr = NULL;
 		char const *name = yyvsp[0].expr->constant.value->v_str.val->str;
-		ParsePos pos = *state->pos;
+		GnmParsePos pos = *state->pos;
 
 		pos.sheet = NULL;
 		pos.wb = yyvsp[-1].wb;
@@ -1843,7 +1843,7 @@ yyreduce:
 #line 654 "parser.y"
     {
 		char const *wb_name = yyvsp[-1].expr->constant.value->v_str.val->str;
-		Workbook *wb = application_workbook_get_by_name (wb_name);
+		Workbook *wb = gnm_app_workbook_get_by_name (wb_name);
 
 		if (wb != NULL) {
 			unregister_allocation (yyvsp[-1].expr); gnm_expr_unref (yyvsp[-1].expr);
@@ -2038,7 +2038,7 @@ yyreduce:
     }
 
 /* Line 999 of yacc.c.  */
-#line 2042 "y.tab.c"
+#line 2042 "parser.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -2249,7 +2249,7 @@ find_matching_close (char const *str, char const **res)
 		} else if (*str == ')')
 			return str;
 		else if (*str == '\'' || *str == '\"') {
-			GString *dummy = g_string_new ("");
+			GString *dummy = g_string_new (NULL);
 			char const *end = gnm_strunescape (dummy, str);
 			g_string_free (dummy, TRUE);
 			if (end == NULL)
@@ -2279,7 +2279,7 @@ yylex (void)
 {
 	gunichar c;
 	char const *start, *end;
-	RangeRef ref;
+	GnmRangeRef ref;
 	gboolean is_number = FALSE;
 	gboolean is_space = FALSE;
 	gboolean error_token = FALSE;
@@ -2329,7 +2329,7 @@ yylex (void)
 			char const *quotes_end;
 			char const *p;
 			char *string, *s;
-			Value *v;
+			GnmValue *v;
 
 			if (*state->ptr == 'q') {
 				quotes_end = "&quot;";
@@ -2491,7 +2491,7 @@ yylex (void)
 	}
 
 	if (is_number) {
-		Value *v = NULL;
+		GnmValue *v = NULL;
 
 		if (c == state->decimal_point || c == 'e' || c == 'E') {
 			/* This is float */
@@ -2565,11 +2565,11 @@ yylex (void)
 			       !g_unichar_isspace (tmp)) {
 				state->ptr = g_utf8_next_char (state->ptr);
 				if (tmp == '!' || tmp == '?') {
-					String *name = string_get_nocopy (g_strndup (start, state->ptr - start));
+					GnmString *name = gnm_string_get_nocopy (g_strndup (start, state->ptr - start));
 					yylval.expr = register_expr_allocation
 						(gnm_expr_new_constant (
 							value_new_error_str (NULL, name)));
-					string_unref (name);
+					gnm_string_unref (name);
 					return CONSTANT;
 				}
 			}
@@ -2586,8 +2586,8 @@ yylex (void)
 		/* Fall through */
 	case '\'':
 	case '"': {
-		GString *s = g_string_new ("");
-		const char *end = gnm_strunescape (s, start);
+		GString *s = g_string_new (NULL);
+		char const *end = gnm_strunescape (s, start);
 
 		if (end == NULL) {
 			size_t len = strlen (start);
@@ -2602,12 +2602,12 @@ yylex (void)
 		state->ptr = (char *)end;
 
 		if (error_token) {
-			Value *v = value_new_error (NULL, s->str);
+			GnmValue *v = value_new_error (NULL, s->str);
 			yylval.expr = register_expr_allocation (gnm_expr_new_constant (v));
 			g_string_free (s, TRUE);
 			return CONSTANT;
 		} else {
-			Value *v = value_new_string_nocopy (g_string_free (s, FALSE));
+			GnmValue *v = value_new_string_nocopy (g_string_free (s, FALSE));
 			yylval.expr = register_expr_allocation (gnm_expr_new_constant (v));
 			return QUOTED_STRING;
 		}
@@ -2627,10 +2627,8 @@ yylex (void)
 		return STRING;
 	}
 
-	if (c == '\n' || c == 0)
-		return 0;
-
-	if (c == '<'){
+	switch (c) {
+	case '<':
 		if (*state->ptr == '='){
 			state->ptr++;
 			return eat_space (state, LTE);
@@ -2640,21 +2638,31 @@ yylex (void)
 			return eat_space (state, NE);
 		}
 		return eat_space (state, c);
-	}
 
-	if (c == '>'){
+	case '>':
 		if (*state->ptr == '='){
 			state->ptr++;
 			return eat_space (state, GTE);
 		}
 		return eat_space (state, c);
+
+	case '\n': return 0;
+
+	case 0x00AC: return NOT;
+	case 0x2212: return '-';
+	case 0x2215: return '/';
+	case 0x2227: return AND;
+	case 0x2228: return OR;
+	case 0x2260: return eat_space (state, NE);
+	case 0x2264: return eat_space (state, LTE);
+	case 0x2265: return eat_space (state, GTE);
 	}
 
 	return c;
 }
 
 int
-yyerror (const char *s)
+yyerror (char const *s)
 {
 #if 0
 	printf ("Error: %s\n", s);
@@ -2670,15 +2678,15 @@ yyerror (const char *s)
  * @error       : optionally NULL ptr to store details of error.
  *
  * Parse a string. if @error is non-null it will be assumed that the
- * caller has passed a pointer to a ParseError struct AND that it will
+ * caller has passed a pointer to a GnmParseError struct AND that it will
  * take responsibility for freeing that struct and its contents.
  * with parse_error_free.
  **/
 GnmExpr const *
-gnm_expr_parse_str (char const *expr_text, ParsePos const *pos,
+gnm_expr_parse_str (char const *expr_text, GnmParsePos const *pos,
 		    GnmExprParseFlags flags,
 		    GnmExprConventions *convs,
-		    ParseError *error)
+		    GnmParseError *error)
 {
 	GnmExpr const *expr;
 	ParserState pstate;
@@ -2697,7 +2705,7 @@ gnm_expr_parse_str (char const *expr_text, ParsePos const *pos,
 
 	pstate.decimal_point = convs->decimal_sep_dot
 		? '.'
-		: format_get_decimal ();
+		: g_utf8_get_char (format_get_decimal ()->str); /* FIXME: one char handled.  */
 	pstate.separator = convs->argument_sep_semicolon
 		? ';'
 		: format_get_arg_sep ();
