@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Support for dynamically-loaded Gnumeric plugin components.
  *
@@ -10,7 +11,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "plugin.h"
 
@@ -24,11 +25,11 @@
 #include "plugin-loader.h"
 #include "plugin-loader-module.h"
 #include "plugin-service.h"
+#include "xml-io.h"
 #include "gnumeric-gconf.h"
 
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -46,9 +47,6 @@
 #include <libxml/xmlmemory.h>
 #include <gsf/gsf-impl-utils.h>
 
-#include <gal/util/e-util.h>
-#include <gal/util/e-xml-utils.h>
-
 #include <glib-object.h>
 
 
@@ -67,10 +65,6 @@ static GHashTable *loader_services = NULL;
 
 static void plugin_get_loader_if_needed (GnmPlugin *pinfo, ErrorInfo **ret_error);
 static void plugin_info_read (GnmPlugin *pinfo, const gchar *dir_name, ErrorInfo **ret_error);
-static GnmPlugin *plugin_info_read_for_dir (const gchar *dir_name, ErrorInfo **ret_error);
-static GSList *plugin_info_list_read_for_subdirs_of_dir (const gchar *dir_name, ErrorInfo **ret_error);
-static GSList *plugin_info_list_read_for_subdirs_of_dir_list (GSList *dir_list, ErrorInfo **ret_error);
-static GSList *plugin_info_list_read_for_all_dirs (ErrorInfo **ret_error);
 static void gnm_plugin_load_base (GnmPlugin *plugin, ErrorInfo **ret_error);
 
 /*
@@ -92,13 +86,14 @@ struct _GnmPlugin {
 
 	gchar   *name;
 	gchar   *description;
+	gboolean require_explicit_enabling;
 
 	gboolean is_active;
-	gint use_refcount; 
+	gint use_refcount;
 	GSList *dependencies;
 	gchar *loader_id;
 	GHashTable *loader_attrs;
-	GnumericPluginLoader *loader;
+	GnmPluginLoader *loader;
 	GSList *services;
 
 	char *saved_textdomain;
@@ -133,6 +128,7 @@ gnm_plugin_init (GObject *obj)
 	plugin->dir_name = NULL;
 	plugin->has_full_info = FALSE;
 	plugin->saved_textdomain = NULL;
+	plugin->require_explicit_enabling = FALSE;
 }
 
 static void
@@ -164,7 +160,7 @@ gnm_plugin_finalize (GObject *obj)
 	parent_class->finalize (obj);
 }
 
-gboolean
+static gboolean
 gnm_plugin_type_module_load (GTypeModule *module)
 {
 	GnmPlugin *plugin = GNM_PLUGIN (module);
@@ -182,7 +178,7 @@ gnm_plugin_type_module_load (GTypeModule *module)
 	return TRUE;
 }
 
-void
+static void
 gnm_plugin_type_module_unload (GTypeModule *module)
 {
 	GnmPlugin *plugin = GNM_PLUGIN (module);
@@ -195,7 +191,7 @@ gnm_plugin_type_module_unload (GTypeModule *module)
 static void
 gnm_plugin_class_init (GObjectClass *gobject_class)
 {
-	GTypeModuleClass *type_module_class = G_TYPE_MODULE_CLASS (gobject_class); 
+	GTypeModuleClass *type_module_class = G_TYPE_MODULE_CLASS (gobject_class);
 
 	parent_class = g_type_class_peek_parent (gobject_class);
 
@@ -204,18 +200,16 @@ gnm_plugin_class_init (GObjectClass *gobject_class)
 	type_module_class->load = gnm_plugin_type_module_load;
 	type_module_class->unload = gnm_plugin_type_module_unload;
 
-	gnm_plugin_signals[STATE_CHANGED] =
-	g_signal_new (
-		"state_changed", 
+	gnm_plugin_signals[STATE_CHANGED] = g_signal_new (
+		"state_changed",
 		G_TYPE_FROM_CLASS (gobject_class),
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmPluginClass, state_changed),
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
-	gnm_plugin_signals[CAN_DEACTIVATE_CHANGED] =
-	g_signal_new (
-		"can_deactivate_changed", 
+	gnm_plugin_signals[CAN_DEACTIVATE_CHANGED] = g_signal_new (
+		"can_deactivate_changed",
 		G_TYPE_FROM_CLASS (gobject_class),
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GnmPluginClass, can_deactivate_changed),
@@ -260,7 +254,7 @@ plugin_info_new_with_id_and_dir_name_only (const gchar *id, const gchar *dir_nam
 
 	return plugin;
 }
-              
+
 
 /*
  * PluginFileState - information about plugin.xml files used in previous
@@ -400,7 +394,7 @@ plugin_info_read_full_info_if_needed (GnmPlugin *pinfo)
 const gchar *
 gnm_plugin_get_textdomain (GnmPlugin *plugin)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (plugin), NULL);
+	g_return_val_if_fail (IS_GNM_PLUGIN (plugin), NULL);
 
 	if (plugin->saved_textdomain == NULL) {
 		plugin->saved_textdomain = g_strconcat ("gnumeric__", plugin->id, NULL);
@@ -418,7 +412,7 @@ gnm_plugin_get_textdomain (GnmPlugin *plugin)
 gboolean
 gnm_plugin_is_active (GnmPlugin *plugin)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (IS_GNM_PLUGIN (plugin), FALSE);
 
 	if (!plugin->has_full_info) {
 		return FALSE;
@@ -436,7 +430,7 @@ gnm_plugin_is_active (GnmPlugin *plugin)
 const gchar *
 gnm_plugin_get_dir_name (GnmPlugin *pinfo)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (pinfo), NULL);
+	g_return_val_if_fail (IS_GNM_PLUGIN (pinfo), NULL);
 
 	return pinfo->dir_name;
 }
@@ -452,7 +446,7 @@ gnm_plugin_get_dir_name (GnmPlugin *pinfo)
 const gchar *
 gnm_plugin_get_id (GnmPlugin *pinfo)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (pinfo), NULL);
+	g_return_val_if_fail (IS_GNM_PLUGIN (pinfo), NULL);
 
 	return pinfo->id;
 }
@@ -468,7 +462,7 @@ gnm_plugin_get_id (GnmPlugin *pinfo)
 const gchar *
 gnm_plugin_get_name (GnmPlugin *pinfo)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (pinfo), NULL);
+	g_return_val_if_fail (IS_GNM_PLUGIN (pinfo), NULL);
 
 	if (!plugin_info_read_full_info_if_needed (pinfo)) {
 		return _("Unknown name");
@@ -487,7 +481,7 @@ gnm_plugin_get_name (GnmPlugin *pinfo)
 const gchar *
 gnm_plugin_get_description (GnmPlugin *pinfo)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (pinfo), NULL);
+	g_return_val_if_fail (IS_GNM_PLUGIN (pinfo), NULL);
 
 	if (!plugin_info_read_full_info_if_needed (pinfo)) {
 		return NULL;
@@ -504,13 +498,13 @@ gnm_plugin_get_description (GnmPlugin *pinfo)
 gboolean
 gnm_plugin_is_loaded (GnmPlugin *pinfo)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (pinfo), FALSE);
+	g_return_val_if_fail (IS_GNM_PLUGIN (pinfo), FALSE);
 
 	if (!pinfo->has_full_info) {
 		return FALSE;
 	}
 	return pinfo->loader != NULL &&
-	       gnumeric_plugin_loader_is_base_loaded (pinfo->loader);
+	       gnm_plugin_loader_is_base_loaded (pinfo->loader);
 }
 
 /* - */
@@ -524,10 +518,10 @@ gnm_plugin_is_loaded (GnmPlugin *pinfo)
  * consists of loader's plugin id and service id concatenated using colon).
  * All requests to create new loader object of this type will be passed to
  * @service.
- * This function is intended for use by PluginService objects.
+ * This function is intended for use by GnmPluginService objects.
  */
 void
-plugins_register_loader (const gchar *loader_id, PluginService *service)
+plugins_register_loader (const gchar *loader_id, GnmPluginService *service)
 {
 	g_return_if_fail (loader_id != NULL);
 	g_return_if_fail (service != NULL);
@@ -542,7 +536,7 @@ plugins_register_loader (const gchar *loader_id, PluginService *service)
  * Unregisters a type of plugin loader identified by @loader_id. After
  * callingthis function Gnumeric will be unable to load plugins supported
  * by the specified loader.
- * This function is intended for use by PluginService objects.
+ * This function is intended for use by GnmPluginService objects.
  */
 void
 plugins_unregister_loader (const gchar *loader_id)
@@ -555,7 +549,7 @@ plugins_unregister_loader (const gchar *loader_id)
 static GType
 get_loader_type_by_id (const gchar *id_str, ErrorInfo **ret_error)
 {
-	PluginService *loader_service;
+	GnmPluginService *loader_service;
 	ErrorInfo *error;
 	GType loader_type;
 
@@ -563,9 +557,9 @@ get_loader_type_by_id (const gchar *id_str, ErrorInfo **ret_error)
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
 	if (strcmp (id_str, BUILTIN_LOADER_MODULE_ID) == 0) {
-		return TYPE_GNUMERIC_PLUGIN_LOADER_MODULE;
+		return TYPE_GNM_PLUGIN_LOADER_MODULE;
 	}
-	loader_service = g_hash_table_lookup (loader_services, id_str);	
+	loader_service = g_hash_table_lookup (loader_services, id_str);
 	if (loader_service == NULL) {
 		*ret_error = error_info_new_printf (
 		             _("Unsupported loader type \"%s\"."),
@@ -608,14 +602,15 @@ plugin_info_read_dependency_list (xmlNode *tree)
 		if (strcmp (node->name, "dep_plugin") == 0) {
 			gchar *plugin_id;
 
-			plugin_id = e_xml_get_string_prop_by_name (node, (xmlChar *)"id");
+			plugin_id = xmlGetProp (node, (xmlChar *)"id");
 			if (plugin_id != NULL) {
 				PluginDependency *dep;
 
 				dep = g_new (PluginDependency, 1);
 				dep->plugin_id = plugin_id;
 				dep->plugin = NULL;
-				dep->force_load = e_xml_get_bool_prop_by_name_with_default (node, (xmlChar *)"force_load", FALSE);
+				if (!xml_node_get_bool (node, "force_load", &(dep->force_load)))
+					dep->force_load = FALSE;
 				GNM_SLIST_PREPEND (dependency_list, dep);
 			}
 		}
@@ -625,7 +620,7 @@ plugin_info_read_dependency_list (xmlNode *tree)
 }
 
 static GSList *
-plugin_info_read_service_list (xmlNode *tree, ErrorInfo **ret_error)
+plugin_info_read_service_list (GnmPlugin *plugin, xmlNode *tree, ErrorInfo **ret_error)
 {
 	GSList *service_list = NULL;
 	GSList *error_list = NULL;
@@ -633,15 +628,17 @@ plugin_info_read_service_list (xmlNode *tree, ErrorInfo **ret_error)
 	gint i;
 
 	g_return_val_if_fail (tree != NULL, NULL);
-	g_return_val_if_fail (strcmp (tree->name, "services") == 0, NULL);
 
-	GNM_INIT_RET_ERROR_INFO (ret_error);
-	for (i = 0, node = tree->xmlChildrenNode; node != NULL; i++, node = node->next) {
+	node = e_xml_get_child_by_name (tree, (xmlChar *)"services");
+	if (node == NULL)
+		return NULL;
+	node = node->xmlChildrenNode;
+	for (i = 0; node != NULL; i++, node = node->next) {
 		if (strcmp (node->name, "service") == 0) {
-			PluginService *service;
+			GnmPluginService *service;
 			ErrorInfo *service_error;
 
-			service = plugin_service_new (node, &service_error);
+			service = plugin_service_new (plugin, node, &service_error);
 
 			if (service != NULL) {
 				g_assert (service_error == NULL);
@@ -681,10 +678,10 @@ plugin_info_read_loader_attrs (xmlNode *tree)
 		if (strcmp (node->name, "attribute") == 0) {
 			gchar *name, *value;
 
-			name = e_xml_get_string_prop_by_name (node, (xmlChar *)"name");
+			name = xmlGetProp (node, (xmlChar *)"name");
 			if (name != NULL) {
 				if (g_hash_table_lookup (hash, name) == NULL) {
-					value = e_xml_get_string_prop_by_name (node, (xmlChar *)"value");
+					value = xmlGetProp (node, (xmlChar *)"value");
 					g_hash_table_insert (hash, name, value);
 				} else {
 					g_warning ("Duplicated \"%s\" attribute in plugin.xml file.", name);
@@ -709,19 +706,18 @@ plugin_dependency_free (gpointer data)
 }
 
 static void
-plugin_info_read (GnmPlugin *pinfo, const gchar *dir_name, ErrorInfo **ret_error)
+plugin_info_read (GnmPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error)
 {
 	gchar *file_name;
 	xmlDocPtr doc;
 	gchar *id, *name, *description;
-	xmlNode *tree, *information_node, *dependencies_node, *loader_node, *services_node;
-	ErrorInfo *services_error;
-	GSList *service_list;
+	xmlNode *tree, *information_node, *dependencies_node, *loader_node;
 	GSList *dependency_list;
 	gchar *loader_id;
 	GHashTable *loader_attrs;
+	gboolean require_explicit_enabling = FALSE;
 
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (plugin));
 	g_return_if_fail (dir_name != NULL);
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
@@ -742,30 +738,29 @@ plugin_info_read (GnmPlugin *pinfo, const gchar *dir_name, ErrorInfo **ret_error
 		return;
 	}
 	tree = doc->xmlRootNode;
-	id = e_xml_get_string_prop_by_name (tree, (xmlChar *)"id");
+	id = xmlGetProp (tree, (xmlChar *)"id");
 	information_node = e_xml_get_child_by_name (tree, (xmlChar *)"information");
 	if (information_node != NULL) {
 		xmlNode *node;
 		xmlChar *val;
 
-		node = e_xml_get_child_by_name_by_lang_list (
-		       information_node, "name", NULL);
+		node = e_xml_get_child_by_name_by_lang (information_node, "name");
 		if (node != NULL) {
 			val = xmlNodeGetContent (node);
 			name = g_strdup ((gchar *)val);
 			xmlFree (val);
-		} else {
+		} else
 			name = NULL;
-		}
-		node = e_xml_get_child_by_name_by_lang_list (
-		       information_node, "description", NULL);
+
+		node = e_xml_get_child_by_name_by_lang (information_node, "description");
 		if (node != NULL) {
 			val = xmlNodeGetContent (node);
 			description = g_strdup ((gchar *)val);
 			xmlFree (val);
-		} else {
+		} else
 			description = NULL;
-		}
+		if (e_xml_get_child_by_name (information_node, (xmlChar const *)"require_explicit_enabling"))
+			require_explicit_enabling = TRUE;
 	} else {
 		name = NULL;
 		description = NULL;
@@ -780,7 +775,7 @@ plugin_info_read (GnmPlugin *pinfo, const gchar *dir_name, ErrorInfo **ret_error
 	if (loader_node != NULL) {
 		char *p;
 
-		loader_id = e_xml_get_string_prop_by_name (loader_node, (xmlChar *)"type");
+		loader_id = xmlGetProp (loader_node, (xmlChar *)"type");
 		if (loader_id != NULL && (p = strchr (loader_id, ':')) != NULL) {
 			loader_attrs = plugin_info_read_loader_attrs (loader_node);
 			if (strcmp (loader_id, BUILTIN_LOADER_MODULE_ID) != 0) {
@@ -801,41 +796,38 @@ plugin_info_read (GnmPlugin *pinfo, const gchar *dir_name, ErrorInfo **ret_error
 		loader_id = NULL;
 		loader_attrs = NULL;
 	}
-	services_node = e_xml_get_child_by_name (tree, (xmlChar *)"services");
-	if (services_node != NULL) {
-		service_list = plugin_info_read_service_list (services_node, &services_error);
-	} else {
-		service_list = NULL;
-		services_error = NULL;
-	}
-	if (id != NULL && name != NULL && loader_id != NULL && service_list != NULL &&
+	if (id != NULL && name != NULL && loader_id != NULL &&
 	    id[strspn (id, PLUGIN_ID_VALID_CHARS)] == '\0') {
+		ErrorInfo *services_error = NULL;
 
-		g_assert (services_error == NULL);
+		g_type_module_set_name (G_TYPE_MODULE (plugin), id);
+		plugin->dir_name = g_strdup (dir_name);
+		plugin->id = id;
+		plugin->name = name;
+		plugin->description = description;
+		plugin->require_explicit_enabling = require_explicit_enabling;
+		plugin->is_active = FALSE;
+		plugin->use_refcount = 0;
+		plugin->dependencies = dependency_list;
+		plugin->loader_id = loader_id;
+		plugin->loader_attrs = loader_attrs;
+		plugin->loader = NULL;
+		plugin->services = plugin_info_read_service_list (plugin, tree, &services_error);
 
-		g_type_module_set_name (G_TYPE_MODULE (pinfo), id);
-		pinfo->dir_name = g_strdup (dir_name);
-		pinfo->id = id;
-		pinfo->name = name;
-		pinfo->description = description;
-		pinfo->is_active = FALSE;
-		pinfo->use_refcount = 0;
-		pinfo->dependencies = dependency_list;
-		pinfo->loader_id = loader_id;
-		pinfo->loader_attrs = loader_attrs;
-		pinfo->loader = NULL;
-		pinfo->services = service_list;
-		GNM_SLIST_FOREACH (pinfo->services, PluginService, service,
-			plugin_service_set_plugin (service, pinfo);
-		);
-		plugin_message (4, "Read plugin.xml file for %s.\n", pinfo->id);
+		if (services_error != NULL) {
+			*ret_error = error_info_new_printf (
+				_("Errors while reading services for plugin with id=\"%s\"."),
+				id);
+			error_info_add_details (*ret_error, services_error);
+		} else if (plugin->services == NULL)
+			*ret_error = error_info_new_printf (
+				_("No services defined for plugin with id=\"%s\"."),
+				id);
+		else
+			plugin_message (4, "Read plugin.xml file for %s.\n", plugin->id);
 	} else {
-		if (id == NULL) {
-			*ret_error = error_info_new_str (_("Plugin has no id."));
-			error_info_free (services_error);
-		} else {
+		if (id != NULL) {
 			GSList *error_list = NULL;
-			ErrorInfo *error;
 
 			if (id[strspn (id, PLUGIN_ID_VALID_CHARS)] != '\0') {
 				GNM_SLIST_PREPEND (error_list, error_info_new_printf (
@@ -849,30 +841,16 @@ plugin_info_read (GnmPlugin *pinfo, const gchar *dir_name, ErrorInfo **ret_error
 				GNM_SLIST_PREPEND (error_list, error_info_new_printf (
 					_("No loader defined or loader id invalid for plugin with id=\"%s\"."), id));
 			}
-			if (service_list == NULL) {
-				if (services_error != NULL) {
-					error = error_info_new_printf (
-					        _("Errors while reading services for plugin with id=\"%s\"."),
-					        id);
-					error_info_add_details (error, services_error);
-				} else {
-					error = error_info_new_printf (
-					        _("No services defined for plugin with id=\"%s\"."),
-					        id);
-				}
-				GNM_SLIST_PREPEND (error_list, error);
-			}
 			g_assert (error_list != NULL);
 			GNM_SLIST_REVERSE (error_list);
 			*ret_error = error_info_new_from_error_list (error_list);
-		}
+		} else
+			*ret_error = error_info_new_str (_("Plugin has no id."));
 
 		g_slist_free_custom (dependency_list, plugin_dependency_free);
-		g_free (pinfo->loader_id);
-		if (pinfo->loader_attrs != NULL) {
-			g_hash_table_destroy (pinfo->loader_attrs);
-		}
-		g_slist_free_custom (service_list, g_object_unref);
+		g_free (plugin->loader_id);
+		if (plugin->loader_attrs != NULL)
+			g_hash_table_destroy (plugin->loader_attrs);
 		g_free (id);
 		g_free (name);
 		g_free (description);
@@ -887,7 +865,7 @@ plugin_get_loader_if_needed (GnmPlugin *pinfo, ErrorInfo **ret_error)
 	GType loader_type;
 	ErrorInfo *error;
 
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (pinfo));
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
 	if (!plugin_info_read_full_info_if_needed_error_info (pinfo, ret_error)) {
@@ -898,14 +876,14 @@ plugin_get_loader_if_needed (GnmPlugin *pinfo, ErrorInfo **ret_error)
 	}
 	loader_type = get_loader_type_by_id (pinfo->loader_id, &error);
 	if (error == NULL) {
-		GnumericPluginLoader *loader;
+		GnmPluginLoader *loader;
 		ErrorInfo *error;
 
-		loader = GNUMERIC_PLUGIN_LOADER (g_object_new (loader_type, NULL));
-		gnumeric_plugin_loader_set_attributes (loader, pinfo->loader_attrs, &error);
+		loader = GNM_PLUGIN_LOADER (g_object_new (loader_type, NULL));
+		gnm_plugin_loader_set_attributes (loader, pinfo->loader_attrs, &error);
 		if (error == NULL) {
 			pinfo->loader = loader;
-			gnumeric_plugin_loader_set_plugin (loader, pinfo);
+			gnm_plugin_loader_set_plugin (loader, pinfo);
 		} else {
 			g_object_unref (loader);
 			loader = NULL;
@@ -936,7 +914,7 @@ gnm_plugin_activate (GnmPlugin *pinfo, ErrorInfo **ret_error)
 	gint i;
 	static GSList *activate_stack = NULL;
 
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (pinfo));
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
 	if (g_slist_find (activate_stack, pinfo) != NULL) {
@@ -984,7 +962,7 @@ gnm_plugin_activate (GnmPlugin *pinfo, ErrorInfo **ret_error)
 	}
 
 	for (l = pinfo->services, i = 0; l != NULL; l = l->next, i++) {
-		PluginService *service = l->data;
+		GnmPluginService *service = l->data;
 		ErrorInfo *service_error;
 
 		plugin_service_activate (service, &service_error);
@@ -1026,7 +1004,7 @@ gnm_plugin_deactivate (GnmPlugin *pinfo, ErrorInfo **ret_error)
 	GSList *l;
 	gint i;
 
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (pinfo));
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
 	if (!pinfo->has_full_info || !pinfo->is_active) {
@@ -1037,7 +1015,7 @@ gnm_plugin_deactivate (GnmPlugin *pinfo, ErrorInfo **ret_error)
 		return;
 	}
 	for (l = pinfo->services, i = 0; l != NULL; l = l->next, i++) {
-		PluginService *service = l->data;
+		GnmPluginService *service = l->data;
 		ErrorInfo *service_error;
 
 		plugin_service_deactivate (service, &service_error);
@@ -1077,7 +1055,7 @@ gnm_plugin_deactivate (GnmPlugin *pinfo, ErrorInfo **ret_error)
 gboolean
 gnm_plugin_can_deactivate (GnmPlugin *pinfo)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (pinfo), FALSE);
+	g_return_val_if_fail (IS_GNM_PLUGIN (pinfo), FALSE);
 
 	if (!pinfo->is_active) {
 		return FALSE;
@@ -1156,7 +1134,7 @@ gnm_plugin_load_base (GnmPlugin *plugin, ErrorInfo **ret_error)
 		return;
 	}
 
-	gnumeric_plugin_loader_load_base (plugin->loader, &error);
+	gnm_plugin_loader_load_base (plugin->loader, &error);
 	if (error != NULL) {
 		*ret_error = error;
 		return;
@@ -1172,12 +1150,12 @@ gnm_plugin_load_base (GnmPlugin *plugin, ErrorInfo **ret_error)
  *
  * Loads base part of the plugin if is not loaded and then loads given
  * plugin service (prepares necessary part of the plugin for direct use).
- * This function is intended for use by PluginService objects.
+ * This function is intended for use by GnmPluginService objects.
  */
 void
-gnm_plugin_load_service (GnmPlugin *pinfo, PluginService *service, ErrorInfo **ret_error)
+gnm_plugin_load_service (GnmPlugin *pinfo, GnmPluginService *service, ErrorInfo **ret_error)
 {
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (pinfo));
 	g_return_if_fail (service != NULL);
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
@@ -1185,7 +1163,7 @@ gnm_plugin_load_service (GnmPlugin *pinfo, PluginService *service, ErrorInfo **r
 	if (*ret_error != NULL) {
 		return;
 	}
-	gnumeric_plugin_loader_load_service (pinfo->loader, service, ret_error);
+	gnm_plugin_loader_load_service (pinfo->loader, service, ret_error);
 }
 
 /**
@@ -1195,12 +1173,12 @@ gnm_plugin_load_service (GnmPlugin *pinfo, PluginService *service, ErrorInfo **r
  * @ret_error   : Pointer used to report errors
  *
  * ...
- * This function is intended for use by PluginService objects.
+ * This function is intended for use by GnmPluginService objects.
  */
 void
-gnm_plugin_unload_service (GnmPlugin *pinfo, PluginService *service, ErrorInfo **ret_error)
+gnm_plugin_unload_service (GnmPlugin *pinfo, GnmPluginService *service, ErrorInfo **ret_error)
 {
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (pinfo));
 	g_return_if_fail (pinfo->loader != NULL);
 	g_return_if_fail (service != NULL);
 
@@ -1208,7 +1186,7 @@ gnm_plugin_unload_service (GnmPlugin *pinfo, PluginService *service, ErrorInfo *
 	if (!plugin_info_read_full_info_if_needed_error_info (pinfo, ret_error)) {
 		return;
 	}
-	gnumeric_plugin_loader_unload_service (pinfo->loader, service, ret_error);
+	gnm_plugin_loader_unload_service (pinfo->loader, service, ret_error);
 }
 
 /**
@@ -1218,7 +1196,7 @@ gnm_plugin_unload_service (GnmPlugin *pinfo, PluginService *service, ErrorInfo *
 void
 gnm_plugin_use_ref (GnmPlugin *plugin)
 {
-	g_return_if_fail (GNM_IS_PLUGIN (plugin));
+	g_return_if_fail (IS_GNM_PLUGIN (plugin));
 	g_return_if_fail (plugin->is_active);
 
 	plugin->use_refcount++;
@@ -1234,7 +1212,7 @@ gnm_plugin_use_ref (GnmPlugin *plugin)
 void
 gnm_plugin_use_unref (GnmPlugin *plugin)
 {
-	g_return_if_fail (GNM_IS_PLUGIN (plugin));
+	g_return_if_fail (IS_GNM_PLUGIN (plugin));
 	g_return_if_fail (plugin->is_active);
 	g_return_if_fail (plugin->use_refcount > 0);
 
@@ -1274,7 +1252,7 @@ gnm_plugin_get_dependencies_ids (GnmPlugin *plugin)
 GSList *
 gnm_plugin_get_services (GnmPlugin *plugin)
 {
-	g_return_val_if_fail (GNM_IS_PLUGIN (plugin), NULL);
+	g_return_val_if_fail (IS_GNM_PLUGIN (plugin), NULL);
 
 	return plugin->services;
 }
@@ -1420,8 +1398,8 @@ gnumeric_extra_plugin_dirs (void)
 {
 	GSList *extra_dirs;
 	gchar const *plugin_path_env;
-	
-	extra_dirs = gnm_app_prefs->plugin_extra_dirs;
+
+	extra_dirs = g_string_slist_copy (gnm_app_prefs->plugin_extra_dirs);
 	plugin_path_env = g_getenv ("GNUMERIC_PLUGIN_PATH");
 	if (plugin_path_env != NULL) {
 		GNM_SLIST_CONCAT (extra_dirs, g_strsplit_to_slist (plugin_path_env, ":"));
@@ -1446,7 +1424,8 @@ plugin_info_list_read_for_all_dirs (ErrorInfo **ret_error)
 				   NULL);
 	GNM_SLIST_CONCAT (dir_list, gnumeric_extra_plugin_dirs ());
 	plugin_info_list = plugin_info_list_read_for_subdirs_of_dir_list (dir_list, &error);
-	e_free_string_slist (dir_list);
+	g_slist_foreach (dir_list, (GFunc)g_free, NULL);
+	g_slist_free (dir_list);
 	*ret_error = error;
 
 	return plugin_info_list;
@@ -1558,7 +1537,7 @@ plugins_get_plugin_by_id (const gchar *plugin_id)
 void
 plugin_db_mark_plugin_for_deactivation (GnmPlugin *pinfo, gboolean mark)
 {
-	g_return_if_fail (GNM_IS_PLUGIN (pinfo));
+	g_return_if_fail (IS_GNM_PLUGIN (pinfo));
 
 	if (mark) {
 		if (plugins_marked_for_deactivation_hash == NULL) {
@@ -1598,7 +1577,7 @@ ghf_set_state_old_unused (gpointer key, gpointer value, gpointer unused)
  *
  *
  */
-void 
+void
 plugins_rescan (ErrorInfo **ret_error, GSList **ret_new_plugins)
 {
 	GSList *error_list = NULL;
@@ -1693,29 +1672,29 @@ plugins_rescan (ErrorInfo **ret_error, GSList **ret_new_plugins)
 }
 
 static void
-ghf_collect_new_plugin_ids (gpointer key, gpointer value, gpointer user_data)
+ghf_collect_new_plugins (gpointer ignored,
+			 PluginFileState *s, GSList **plugin_list)
 {
-	PluginFileState *state = value;
-	GSList **new_ids = user_data;
-
-	if (state->age == PLUGIN_NEW) {
-		GNM_SLIST_PREPEND (*new_ids, g_strdup (state->plugin_id));
+	if (s->age == PLUGIN_NEW) {
+		GnmPlugin *plugin = plugins_get_plugin_by_id (s->plugin_id);
+		if (plugin != NULL && !plugin->require_explicit_enabling)
+			GNM_SLIST_PREPEND (*plugin_list, plugin);
 	}
 }
 
 /**
  * plugins_init:
- * @context     : CommandContext used to report errors
+ * @context     : #GnmCmdContext used to report errors
  *
  * Initializes the plugin subsystem. Don't call this function more than
  * once.
  */
 void
-plugins_init (CommandContext *context)
+plugins_init (GnmCmdContext *context)
 {
 	GSList *error_list = NULL;
 	ErrorInfo *error;
-	GSList *saved_active_ids, *plugin_list, *state_str_list;
+	GSList *plugin_list;
 
 	gnumeric_time_counter_push ();
 
@@ -1723,16 +1702,13 @@ plugins_init (CommandContext *context)
 
 	/* initialize hash table with information about known plugin.xml files */
 	plugin_file_state_dir_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, plugin_file_state_free);
-	state_str_list = gnm_app_prefs->plugin_file_states;
-	GNM_SLIST_FOREACH (state_str_list, char, state_str,
+	GNM_SLIST_FOREACH (gnm_app_prefs->plugin_file_states, char, state_str,
 		PluginFileState *state;
 
 		state = plugin_file_state_from_string (state_str);
-		if (state != NULL) {
+		if (state != NULL)
 			g_hash_table_insert (plugin_file_state_dir_hash, state->dir_name, state);
-		}
 	);
-	g_slist_free_custom (state_str_list, g_free);
 	plugin_file_state_hash_changed = FALSE;
 
 	/* collect information about the available plugins */
@@ -1748,24 +1724,21 @@ plugins_init (CommandContext *context)
 			_("Errors while reading info about available plugins."), error));
 	}
 
-	/* activate all previously active and (optionally) new plugins */
-	saved_active_ids = gnm_app_prefs->active_plugins;
-	if (gnm_app_prefs->activate_new_plugins) {
+	/* get descriptors for all previously active plugins */
+	plugin_list = NULL;
+	GNM_SLIST_FOREACH (gnm_app_prefs->active_plugins, char, plugin_id,
+		GnmPlugin *plugin = plugins_get_plugin_by_id (plugin_id);
+		if (plugin != NULL)
+			GNM_SLIST_PREPEND (plugin_list, plugin);
+	);
+
+	/* get descriptors for new plugins */
+	if (gnm_app_prefs->activate_new_plugins)
 		g_hash_table_foreach (
 			plugin_file_state_dir_hash,
-			ghf_collect_new_plugin_ids,
-			&saved_active_ids);
-	}
-	plugin_list = NULL;
-	GNM_SLIST_FOREACH (saved_active_ids, char, plugin_id,
-		GnmPlugin *plugin;
+			(GHFunc) ghf_collect_new_plugins,
+			&plugin_list);
 
-		plugin = plugins_get_plugin_by_id (plugin_id);
-		if (plugin != NULL) {
-			GNM_SLIST_PREPEND (plugin_list, plugin);
-		}
-	);
-	g_slist_free_custom (saved_active_ids, g_free);
 	plugin_list = g_slist_reverse (plugin_list);
 	plugin_db_activate_plugin_list (plugin_list, &error);
 	g_slist_free (plugin_list);
@@ -1781,7 +1754,7 @@ plugins_init (CommandContext *context)
 		        _("Errors while initializing plugin system."),
 		        error_list);
 
-		gnumeric_error_error_info (context, error);
+		gnm_cmd_context_error_info (context, error);
 		error_info_free (error);
 	}
 
@@ -1799,6 +1772,24 @@ ghf_collect_used_plugin_state_strings (gpointer key, gpointer value, gpointer us
 	}
 }
 
+/**
+ * gnm_plugin_try_unref
+ *
+ * Unref plugin object if it is legal to destroy it. Destruction is
+ * not legal if a type or interface has been registered for it. "Once
+ * a GTypeModule is initialized, it must exist forever" - docs of
+ * g_type_module_unuse().
+ */
+static void
+gnm_plugin_try_unref (gpointer plugin)
+{
+	GTypeModule *module = G_TYPE_MODULE (plugin);
+
+	if (!module->type_infos && !module->interface_infos) {
+		g_object_unref (plugin);
+	}
+}
+
 /**
  * plugins_shutdown:
  *
@@ -1841,13 +1832,13 @@ plugins_shutdown (void)
 	    g_hash_table_size (plugin_file_state_dir_hash) != g_slist_length (used_plugin_state_strings)) {
 		gnm_gconf_set_plugin_file_states (used_plugin_state_strings);
 		plugin_message (5, "Plugin cache changed\n");
-	}
-	g_slist_free_custom (used_plugin_state_strings, g_free);
+	} else
+		g_slist_free_custom (used_plugin_state_strings, g_free);
 
 	g_hash_table_destroy (plugin_file_state_dir_hash);
 	g_hash_table_destroy (loader_services);
 	g_hash_table_destroy (available_plugins_id_hash);
-	g_slist_free_custom (available_plugins, g_object_unref);
+	g_slist_free_custom (available_plugins, gnm_plugin_try_unref);
 
 	gnm_conf_sync ();
 }
