@@ -1,10 +1,10 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * print-cell.c: Printing of cell regions and cells.
  *
  * Author:
+ *    Jody Goldberg 2000-2002	(jody@gnome.org)
  *    Miguel de Icaza 1999 (miguel@kernel.org)
- *
- * i18n of printing: Copyright 2001 by Vlad Harchev <hvv@hippo.ru>
  */
 #include <gnumeric-config.h>
 #include "gnumeric.h"
@@ -26,21 +26,9 @@
 #include "rendered-value.h"
 #include "str.h"
 
-#include <ctype.h>
+#include <string.h>
 #include <locale.h>
 
-/*
-  Define this to enable i18n-wise printing and string measuring - it requires
-  mbstowcs to be available. Most probably printing work fine for ANY locale
-  (though gnome-print doesn't support CJK yet - but when it will be ready, no
-  changes will be needed in the code used when _PROPER_I18N is defined.
-
-  If this macro is undefined, printing will work only for iso-8859-1, so please
-  try hard to avoid undefining it.
-      - Vlad Harchev <hvv@hippo.ru>
-*/
-#define _PROPER_I18N
-
 #if 0
 #define MERGE_DEBUG(range, str) do { range_dump (range, str); } while (0)
 #else
@@ -56,94 +44,6 @@ print_hline (GnomePrintContext *context,
 	gnome_print_stroke (context);
 }
 
-int
-print_show (GnomePrintContext *pc, char const *text)
-{
-#ifdef _PROPER_I18N
-	wchar_t* wcs,wcbuf[4096];
-	char* utf8,utf8buf[4096];
-
-	size_t conv_status;
-	int n = strlen (text);
-	int retval;
-	const int wcbuf_len = sizeof (wcbuf) / sizeof (wcbuf[0]);
-
-	g_return_val_if_fail (pc && text, -1);
-
-	if ( n > wcbuf_len)
-		wcs = g_new (wchar_t,n);
-	else
-		wcs = wcbuf;
-
-	conv_status = mbstowcs (wcs, text, n);
-
-	if (conv_status == (size_t)(-1)){
-		if (wcs != wcbuf)
-			g_free (wcs);
-		return 0;
-	};
-	if (conv_status * 6 > sizeof (utf8buf))
-		utf8 = g_new (gchar, conv_status * 6);
-	else
-		utf8 = utf8buf;
-
-	{
-		size_t i;
-		char* p = utf8;
-		for(i = 0; i < conv_status; ++i)
-			p += g_unichar_to_utf8 ( (gint) wcs[i], p);
-		if (wcs != wcbuf)
-			g_free (wcs);
-		retval = gnome_print_show_sized (pc, utf8, p - utf8);
-	}
-
-	if (utf8 != utf8buf)
-		g_free (utf8);
-	return retval;
-#else
-	return print_show_iso8859_1 (pc, text);
-#endif
-};
-
-double
-get_width_string_n (GnomeFont *font, char const* text, guint n)
-{
-#ifdef _PROPER_I18N
-	wchar_t* wcs, wcbuf[4000];
-	size_t conv_status, i;
-	double total = 0;
-
-	if ( n > (sizeof(wcbuf)/sizeof(wcbuf[0])))
-		wcs = g_new (wchar_t,n);
-	else
-		wcs = wcbuf;
-
-	conv_status = mbstowcs (wcs, text, n);
-
-	if (conv_status == (size_t)(-1)){
-		if (wcs != wcbuf)
-			g_free (wcs);
-		return 0;
-	};
-	for (i = 0; i < conv_status; ++i)
-		total += gnome_font_get_glyph_width (font,
-				gnome_font_lookup_default (font, wcs[i]));
-
-	if (wcs != wcbuf)
-		g_free (wcs);
-	return total;
-#else
-	return gnome_font_get_width_string_n (font, text, n);
-#endif
-};
-
-
-double
-get_width_string (GnomeFont *font, char const* text)
-{
-	return get_width_string_n (font, text, strlen(text));
-};
-
 /***********************************************************/
 
 /*
@@ -154,11 +54,11 @@ get_width_string (GnomeFont *font, char const* text)
 
 static inline void
 print_text (GnomePrintContext *context,
-	    double x, double text_base, char const * text, double len_pts,
+	    double x, double text_base, char const *text, double len_pts,
 	    double const * const line_offset, int num_lines)
 {
 	gnome_print_moveto (context, x, text_base);
-	print_show (context, text);
+	gnome_print_show (context, text);
 
 	/* FIXME how to handle small fonts ?
 	 * the text_base should be at least 2 pixels above the bottom */
@@ -170,47 +70,64 @@ print_text (GnomePrintContext *context,
 	}
 }
 
+static char const hashes[] =
+"################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################";
+
 static void
 print_overflow (GnomePrintContext *context, GnomeFont *font,
 		double x1, double text_base, double width,
 		double const * const line_offset, int num_lines)
 {
-	double const len = get_width_string_n (font, "#", 1);
-	int count = 0;
+	double const len = gnome_font_get_width_utf8_sized (font, "##########", 10) / 10;
+	unsigned count = 1;
 
-	if (len != 0)  {
+	if (len != 0)
 		count = width / len;
-		if (count == 0)
-			count = 1;
+	if (count == 0)
+		count = 1;
+	else if (count >= sizeof (hashes))
+		count = sizeof (hashes) - 1;
+	x1 += (width - count*len) / 2; /* Center */
+	print_text (context, x1, text_base, hashes + sizeof (hashes) - count - 1,
+		    count*len, line_offset, num_lines);
+}
+
+static GList *
+cell_split_text_no_wrap (char const *text)
+{
+	GList *lines = NULL;
+	gchar const *next = text;
+
+	while (next != NULL) {
+		gchar const *this = next;
+		gchar *str;
+
+		next = g_utf8_strchr (next, -1, '\n');
+		if (next == NULL)
+			str = g_strdup (this);
+		else {
+			str = g_strndup (this, next - this);
+			next++;
+		}
+		lines = g_list_append (lines, str);
 	}
-
-	/* Center */
-	for (x1 += (width - count*len) / 2; --count >= 0 ; x1 += len )
-		print_text (context, x1, text_base, "#", len,
-			   line_offset, num_lines);
+	return lines;
 }
 
-/*
- * WARNING : This code is an almost exact duplicate of
- *          cell-draw.c:cell_split_text
- * and is very similar to
- *          rendered-value.c:rendered_value_calc_size_ext
- *
- * Try to keep it that way.
- */
 static GList *
 cell_split_text (GnomeFont *font, char const *text, int const width)
 {
-	char const *p, *line_begin;
+	char const *p, *next, *line_begin;
 	char const *first_whitespace = NULL;
 	char const *last_whitespace = NULL;
 	gboolean prev_was_space = FALSE;
 	GList *list = NULL;
 	double used = 0., used_last_space = 0.;
+	double len_current;
 
-	for (line_begin = p = text; *p; p++) {
-		double const len_current =
-			get_width_string_n (font, p, 1);
+	for (line_begin = p = text; *p; p = next) {
+		next = g_utf8_next_char (p);
+		len_current = gnome_font_get_width_utf8_sized (font, p, next - p);
 
 		/* Wrap if there is an embeded newline, or we have overflowed */
 		if (*p == '\n' || used + len_current > width) {
@@ -246,7 +163,7 @@ cell_split_text (GnomeFont *font, char const *text, int const width)
 			last_whitespace = p;
 			first_whitespace = p+1;
 			prev_was_space = TRUE;
-		} else if (isspace (*(unsigned char *)p)) {
+		} else if (g_unichar_isspace (g_utf8_get_char (p))) {
 			used_last_space = used;
 			last_whitespace = p;
 			if (!prev_was_space)
@@ -300,16 +217,16 @@ print_make_rectangle_path (GnomePrintContext *pc,
  *      \------/
  */
 static void
-print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
+print_cell (GnmCell const *cell, GnmStyle const *mstyle, GnomePrintContext *context,
 	    double x1, double y1, double width, double height, double h_center)
 {
-	StyleFont *style_font = mstyle_get_font (mstyle, 1.0);
+	Sheet const * const sheet = cell->base.sheet;
+	GnmFont *style_font = mstyle_get_font (mstyle, sheet->context, 1.0);
 	GnomeFont *print_font = style_font->gnome_print_font;
 	double const font_descent = gnome_font_get_descender (print_font);
 	double const font_ascent = gnome_font_get_ascender (print_font);
 	double rect_x, rect_width, rect_y, rect_height;
 
-	Sheet const * const sheet = cell->base.sheet;
 	ColRowInfo const * const ci = cell->col_info; /* DEPRECATED */
 	ColRowInfo const * const ri = cell->row_info; /* DEPRECATED */
 	double text_base;
@@ -319,25 +236,25 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 	int num_lines = 0;
 	double line_offset [3]; /* There are up to 3 lines, double underlined strikethroughs */
 	char const *text;
-	StyleColor *fore;
+	const PangoColor *fore;
 	double cell_width_pts, indent = 0.;
 
+	gboolean is_multi_line_text;
+
 	/* Don't print zeros if they should be ignored. */
 	if (sheet && sheet->hide_zero && cell_is_zero (cell) &&
 	    (!sheet->display_formulas || !cell_has_expr (cell)))
 		return;
 
-	if (cell->rendered_value == NULL)
-		cell_render_value ((Cell *)cell, TRUE);
-
-	g_return_if_fail (cell->rendered_value->rendered_text);
-
-	if (cell->rendered_value->rendered_text->str == NULL) {
+	if (cell->rendered_value == NULL) {
 		g_warning ("Serious cell error at '%s'", cell_name (cell));
 		/* This can occur when eg. a plugin function fires up a dialog */
-		text = "Pending";
-	} else
-		text = cell->rendered_value->rendered_text->str;
+		return;
+	}
+
+	text = rendered_value_get_text (cell->rendered_value);
+	fore = cell_get_render_color (cell);
+	g_return_if_fail (fore != NULL); /* Be extra careful */
 
 	/* Get the sizes exclusive of margins and grids */
 	/* FIXME : all callers will eventually pass in their cell size */
@@ -404,10 +321,6 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 	gnome_print_clip (context);
 
 	/* Set the font colour */
-	fore = cell->rendered_value->render_color;
-	if (fore == NULL)
-		fore = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
-	g_return_if_fail (fore != NULL); /* Be extra careful */
 	gnome_print_setrgbcolor (context,
 				 fore->red   / (double) 0xffff,
 				 fore->green / (double) 0xffff,
@@ -431,7 +344,7 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 
 	/* FIXME : This will be wrong for JUSTIFIED halignments */
 	halign = style_default_halign (mstyle, cell);
-	cell_width_pts = get_width_string (print_font, text);
+	cell_width_pts = gnome_font_get_width_utf8 (print_font, text);
 	if (halign == HALIGN_LEFT || halign == HALIGN_RIGHT) {
 		/* 2*width seems to be pretty close to XL's notion */
 		/* FIXME: Why use digit?  */
@@ -452,8 +365,11 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 	if (halign == HALIGN_CENTER_ACROSS_SELECTION || h_center <= 0.)
 		h_center = width / 2.;
 
+	is_multi_line_text = mstyle_get_wrap_text (mstyle) 
+		|| (NULL != g_utf8_strchr (text, -1, '\n')); 
+	
 	if (halign != HALIGN_JUSTIFY && valign != VALIGN_JUSTIFY &&
-	    !mstyle_get_wrap_text (mstyle)) {
+	    !is_multi_line_text) {
 		double x, total, len = cell_width_pts;
 
 		switch (halign) {
@@ -472,7 +388,7 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 			break;
 
 		default:
-			g_warning ("Single-line justitfication style not supported.");
+			g_warning ("Single-line justification style not supported.");
 			x = rect_x;
 			break;
 		}
@@ -491,7 +407,9 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 		int line_count;
 		double x, y_offset, inter_space;
 
-		lines = cell_split_text (print_font, text, width);
+		lines = mstyle_get_wrap_text (mstyle) ? 
+			cell_split_text (print_font, text, width) :
+			cell_split_text_no_wrap (text);
 		line_count = g_list_length (lines);
 
 		switch (valign) {
@@ -549,17 +467,17 @@ print_cell (Cell const *cell, MStyle const *mstyle, GnomePrintContext *context,
 				/* Be cheap, only calculate the width of the
 				 * string if we need to. */
 				if (num_lines > 0)
-					len = get_width_string (print_font, str);
+					len = gnome_font_get_width_utf8 (print_font, str);
 				break;
 
 			case HALIGN_RIGHT:
-				len = get_width_string (print_font, str);
+				len = gnome_font_get_width_utf8 (print_font, str);
 				x = rect_x + rect_width - 1 - len - indent;
 				break;
 
 			case HALIGN_CENTER:
 			case HALIGN_CENTER_ACROSS_SELECTION:
-				len = get_width_string (print_font, str);
+				len = gnome_font_get_width_utf8 (print_font, str);
 				x = rect_x + h_center - len / 2;
 			}
 
@@ -597,7 +515,7 @@ print_rectangle (GnomePrintContext *context,
 
 static void
 print_cell_background (GnomePrintContext *context,
-		       MStyle const *style, int col, int row,
+		       GnmStyle const *style, int col, int row,
 		       float x, float y, float w, float h)
 {
 	if (gnumeric_background_set_pc (style, context))
@@ -615,11 +533,11 @@ print_cell_background (GnomePrintContext *context,
 static void
 print_merged_range (GnomePrintContext *context, Sheet const *sheet,
 		    double start_x, double start_y,
-		    Range const *view, Range const *range)
+		    GnmRange const *view, GnmRange const *range)
 {
 	float l, r, t, b;
-	Cell  const *cell    = sheet_cell_get (sheet, range->start.col, range->start.row);
-	MStyle const *mstyle = sheet_style_get (sheet, range->start.col, range->start.row);
+	GnmCell  const *cell    = sheet_cell_get (sheet, range->start.col, range->start.row);
+	GnmStyle const *mstyle = sheet_style_get (sheet, range->start.col, range->start.row);
 
 	l = sheet_col_get_distance_pts (sheet,
 		view->start.col, range->start.col) + start_x;
@@ -648,14 +566,14 @@ print_merged_range (GnomePrintContext *context, Sheet const *sheet,
 }
 
 static gint
-merged_col_cmp (Range const *a, Range const *b)
+merged_col_cmp (GnmRange const *a, GnmRange const *b)
 {
 	return a->start.col - b->start.col;
 }
 
 void
 print_cell_range (GnomePrintContext *context,
-		  Sheet const *sheet, Range *range,
+		  Sheet const *sheet, GnmRange *range,
 		  double base_x, double base_y,
 		  gboolean hide_grid)
 {
@@ -664,13 +582,13 @@ print_cell_range (GnomePrintContext *context,
 	ColRowInfo const *ri = NULL, *next_ri = NULL;
 	int start_row, start_col, end_col, end_row;
 
-	StyleRow sr, next_sr;
-	MStyle const **styles;
-	StyleBorder const **borders, **prev_vert;
-	StyleBorder const *none =
+	GnmRow sr, next_sr;
+	GnmStyle const **styles;
+	GnmBorder const **borders, **prev_vert;
+	GnmBorder const *none =
 		hide_grid ? NULL : style_border_none ();
 
-	Range     view;
+	GnmRange     view;
 	GSList	 *merged_active, *merged_active_seen,
 		 *merged_used, *merged_unused, *ptr, **lag;
 
@@ -706,8 +624,8 @@ print_cell_range (GnomePrintContext *context,
 
 	/*
 	 * allocate a single blob of memory for all 8 arrays of pointers.
-	 * 	- 6 arrays of n StyleBorder const *
-	 * 	- 2 arrays of n MStyle const *
+	 * 	- 6 arrays of n GnmBorder const *
+	 * 	- 2 arrays of n GnmStyle const *
 	 *
 	 * then alias the arrays for easy access so that array [col] is valid
 	 * for all elements start_col-1 .. end_col+1 inclusive.
@@ -751,12 +669,18 @@ print_cell_range (GnomePrintContext *context,
 			}
 		}
 
+		/* it is safe to const_cast because only the a non-default row
+		 * will ever get flagged.
+		 */
+		if (ri->needs_respan)
+			row_calc_spans ((ColRowInfo *)ri, sheet);
+
 		/* look for merges that start on this row, on the first painted row
 		 * also check for merges that start above. */
 		view.start.row = row;
 		lag = &merged_unused;
 		for (ptr = merged_unused; ptr != NULL; ) {
-			Range * const r = ptr->data;
+			GnmRange * const r = ptr->data;
 
 			if (r->start.row <= row) {
 				GSList *tmp = ptr;
@@ -784,13 +708,13 @@ print_cell_range (GnomePrintContext *context,
 		}
 
 		for (col = start_col, x = base_x; col <= end_col ; col++) {
-			MStyle const *style;
+			GnmStyle const *style;
 			CellSpanInfo const *span;
 			ColRowInfo const *ci = sheet_col_get_info (sheet, col);
 
 			if (!ci->visible) {
 				if (merged_active != NULL) {
-					Range const *r = merged_active->data;
+					GnmRange const *r = merged_active->data;
 					if (r->end.col == col) {
 						ptr = merged_active;
 						merged_active = merged_active->next;
@@ -810,7 +734,7 @@ print_cell_range (GnomePrintContext *context,
 
 			/* Skip any merged regions */
 			if (merged_active) {
-				Range const *r = merged_active->data;
+				GnmRange const *r = merged_active->data;
 				if (r->start.col <= col) {
 					gboolean clear_top, clear_bottom = TRUE;
 					int i, first = r->start.col;
@@ -869,8 +793,8 @@ print_cell_range (GnomePrintContext *context,
 			if (ri->pos == -1 || NULL == (span = row_span_get (ri, col))) {
 
 				/* no need to draw blanks */
-				Cell const *cell = sheet_cell_get (sheet, col, row);
-				if (!cell_is_blank (cell))
+				GnmCell const *cell = sheet_cell_get (sheet, col, row);
+				if (!cell_is_empty (cell))
 					print_cell (cell, style, context,
 						    x, y, -1., -1., -1.);
 
@@ -879,7 +803,7 @@ print_cell_range (GnomePrintContext *context,
 			 * to draw the edit cell, or blanks.
 			 */
 			} else if (col == span->right || col == end_col) {
-				Cell const *cell = span->cell;
+				GnmCell const *cell = span->cell;
 				int const start_span_col = span->left;
 				int const end_span_col = span->right;
 				double real_x = x;
@@ -924,7 +848,7 @@ print_cell_range (GnomePrintContext *context,
 
 		/* In case there were hidden merges that trailed off the end */
 		while (merged_active != NULL) {
-			Range const *r = merged_active->data;
+			GnmRange const *r = merged_active->data;
 			ptr = merged_active;
 			merged_active = merged_active->next;
 			if (r->end.row <= row) {
@@ -949,12 +873,8 @@ print_cell_range (GnomePrintContext *context,
 	style_borders_row_print (prev_vert, &sr,
 				 context, base_x, y, y, sheet, FALSE);
 
-	if (merged_used)	/* ranges whose bottoms are in the view */
-		g_slist_free (merged_used);
-	if (merged_active_seen) /* ranges whose bottoms are below the view */
-		g_slist_free (merged_active_seen);
-	if (merged_unused != NULL);	/* merges in hidden rows */
-		g_slist_free (merged_unused);
-
+	g_slist_free (merged_used);	   /* merges with bottom in view */
+	g_slist_free (merged_active_seen); /* merges with bottom the view */
+	g_slist_free (merged_unused);	   /* merges in hidden rows */
 	g_return_if_fail (merged_active == NULL);
 }
