@@ -7,7 +7,9 @@
 
 typedef enum {
 	PRINT_ORIENT_HORIZONTAL,
-	PRINT_ORIENT_VERTICAL
+	PRINT_ORIENT_VERTICAL,
+	PRINT_ORIENT_HORIZONTAL_UPSIDE_DOWN,
+	PRINT_ORIENT_VERTICAL_UPSIDE_DOWN
 } PrintOrientation;
 
 /*
@@ -62,11 +64,9 @@ typedef struct {
 } PrintRepeatRange;
 
 struct _PrintInformation {
-	PrintOrientation orientation;
 	PrintScaling     scaling;
 	PrintMargins     margins;
 	PrintRepeatRange repeat_top, repeat_left;
-	int	         n_copies;
 	unsigned int     center_vertically:1;
 	unsigned int     center_horizontally:1;
 
@@ -126,14 +126,18 @@ double      unit_convert             (double value,
 void        print_init               (void);
 void        print_shutdown           (void);
 
+void        print_info_set_n_copies  (PrintInformation *pi, guint copies);
 gboolean    print_info_get_margins   (PrintInformation const *pi,
 				      double *header, double *footer, double *left, double *right);
 void        print_info_set_margins   (PrintInformation *pi,
 				      double header, double footer, double left, double right);
-void        print_info_set_margin_header   (PrintInformation *pi, double header);
-void        print_info_set_margin_footer   (PrintInformation *pi, double footer);
-void        print_info_set_margin_left     (PrintInformation *pi, double left);
-void        print_info_set_margin_right    (PrintInformation *pi, double right);
+void        print_info_set_margin_header (PrintInformation *pi, double header);
+void        print_info_set_margin_footer (PrintInformation *pi, double footer);
+void        print_info_set_margin_left   (PrintInformation *pi, double left);
+void        print_info_set_margin_right  (PrintInformation *pi, double right);
+void        print_info_set_orientation   (PrintInformation *pi, 
+					  PrintOrientation orient); 
+PrintOrientation print_info_get_orientation   (PrintInformation *pi); 
 
 /* Formats known */
 extern GList *hf_formats;
