@@ -2,11 +2,11 @@
  * search.c:  Search-and-replace for Gnumeric.
  *
  * Author:
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "search.h"
 
@@ -17,6 +17,7 @@
 #include "position.h"
 #include "cell.h"
 #include "value.h"
+#include "regutf8.h"
 #include "sheet-object-cell-comment.h"
 
 #include <string.h>
@@ -24,16 +25,16 @@
 
 /* ------------------------------------------------------------------------- */
 
-SearchReplace *
+GnmSearchReplace *
 search_replace_new (void)
 {
-	return g_new0 (SearchReplace, 1);
+	return g_new0 (GnmSearchReplace, 1);
 }
 
 /* ------------------------------------------------------------------------- */
 
 void
-search_replace_free (SearchReplace *sr)
+search_replace_free (GnmSearchReplace *sr)
 {
 	g_free (sr->search_text);
 	g_free (sr->replace_text);
@@ -48,7 +49,7 @@ search_replace_free (SearchReplace *sr)
 /* ------------------------------------------------------------------------- */
 
 static int
-search_replace_compile (SearchReplace *sr, gboolean repl)
+search_replace_compile (GnmSearchReplace *sr, gboolean repl)
 {
 	const char *pattern;
 	char *tmp;
@@ -72,24 +73,9 @@ search_replace_compile (SearchReplace *sr, gboolean repl)
 		 * string.  (Thus hoping the regular expression search
 		 * routines are pretty good.)
 		 */
-
-		const char *src = sr->search_text;
-		const char *nextsrc;
-		char *dst = tmp = g_new (char, strlen (src) * 2 + 1);
-		pattern = tmp;
-
-		for (; *src; src = nextsrc) {
-			nextsrc = g_utf8_next_char (src);
-			switch (*src) {
-			case '.': case '[': case '\\':
-			case '*': case '^': case '$':
-				*dst++ = '\\';
-			}
-
-			memcpy (dst, src, nextsrc - src);
-			dst += (nextsrc - src);
-		}
-		*dst = 0;
+		GString *regexp = g_string_new (NULL);
+		gnumeric_regexp_quote (regexp, sr->search_text);
+		pattern = tmp = g_string_free (regexp, FALSE);
 
 		sr->plain_replace = TRUE;
 	}
@@ -106,10 +92,10 @@ search_replace_compile (SearchReplace *sr, gboolean repl)
 
 /* ------------------------------------------------------------------------- */
 
-SearchReplace *
-search_replace_copy (const SearchReplace *sr)
+GnmSearchReplace *
+search_replace_copy (const GnmSearchReplace *sr)
 {
-	SearchReplace *dst = search_replace_new ();
+	GnmSearchReplace *dst = search_replace_new ();
 	gboolean repl = (sr->replace_text != NULL);
 
 	*dst = *sr;
@@ -126,7 +112,7 @@ search_replace_copy (const SearchReplace *sr)
 /* ------------------------------------------------------------------------- */
 
 char *
-search_replace_verify (SearchReplace *sr, gboolean repl)
+search_replace_verify (GnmSearchReplace *sr, gboolean repl)
 {
 	int err;
 	g_return_val_if_fail (sr != NULL, NULL);
@@ -188,7 +174,7 @@ search_replace_verify (SearchReplace *sr, gboolean repl)
 /* ------------------------------------------------------------------------- */
 
 static gboolean
-match_is_word (SearchReplace *sr, const char *src,
+match_is_word (GnmSearchReplace *sr, const char *src,
 	       const regmatch_t *pm, gboolean bolp)
 {
 	/* The empty string is not a word.  */
@@ -255,7 +241,7 @@ inspect_case (const char *p, const char *pend)
 
 
 static char *
-calculate_replacement (SearchReplace *sr, const char *src, const regmatch_t *pm)
+calculate_replacement (GnmSearchReplace *sr, const char *src, const regmatch_t *pm)
 {
 	char *res;
 
@@ -339,7 +325,7 @@ calculate_replacement (SearchReplace *sr, const char *src, const regmatch_t *pm)
 /* ------------------------------------------------------------------------- */
 
 gboolean
-search_match_string (SearchReplace *sr, const char *src)
+search_match_string (GnmSearchReplace *sr, const char *src)
 {
 	int flags = 0;
 
@@ -381,7 +367,7 @@ search_match_string (SearchReplace *sr, const char *src)
  * Returns NULL if nothing changed, or a g_malloc string otherwise.
  */
 char *
-search_replace_string (SearchReplace *sr, const char *src)
+search_replace_string (GnmSearchReplace *sr, const char *src)
 {
 	int nmatch;
 	regmatch_t *pmatch;
@@ -469,8 +455,8 @@ search_replace_string (SearchReplace *sr, const char *src)
 static int
 cb_order_sheet_row_col (const void *_a, const void *_b)
 {
-	const EvalPos *a = *(const EvalPos **)_a;
-	const EvalPos *b = *(const EvalPos **)_b;
+	const GnmEvalPos *a = *(const GnmEvalPos **)_a;
+	const GnmEvalPos *b = *(const GnmEvalPos **)_b;
 	int i;
 
 	i = strcmp (a->sheet->name_unquoted_collate_key,
@@ -488,8 +474,8 @@ cb_order_sheet_row_col (const void *_a, const void *_b)
 static int
 cb_order_sheet_col_row (const void *_a, const void *_b)
 {
-	const EvalPos *a = *(const EvalPos **)_a;
-	const EvalPos *b = *(const EvalPos **)_b;
+	const GnmEvalPos *a = *(const GnmEvalPos **)_a;
+	const GnmEvalPos *b = *(const GnmEvalPos **)_b;
 	int i;
 
 	i = strcmp (a->sheet->name_unquoted_collate_key,
@@ -504,11 +490,11 @@ cb_order_sheet_col_row (const void *_a, const void *_b)
 	return i;
 }
 
-static Value *
+static GnmValue *
 search_collect_cells_cb (Sheet *sheet, int col, int row,
-			 Cell *cell, GPtrArray *cells)
+			 GnmCell *cell, GPtrArray *cells)
 {
-	EvalPos *ep = g_new (EvalPos, 1);
+	GnmEvalPos *ep = g_new (GnmEvalPos, 1);
 
 	ep->sheet = sheet;
 	ep->eval.col = col;
@@ -522,7 +508,7 @@ search_collect_cells_cb (Sheet *sheet, int col, int row,
 
 /* Collect a list of all cells subject to search.  */
 GPtrArray *
-search_collect_cells (SearchReplace *sr, Sheet *sheet)
+search_collect_cells (GnmSearchReplace *sr, Sheet *sheet)
 {
 	GPtrArray *cells;
 
@@ -540,7 +526,7 @@ search_collect_cells (SearchReplace *sr, Sheet *sheet)
 	case SRS_range:
 	{
 		GSList *range_list;
-		EvalPos ep;
+		GnmEvalPos ep;
 		cells = g_ptr_array_new ();
 		range_list = global_range_list_parse (sr->curr_sheet, sr->range_text);
 		global_range_list_foreach (range_list,
@@ -578,13 +564,13 @@ search_collect_cells_free (GPtrArray *cells)
 /* ------------------------------------------------------------------------- */
 /*
  * sr: The search spec.
- * cells: An array of EvalPos*, presumably a result of search_collect_cells.
+ * cells: An array of GnmEvalPos*, presumably a result of search_collect_cells.
  *
  * Returns an array of SearchFilterResult*s, which the caller must free.
  */
 
 GPtrArray *
-search_filter_matching (SearchReplace *sr, const GPtrArray *cells)
+search_filter_matching (GnmSearchReplace *sr, const GPtrArray *cells)
 {
 	unsigned i;
 	GPtrArray *result = g_ptr_array_new ();
@@ -594,7 +580,7 @@ search_filter_matching (SearchReplace *sr, const GPtrArray *cells)
 		SearchReplaceValueResult value_res;
 		SearchReplaceCommentResult comment_res;
 		gboolean found;
-		const EvalPos *ep = g_ptr_array_index (cells, i);
+		const GnmEvalPos *ep = g_ptr_array_index (cells, i);
 
 		found = search_replace_cell (sr, ep, FALSE, &cell_res);
 		g_free (cell_res.old_text);
@@ -641,8 +627,8 @@ search_filter_matching_free (GPtrArray *matches)
 /* ------------------------------------------------------------------------- */
 
 gboolean
-search_replace_comment (SearchReplace *sr,
-			const EvalPos *ep,
+search_replace_comment (GnmSearchReplace *sr,
+			const GnmEvalPos *ep,
 			gboolean repl,
 			SearchReplaceCommentResult *res)
 {
@@ -671,13 +657,13 @@ search_replace_comment (SearchReplace *sr,
 /* ------------------------------------------------------------------------- */
 
 gboolean
-search_replace_cell (SearchReplace *sr,
-		     const EvalPos *ep,
+search_replace_cell (GnmSearchReplace *sr,
+		     const GnmEvalPos *ep,
 		     gboolean repl,
 		     SearchReplaceCellResult *res)
 {
-	Cell *cell;
-	Value *v;
+	GnmCell *cell;
+	GnmValue *v;
 	gboolean is_expr, is_value, is_string, is_other;
 
 	g_return_val_if_fail (res, FALSE);
@@ -694,7 +680,7 @@ search_replace_cell (SearchReplace *sr,
 	v = cell->value;
 
 	is_expr = cell_has_expr (cell);
-	is_value = !is_expr && !cell_is_blank (cell) && v;
+	is_value = !is_expr && !cell_is_empty (cell) && v;
 	is_string = is_value && (v->type == VALUE_STRING);
 	is_other = is_value && !is_string;
 
@@ -735,11 +721,11 @@ search_replace_cell (SearchReplace *sr,
 /* ------------------------------------------------------------------------- */
 
 gboolean
-search_replace_value (SearchReplace *sr,
-		      const EvalPos *ep,
+search_replace_value (GnmSearchReplace *sr,
+		      const GnmEvalPos *ep,
 		      SearchReplaceValueResult *res)
 {
-	Cell *cell;
+	GnmCell *cell;
 
 	g_return_val_if_fail (res, FALSE);
 
