@@ -9,7 +9,7 @@
  *  (C) 1999-2001 Jody Goldberg
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "selection.h"
 
@@ -343,8 +343,10 @@ selection_first_range (SheetView const *sv,
 
 	r = l->data;
 	if (cc != NULL && l->next != NULL) {
-		gnm_cmd_context_error_invalid (cc, cmd_name,
-			_("cannot be performed with multiple ranges selected"));
+		GError *msg = g_error_new (gnm_error_invalid(), 0,
+			_("%s does not support multiple ranges"), cmd_name);
+		gnm_cmd_context_error (cc, msg);
+		g_error_free (msg);
 		return NULL;
 	}
 
@@ -563,8 +565,6 @@ sv_selection_set (SheetView *sv, GnmCellPos const *edit,
 		  int base_col, int base_row,
 		  int move_col, int move_row)
 {
-	g_return_if_fail (IS_SHEET_VIEW (sv));
-
 	sheet_selection_set_internal (sv, edit,
 		base_col, base_row,
 		move_col, move_row, FALSE);
