@@ -7,7 +7,7 @@
  *    Jody Goldberg (jody@gnome.org)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "sheet-control-gui-priv.h"
 
@@ -54,10 +54,11 @@
 #include <gtk/gtkhscrollbar.h>
 #include <gtk/gtkvscrollbar.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
 
 #include <string.h>
 
-static SheetControlClass *scg_parent_class;
+static GObjectClass *scg_parent_class;
 
 static void scg_ant                    (SheetControl *sc);
 static void scg_unant                  (SheetControl *sc);
@@ -477,7 +478,7 @@ scg_select_all (SheetControlGUI *scg)
 			0, 0, SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
 		gnm_expr_entry_signal_update (
 			wbcg_get_entry_logical (scg->wbcg), TRUE);
-	} else if (!wbcg_edit_has_guru (scg->wbcg)) {
+	} else if (wbcg_edit_get_guru (scg->wbcg) == NULL) {
 		scg_mode_edit (SHEET_CONTROL (sc));
 		wbcg_edit_finish (scg->wbcg, FALSE, NULL);
 		sv_selection_reset (sc->view);
@@ -1254,8 +1255,7 @@ scg_finalize (GObject *object)
 		scg->table =NULL;
 	}
 
-	if (G_OBJECT_CLASS (scg_parent_class)->finalize)
-		(*G_OBJECT_CLASS (scg_parent_class)->finalize)(object);
+	(*scg_parent_class->finalize) (object);
 }
 
 static void
@@ -1685,7 +1685,7 @@ scg_mode_edit (SheetControl *sc)
 	if (sc->sheet != NULL)
 		scg_cursor_visible (scg, TRUE);
 
-	if (wbcg_edit_has_guru (scg->wbcg))
+	if (wbcg_edit_get_guru (scg->wbcg) != NULL)
 		wbcg_edit_finish (scg->wbcg, FALSE, NULL);
 	wb_control_menu_state_update (WORKBOOK_CONTROL (scg->wbcg),
 		MS_CLIPBOARD);
@@ -1792,6 +1792,9 @@ scg_object_update_bbox (SheetControlGUI *scg, SheetObject *so,
 {
 	double l, t, r ,b;
 
+	if (scg == NULL)
+		return;
+
 	g_return_if_fail (IS_SHEET_CONTROL_GUI (scg));
 
 	if (so == NULL)
