@@ -45,7 +45,7 @@
 #include <gdk/gdkevents.h>
 #include <gdk/gdkkeysyms.h>
 #include <gsf/gsf-impl-utils.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 
 typedef struct {
 	SheetObject parent;
@@ -132,8 +132,7 @@ filter_field_finalize (GObject *object)
 	}
 
 	parent = g_type_class_peek (SHEET_OBJECT_TYPE);
-	if (parent != NULL && parent->finalize != NULL)
-		parent->finalize (object);
+	parent->finalize (object);
 }
 
 static void
@@ -594,8 +593,8 @@ filter_field_class_init (GObjectClass *object_class)
 	/* SheetObject class method overrides */
 	sheet_object_class->update_view_bounds = filter_field_update_bounds;
 	sheet_object_class->new_view	  = filter_field_new_view;
-	sheet_object_class->read_xml      = NULL;
-	sheet_object_class->write_xml     = NULL;
+	sheet_object_class->read_xml_dom  = NULL;
+	sheet_object_class->write_xml_dom = NULL;
 	sheet_object_class->print         = NULL;
 	sheet_object_class->clone         = NULL;
 }
