@@ -8,12 +8,11 @@
  *   Michael Meeks (mmeeks@gnu.org)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "sheet-object-container.h"
 
 #include "workbook-control-gui-priv.h"
-#include "workbook-private.h"
 #include "sheet.h"
 #include "gui-util.h"
 #include "sheet-control-gui.h"
@@ -40,12 +39,12 @@ static GtkObject *
 sheet_object_container_new_view (SheetObject *so, SheetControlGUI *scg)
 {
 	/* FIXME : this is bogus */
-	GnumericCanvas *gcanvas = scg_pane (scg, 0);
+	GnmCanvas *gcanvas = scg_pane (scg, 0);
 	WorkbookControlGUI *wbcg = scg_get_wbcg	(scg);
 	SheetObjectContainer *soc;
 	BonoboViewFrame *view_frame;
 	GtkWidget	*view_widget;
-	GnomeCanvasItem *view_item;
+	FooCanvasItem *view_item;
 
 	g_return_val_if_fail (IS_SHEET_OBJECT_CONTAINER (so), NULL);
 
@@ -60,10 +59,10 @@ sheet_object_container_new_view (SheetObject *so, SheetControlGUI *scg)
 	}
 
 	view_widget = bonobo_view_frame_get_wrapper (view_frame);
-	gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (gcanvas->sheet_object_group));
-	view_item = gnome_canvas_item_new (
+	foo_canvas_item_raise_to_top (FOO_CANVAS_ITEM (gcanvas->sheet_object_group));
+	view_item = foo_canvas_item_new (
 		gcanvas->sheet_object_group,
-		gnome_canvas_widget_get_type (),
+		foo_canvas_widget_get_type (),
 		"widget", view_widget,
 		"size_pixels", FALSE,
 		NULL);
@@ -90,16 +89,16 @@ sheet_object_container_update_bounds (SheetObject *so, GtkObject *view,
 
 	/* NOTE : far point is EXCLUDED so we add 1 */
 	scg_object_view_position (scg, so, coords);
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (view),
+	foo_canvas_item_set (FOO_CANVAS_ITEM (view),
 		"x", coords [0], "y", coords [1],
 		"width",  coords [2] - coords [0] + 1.,
 		"height", coords [3] - coords [1] + 1.,
 		NULL);
 
 	if (so->is_visible)
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (view));
+		foo_canvas_item_show (FOO_CANVAS_ITEM (view));
 	else
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (view));
+		foo_canvas_item_hide (FOO_CANVAS_ITEM (view));
 }
 
 static void
@@ -137,7 +136,7 @@ sheet_object_container_class_init (GtkObjectClass *object_class)
 	/* SheetObject class method overrides */
 	so_class = SHEET_OBJECT_CLASS (object_class);
 	so_class->new_view 	= sheet_object_container_new_view;
-	so_class->update_bounds = sheet_object_container_update_bounds;
+	so_class->update_view_bounds = sheet_object_container_update_bounds;
 	so_class->set_active    = sheet_object_container_set_active;
 }
 
