@@ -21,7 +21,7 @@
  * USA
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "sheet-object-widget.h"
 
@@ -522,9 +522,9 @@ SOW_MAKE_TYPE (button, Button,
 	       &sheet_widget_button_read_xml_dom)
 
 /****************************************************************************/
-#define SHEET_WIDGET_ADJUSTMENT_TYPE    (sheet_widget_adjustment_get_type())
-#define SHEET_WIDGET_ADJUSTMENT(obj)     (G_TYPE_CHECK_INSTANCE_CAST ((obj), SHEET_WIDGET_ADJUSTMENT_TYPE, SheetWidgetAdjustment))
-#define DEP_TO_ADJUSTMENT(d_ptr)		(SheetWidgetAdjustment *)(((char *)d_ptr) - G_STRUCT_OFFSET(SheetWidgetAdjustment, dep))
+#define SHEET_WIDGET_ADJUSTMENT_TYPE	(sheet_widget_adjustment_get_type())
+#define SHEET_WIDGET_ADJUSTMENT(obj)	(G_TYPE_CHECK_INSTANCE_CAST ((obj), SHEET_WIDGET_ADJUSTMENT_TYPE, SheetWidgetAdjustment))
+#define DEP_TO_ADJUSTMENT(d_ptr)	(SheetWidgetAdjustment *)(((char *)d_ptr) - G_STRUCT_OFFSET(SheetWidgetAdjustment, dep))
 
 typedef struct {
 	SheetObjectWidget	sow;
@@ -1524,19 +1524,17 @@ radio_button_eval (GnmDependent *dep)
 {
 	GnmValue *v;
 	GnmEvalPos pos;
-/*	gboolean err; */
+	gboolean err;
 	int result;
 
 	v = gnm_expr_eval (dep->expression, eval_pos_init_dep (&pos, dep),
 			   GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	result = value_get_as_int (v);
 	value_release (v);
-#if 0
 	if (!err) {
 		/* FIXME : finish this when I have a better idea of a group */
 		/* SheetWidgetRadioButton *swrb = DEP_TO_RADIO_BUTTON (dep); */
 	}
-#endif
 }
 
 static void
@@ -1666,18 +1664,16 @@ list_content_eval (GnmDependent *dep)
 {
 	GnmValue *v;
 	GnmEvalPos pos;
-/*	gboolean err;*/
+	gboolean err;
 	int result;
 
 	v = gnm_expr_eval (dep->expression, eval_pos_init_dep (&pos, dep),
 			   GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	result = value_get_as_int (v);
 	value_release (v);
-#if 0
 	if (!err) {
 		/* SheetWidgetListBase *swc = DEP_TO_LIST_BASE_INPUT (dep); */
 	}
-#endif
 }
 
 static void
@@ -1694,18 +1690,16 @@ list_output_eval (GnmDependent *dep)
 {
 	GnmValue *v;
 	GnmEvalPos pos;
-/*	gboolean err;*/
+	gboolean err;
 	int result;
 
 	v = gnm_expr_eval (dep->expression, eval_pos_init_dep (&pos, dep),
 			   GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	result = value_get_as_int (v);
 	value_release (v);
-#if 0
 	if (!err) {
 		/* SheetWidgetListBase *swc = DEP_TO_LIST_BASE_OUTPUT (dep); */
 	}
-#endif
 }
 
 static void
@@ -1896,6 +1890,7 @@ sheet_object_widget_register (void)
 	SHEET_WIDGET_SCROLLBAR_TYPE;
 	SHEET_WIDGET_CHECKBOX_TYPE;
 	SHEET_WIDGET_RADIO_BUTTON_TYPE;
+	SHEET_WIDGET_LIST_TYPE;
 	SHEET_WIDGET_COMBO_TYPE;
 	SHEET_WIDGET_SPINBUTTON_TYPE;
 	SHEET_WIDGET_SLIDER_TYPE;
