@@ -9,7 +9,7 @@
  *   Jody Goldberg   (jody@gnome.org)
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "sheet-object.h"
 
@@ -235,8 +235,7 @@ sheet_object_finalize (GObject *object)
 		so->sheet = NULL;
 	}
 
-	if (parent_klass != NULL && parent_klass->finalize != NULL)
-		(*parent_klass->finalize)(object);
+	(*parent_klass->finalize) (object);
 }
 
 static void
