@@ -21,7 +21,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "stf.h"
 
@@ -386,7 +386,7 @@ stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, gchar const *enc,
 /***********************************************************************************/
 
 static gboolean
-stf_write_func (const char *string, GsfOutput *output)
+stf_write_func (char const *string, GsfOutput *output)
 {
 	return (gsf_output_puts (output, string) >= 0);
 }
@@ -395,7 +395,7 @@ static void
 stf_write_workbook (GnmFileSaver const *fs, IOContext *context,
 		    WorkbookView const *wbv, GsfOutput *output)
 {
-	StfE_Result_t *result = NULL;
+	StfExportOptions_t *result = NULL;
 
 	if (IS_WORKBOOK_CONTROL_GUI (context->impl))
 		result = stf_export_dialog (WORKBOOK_CONTROL_GUI (context->impl),
@@ -406,13 +406,12 @@ stf_write_workbook (GnmFileSaver const *fs, IOContext *context,
 		return;
 	}
 
-	stf_export_options_set_write_callback (result->export_options,
+	stf_export_options_set_write_callback (result,
 		(StfEWriteFunc) stf_write_func, (gpointer) output);
-	if (stf_export (result->export_options) == FALSE)
+	if (stf_export (result) == FALSE)
 		gnm_cmd_context_error_import (GNM_CMD_CONTEXT (context),
-#warning change string after branch
-			_("Error while trying to write csv file"));
-	stf_export_dialog_result_free (result);
+			_("Error while trying to export file as text"));
+	stf_export_options_free (result);
 }
 
 static void
@@ -431,7 +430,7 @@ stf_write_csv (GnmFileSaver const *fs, IOContext *context,
 
 	if (stf_export (config) == FALSE)
 		gnm_cmd_context_error_import (GNM_CMD_CONTEXT (context),
-			_("Error while trying to write csv file"));
+			_("Error while trying to write CSV file"));
 	stf_export_options_free (config);
 }
 
@@ -452,12 +451,12 @@ csv_tsv_probe (GnmFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
 				return FALSE;
 		return TRUE;
 	} else {
-	char const *name = gsf_input_name (input);
-	if (name == NULL)
-		return FALSE;
-	name = gsf_extension_pointer (name);
-	return (name != NULL &&
-		(g_ascii_strcasecmp (name, "csv") == 0 ||
+		char const *name = gsf_input_name (input);
+		if (name == NULL)
+			return FALSE;
+		name = gsf_extension_pointer (name);
+		return (name != NULL &&
+			(g_ascii_strcasecmp (name, "csv") == 0 ||
 			 g_ascii_strcasecmp (name, "tsv") == 0 ||
 			 g_ascii_strcasecmp (name, "txt") == 0));
 	}
