@@ -25,15 +25,17 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "auto-correct.h"
 
 #include "dates.h"
 #include "application.h"
 #include "gutils.h"
+#include "gnumeric-gconf.h"
 #include <gsf/gsf-impl-utils.h>
 
+#include <gconf/gconf-client.h>
 #include <string.h>
 
 static struct {
@@ -51,15 +53,12 @@ static struct {
 } autocorrect;
 
 #define AUTOCORRECT_DIRECTORY "/apps/gnumeric/autocorrect"
-#define AUTOCORRECT_INIT_CAPS AUTOCORRECT_DIRECTORY "/init-caps"
-#define AUTOCORRECT_INIT_CAPS_LIST AUTOCORRECT_DIRECTORY "/init-caps-list"
-#define AUTOCORRECT_FIRST_LETTER AUTOCORRECT_DIRECTORY "/first-letter"
-#define AUTOCORRECT_FIRST_LETTER_LIST AUTOCORRECT_DIRECTORY "/first-letter-list"
-#define AUTOCORRECT_NAMES_OF_DAYS AUTOCORRECT_DIRECTORY "/names-of-days"
-#define AUTOCORRECT_REPLACE AUTOCORRECT_DIRECTORY "/replace"
-
-static void cb_autocorrect_update (GConfClient *gconf, guint cnxn_id,
-				   GConfEntry *entry, gpointer ignore);
+#define AUTOCORRECT_INIT_CAPS		AUTOCORRECT_DIRECTORY "/init-caps"
+#define AUTOCORRECT_INIT_CAPS_LIST	AUTOCORRECT_DIRECTORY "/init-caps-list"
+#define AUTOCORRECT_FIRST_LETTER	AUTOCORRECT_DIRECTORY "/first-letter"
+#define AUTOCORRECT_FIRST_LETTER_LIST	AUTOCORRECT_DIRECTORY "/first-letter-list"
+#define AUTOCORRECT_NAMES_OF_DAYS	AUTOCORRECT_DIRECTORY "/names-of-days"
+#define AUTOCORRECT_REPLACE		AUTOCORRECT_DIRECTORY "/replace"
 
 static void
 autocorrect_clear (void)
@@ -71,7 +70,7 @@ autocorrect_clear (void)
 static void
 autocorrect_load (void)
 {
-	GConfClient *client = application_get_gconf_client ();
+	GConfClient *client = gnm_app_get_gconf_client ();
 
 	autocorrect.init_caps = gconf_client_get_bool (client,
 		AUTOCORRECT_INIT_CAPS, NULL);
@@ -89,6 +88,13 @@ autocorrect_load (void)
 		AUTOCORRECT_REPLACE, NULL);
 }
 
+static void
+cb_autocorrect_update ()
+{
+	autocorrect_clear ();
+	autocorrect_load ();
+}
+
 static void
 autocorrect_init (void)
 {
@@ -97,24 +103,14 @@ autocorrect_init (void)
 
 	autocorrect_load ();
 	autocorrect.notification_id = gconf_client_notify_add (
-		application_get_gconf_client (),
-		AUTOCORRECT_DIRECTORY, cb_autocorrect_update,
+		gnm_app_get_gconf_client (), AUTOCORRECT_DIRECTORY,
+		(GConfClientNotifyFunc) cb_autocorrect_update,
 		NULL, NULL, NULL);
-	g_object_set_data_full (gnumeric_application_get_app (),
+	g_object_set_data_full (gnm_app_get_app (),
 		"ToolsAutoCorrect", GINT_TO_POINTER (1),
 		(GDestroyNotify) autocorrect_clear);
 }
 
-static void
-cb_autocorrect_update (G_GNUC_UNUSED GConfClient *gconf,
-		       G_GNUC_UNUSED guint cnxn_id,
-		       G_GNUC_UNUSED GConfEntry *entry,
-		       G_GNUC_UNUSED gpointer ignore)
-{
-	autocorrect_clear ();
-	autocorrect_load ();
-}
-
 void
 autocorrect_store_config (void)
 {
@@ -133,9 +129,9 @@ autocorrect_store_config (void)
 	gconf_change_set_set_bool (cs, AUTOCORRECT_REPLACE,
 		autocorrect.replace);
 
-	gconf_client_commit_change_set (application_get_gconf_client (),
+	gconf_client_commit_change_set (gnm_app_get_gconf_client (),
 					cs, FALSE, NULL);
-	gconf_client_suggest_sync (application_get_gconf_client (), NULL);
+	gconf_client_suggest_sync (gnm_app_get_gconf_client (), NULL);
 	gconf_change_set_unref (cs);
 }
 
