@@ -26,7 +26,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include "dao.h"
 
@@ -81,6 +81,25 @@ dao_init (data_analysis_output_t *dao,
 	return dao;
 }
 
+data_analysis_output_t *
+dao_load_from_value (data_analysis_output_t *dao,
+		     GnmValue *output_range)
+{
+	g_return_val_if_fail (output_range != NULL, dao);
+	g_return_val_if_fail 
+		(output_range->type == VALUE_CELLRANGE, dao);
+
+	dao->start_col = output_range->v_range.cell.a.col;
+	dao->start_row = output_range->v_range.cell.a.row;
+	dao->cols = output_range->v_range.cell.b.col
+		- output_range->v_range.cell.a.col + 1;
+	dao->rows = output_range->v_range.cell.b.row
+		- output_range->v_range.cell.a.row + 1;
+	dao->sheet = output_range->v_range.cell.a.sheet;
+	
+	return dao;
+}
+
 /**
  * dao_range_name:
  * @dao:
@@ -940,9 +959,9 @@ dao_convert_to_values (data_analysis_output_t *dao)
 	workbook_recalc (dao->sheet->workbook);
 	for (row = 0; row < dao->rows; row++) {
 		for (col = 0; col < dao->cols; col++) {
-			GnmCell *cell = sheet_cell_fetch (dao->sheet, 
+			GnmCell *cell = sheet_cell_get (dao->sheet, 
 				dao->start_col + col, dao->start_row + row);
-			if (cell_has_expr (cell))
+			if (cell != NULL && cell_has_expr (cell))
 				cell_convert_expr_to_value (cell);
 		}
 	}
