@@ -30,8 +30,7 @@
 
 #include "gnumeric.h"
 #include "numbers.h"
-#include <widgets/gnumeric-expr-entry.h>
-#include <glade/glade.h>
+#include "style.h"
 
 typedef enum {
         NewSheetOutput, NewWorkbookOutput, RangeOutput, InPlaceOutput
@@ -43,15 +42,18 @@ typedef struct {
         int                         start_col, cols;
         int                         start_row, rows;
 	int                         offset_col, offset_row;
-	gboolean                    autofit_flag : 1;
-	gboolean                    clear_outputrange : 1;
-	gboolean                    retain_format : 1;
-	gboolean                    retain_comments : 1;
+	gboolean                    autofit_flag;
+	gboolean                    clear_outputrange;
+	gboolean                    retain_format;
+	gboolean                    retain_comments;
+	gboolean                    put_formulas;
 	WorkbookControl             *wbc;
 } data_analysis_output_t;
 
 data_analysis_output_t *dao_init (data_analysis_output_t *dao, 
 				  data_analysis_output_type_t type);
+data_analysis_output_t *dao_load_from_value (data_analysis_output_t *dao,
+					     GnmValue *output_range);
 
 void dao_autofit_columns      (data_analysis_output_t *dao);
 void dao_autofit_these_columns (data_analysis_output_t *dao, int from_col,
@@ -64,22 +66,32 @@ void dao_set_underlined       (data_analysis_output_t *dao, int col1, int row1,
 			       int col2, int row2);
 void dao_set_percent          (data_analysis_output_t *dao, int col1, int row1,
 			       int col2, int row2);
+void dao_set_date             (data_analysis_output_t *dao, int col1, int row1,
+			       int col2, int row2);
 
+void dao_set_colors (data_analysis_output_t *dao, int col1, int row1,
+		     int col2, int row2,
+		     GnmColor *fore, GnmColor *back);
+void dao_set_align (data_analysis_output_t *dao, int col1, int row1,
+		    int col2, int row2,
+		    StyleHAlignFlags align_h, StyleVAlignFlags align_v);
 void dao_set_cell             (data_analysis_output_t *dao, int col, int row,
 			       char const *text);
 void dao_set_cell_printf      (data_analysis_output_t *dao,
 			       int col, int row, char const *fmt, ...)
                            G_GNUC_PRINTF (4, 5);
 void dao_set_cell_value       (data_analysis_output_t *dao, int col, int row,
-			       Value *v);
+			       GnmValue *v);
+void dao_set_cell_expr       (data_analysis_output_t *dao, int col, int row,
+			       GnmExpr const *expr);
 void dao_set_cell_float       (data_analysis_output_t *dao,
-			       int col, int row, gnum_float v);
+			       int col, int row, gnm_float v);
 void dao_set_cell_int         (data_analysis_output_t *dao,
 			       int col, int row, int v);
 void dao_set_cell_na          (data_analysis_output_t *dao,
 			       int col, int row);
 void dao_set_cell_float_na    (data_analysis_output_t *dao, int col, int row,
-			       gnum_float v, 
+			       gnm_float v, 
 			   gboolean is_valid);
 void dao_set_cell_comment (data_analysis_output_t *dao, int col, int row,
 			   const char *comment);
@@ -101,4 +113,11 @@ void dao_write_header (data_analysis_output_t *dao, const gchar *toolname,
 char *dao_find_name (Sheet *sheet, int col, int row);
 void dao_append_date (GString *buf);
 
+gboolean dao_put_formulas (data_analysis_output_t *dao);
+
+void dao_convert_to_values (data_analysis_output_t *dao);
+void dao_redraw_respan (data_analysis_output_t *dao);
+
+
+
 #endif
