@@ -22,7 +22,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 
@@ -92,7 +92,7 @@ eval_inputs_list (simulation_t *sim, gnm_float **outputs, int iter,
 
 	/* Recompute inputs. */
 	for (cur = sim->list_inputs; cur != NULL; cur = cur->next) {
-		Cell *cell = (Cell *) cur->data;
+		GnmCell *cell = (GnmCell *) cur->data;
 
 		cell_queue_recalc (cell);
 		cell_eval (cell);
@@ -119,7 +119,7 @@ eval_outputs_list (simulation_t *sim, gnm_float **outputs, int iter,
 
 	/* Recompute outputs. */
 	for (cur = sim->list_outputs; cur != NULL; cur = cur->next) {
-		Cell *cell = (Cell *) cur->data;
+		GnmCell *cell = (GnmCell *) cur->data;
 
 		cell_eval (cell);
 		if (cell->value == NULL || ! VALUE_IS_NUMBER (cell->value)) {
@@ -383,14 +383,14 @@ simulation_tool (WorkbookControl        *wbc,
 
 	i = 0;
 	for (cur = sim->list_outputs; cur != NULL; cur = cur->next) {
-		Cell *cell = (Cell *) cur->data;
+		GnmCell *cell = (GnmCell *) cur->data;
 
 		sim->cellnames [i++] =
 			(gchar *) dao_find_name (sheet, cell->pos.col,
 						 cell->pos.row);
 	}
 	for (cur = sim->list_inputs; cur != NULL; cur = cur->next) {
-		Cell *cell = (Cell *) cur->data;
+		GnmCell *cell = (GnmCell *) cur->data;
 		gchar *tmp = dao_find_name (sheet, cell->pos.col,
 					    cell->pos.row);
 		gchar *prefix = _("(Input) ");
