@@ -23,7 +23,7 @@
 */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "reports.h"
 
 #include "parse-util.h"
@@ -86,8 +86,8 @@ solver_results_init (const SolverParameters *sp)
 
 	res->optimal_values    = g_new (gnm_float,  sp->n_variables);
 	res->original_values   = g_new (gnm_float,  sp->n_variables);
-	res->variable_names    = g_new0 (gchar *,    sp->n_variables);
-	res->constraint_names  = g_new0 (gchar *,    sp->n_total_constraints);
+	res->variable_names    = g_new0 (gchar *,   sp->n_variables);
+	res->constraint_names  = g_new0 (gchar *,   sp->n_total_constraints);
 	res->shadow_prizes     = g_new0 (gnm_float, sp->n_total_constraints);
 	res->slack             = g_new0 (gnm_float, sp->n_total_constraints);
 	res->lhs               = g_new0 (gnm_float, sp->n_total_constraints);
@@ -149,7 +149,7 @@ solver_results_free (SolverResults *res)
 
 /* ------------------------------------------------------------------------- */
 
-Cell *
+GnmCell *
 solver_get_target_cell (Sheet *sheet)
 {
         return sheet_cell_get (sheet,
@@ -157,7 +157,7 @@ solver_get_target_cell (Sheet *sheet)
 			       sheet->solver_parameters->target_cell->pos.row);
 }
 
-Cell *
+GnmCell *
 solver_get_input_var (SolverResults *res, int n)
 {
         return res->input_cells_array[n];
@@ -197,8 +197,9 @@ write_constraint_str (int lhs_col, int lhs_row, int rhs_col,
 		      int cols, int rows)
 {
 	GString    *buf = g_string_new (NULL);
-	const char *type_str[] = { "<=", ">=", "=", "Int", "Bool" };
-	char       *result;
+	const char *type_str[] = { "\xe2\x89\xa4" /* "<=" */,
+				   "\xe2\x89\xa5" /* ">=" */,
+				   "=", "Int", "Bool" };
 
 	if (cols == 1 && rows == 1)
 		g_string_append_printf (buf, "%s %s ",
@@ -229,9 +230,7 @@ write_constraint_str (int lhs_col, int lhs_row, int rhs_col,
 		}
 	}
 
-	result = buf->str;
-	g_string_free (buf, FALSE);
-	return result;
+	return g_string_free (buf, FALSE);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -244,7 +243,7 @@ write_constraint_str (int lhs_col, int lhs_row, int rhs_col,
  * function.
  */
 static inline gnm_float
-get_lp_coeff (Cell *target, Cell *change, gnm_float *x0)
+get_lp_coeff (GnmCell *target, GnmCell *change, gnm_float *x0)
 {
         gnm_float tmp = *x0;
 
@@ -268,11 +267,11 @@ save_original_values (SolverResults          *res,
 {
   
 	GSList   *inputs = param->input_cells;
-	Cell     *cell;
+	GnmCell  *cell;
 	int      i = 0;
 
 	while (inputs != NULL) {
-	        cell = (Cell *) inputs->data;
+	        cell = (GnmCell *) inputs->data;
 
 		if (cell == NULL || cell->value == NULL)
 		        res->original_values[i] = 0;
@@ -295,11 +294,11 @@ restore_original_values (SolverResults *res)
 {
   
 	GSList   *inputs = res->param->input_cells;
-	Cell     *cell;
+	GnmCell  *cell;
 	int      i = 0;
 
 	while (inputs != NULL) {
-	        cell = (Cell *) inputs->data;
+	        cell = (GnmCell *) inputs->data;
 
 		sheet_cell_set_value (cell,
 				      value_new_float(res->original_values[i]));
@@ -312,10 +311,10 @@ restore_original_values (SolverResults *res)
  */
 
 static int
-get_col_nbr (SolverResults *res, CellPos *pos)
+get_col_nbr (SolverResults *res, GnmCellPos *pos)
 {
         int  i;
-        Cell *cell;
+        GnmCell *cell;
 
 	for (i = 0; i < res->param->n_variables; i++) {
 	        cell = solver_get_input_var (res, i);
@@ -348,7 +347,7 @@ lp_qp_solver_init (Sheet *sheet, const SolverParameters *param,
 		   gnm_float start_time, GTimeVal start, const gchar **errmsg)
 {
         SolverProgram     program;
-	Cell              *target;
+	GnmCell          *target;
 	gnm_float         x, x0, base;
 	int               i, n, ind;
 
@@ -501,9 +500,9 @@ check_program_definition_failures (Sheet            *sheet,
 {
 	GSList           *inputs;
 	GSList           *c;
-	Cell             *cell;
-	int              i, n;
-	Cell             **input_cells_array;
+	GnmCell          *cell;
+	int               i, n;
+	GnmCell          **input_cells_array;
 	SolverConstraint **constraints_array;
 
 	param->n_variables = 0;
@@ -515,7 +514,7 @@ check_program_definition_failures (Sheet            *sheet,
 	/* Count the nbr of the input cells and check that each cell
 	 * is in the list only once. */
  	for (inputs = param->input_cells; inputs ; inputs = inputs->next) {
-	        cell = (Cell *) inputs->data;
+	        cell = (GnmCell *) inputs->data;
 
 		/* Check that the cell contains a number or is empty. */
 		if (! (cell->value == NULL || VALUE_IS_EMPTY (cell->value)
@@ -528,10 +527,10 @@ check_program_definition_failures (Sheet            *sheet,
 		
 	        param->n_variables += 1;
 	}
-	input_cells_array = g_new (Cell *, param->n_variables);
+	input_cells_array = g_new (GnmCell *, param->n_variables);
 	i = 0;
  	for (inputs = param->input_cells; inputs ; inputs = inputs->next)
-	        input_cells_array[i++] = (Cell *) inputs->data;
+	        input_cells_array[i++] = (GnmCell *) inputs->data;
 
 	param->n_constraints      = 0;
 	param->n_int_constraints  = 0;
@@ -684,7 +683,9 @@ solver_lp_copy (const SolverParameters *src_param, Sheet *new_sheet)
 	g_free (dst_param->input_entry_str);
 	dst_param->input_entry_str = g_strdup (src_param->input_entry_str);
 
+	g_free (dst_param->options.scenario_name);
 	dst_param->options = src_param->options;
+	dst_param->options.scenario_name = g_strdup (src_param->options.scenario_name);
 	/* Had there been any non-scalar options, we'd copy them here.  */
 
 	/* Copy the constraints */
@@ -704,8 +705,8 @@ solver_lp_copy (const SolverParameters *src_param, Sheet *new_sheet)
 
 	/* Copy the input cell list */
 	for (inputs = src_param->input_cells; inputs ; inputs = inputs->next) {
-		Cell *cell = inputs->data;
-		Cell *new_cell;
+		GnmCell *cell = inputs->data;
+		GnmCell *new_cell;
 
 		new_cell = cell_copy (cell);
 		new_cell->base.sheet = new_sheet;
@@ -732,9 +733,9 @@ void
 solver_insert_rows (Sheet *sheet, int row, int count)
 {
 	SolverParameters *param = sheet->solver_parameters;
-	GSList           *constraints;
-        Value            *input_range;
-	Range            range;
+	GSList	 *constraints;
+        GnmValue *input_range;
+	GnmRange	  range;
 
 	/* Adjust the input range. */
 	input_range = global_range_parse (sheet, param->input_entry_str);
@@ -775,9 +776,9 @@ void
 solver_insert_cols (Sheet *sheet, int col, int count)
 {
 	SolverParameters *param = sheet->solver_parameters;
-	GSList           *constraints;
-        Value            *input_range;
-	Range            range;
+	GSList	 *constraints;
+        GnmValue *input_range;
+	GnmRange	  range;
 
 	/* Adjust the input range. */
 	input_range = global_range_parse (sheet, param->input_entry_str);
@@ -818,9 +819,9 @@ void
 solver_delete_rows (Sheet *sheet, int row, int count)
 {
 	SolverParameters *param = sheet->solver_parameters;
-	GSList           *constraints;
-        Value            *input_range;
-	Range            range;
+	GSList	 *constraints;
+        GnmValue *input_range;
+	GnmRange	  range;
 
 	/* Adjust the input range. */
 	input_range = global_range_parse (sheet, param->input_entry_str);
@@ -865,9 +866,9 @@ void
 solver_delete_cols (Sheet *sheet, int col, int count)
 {
 	SolverParameters *param = sheet->solver_parameters;
-	GSList           *constraints;
-        Value            *input_range;
-	Range            range;
+	GSList	 *constraints;
+        GnmValue *input_range;
+	GnmRange	  range;
 
 	/* Adjust the input range. */
 	input_range = global_range_parse (sheet, param->input_entry_str);
