@@ -7,6 +7,7 @@
  *   Jody Goldberg (jody@gnome.org)
  */
 #include <gnumeric-config.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "workbook-cmd-format.h"
 
@@ -24,8 +25,6 @@
 #include "style-border.h"
 #include "style-color.h"
 
-#include <libgnome/gnome-i18n.h>
-
 /* Adds borders to all the selected regions on the sheet.
  * FIXME: This is a little more simplistic then it should be, it always
  * removes and/or overwrites any borders. What we should do is
@@ -35,7 +34,7 @@
 void
 workbook_cmd_mutate_borders (WorkbookControl *wbc, Sheet *sheet, gboolean add)
 {
-	StyleBorder *borders [STYLE_BORDER_EDGE_MAX];
+	GnmBorder *borders [STYLE_BORDER_EDGE_MAX];
 	int i;
 
 	for (i = STYLE_BORDER_TOP; i < STYLE_BORDER_EDGE_MAX; ++i)
@@ -46,8 +45,8 @@ workbook_cmd_mutate_borders (WorkbookControl *wbc, Sheet *sheet, gboolean add)
 		else
 			borders[i] = NULL;
 
-	cmd_format (wbc, sheet, NULL, borders,
-		    add ? _("Add Borders") : _("Remove borders"));
+	cmd_selection_format (wbc, NULL, borders,
+			      add ? _("Add Borders") : _("Remove borders"));
 }
 
 struct closure_colrow_resize {
@@ -56,7 +55,7 @@ struct closure_colrow_resize {
 };
 
 static gboolean
-cb_colrow_collect (Sheet *sheet, Range const *r, gpointer user_data)
+cb_colrow_collect (SheetView *sv, GnmRange const *r, gpointer user_data)
 {
 	struct closure_colrow_resize *info = user_data;
 	int first, last;
@@ -74,21 +73,22 @@ cb_colrow_collect (Sheet *sheet, Range const *r, gpointer user_data)
 }
 
 void
-workbook_cmd_resize_selected_colrow (WorkbookControl *wbc, gboolean is_cols,
-				     Sheet *sheet, int new_size_pixels)
+workbook_cmd_resize_selected_colrow (WorkbookControl *wbc, Sheet *sheet,
+				     gboolean is_cols, int new_size_pixels)
 {
 	struct closure_colrow_resize closure;
 	closure.is_cols = is_cols;
 	closure.selection = NULL;
-	selection_foreach_range (sheet, TRUE, &cb_colrow_collect, &closure);
+	selection_foreach_range (sheet_get_view (sheet, wb_control_view (wbc)),
+				 TRUE, &cb_colrow_collect, &closure);
 	cmd_resize_colrow (wbc, sheet, is_cols, closure.selection, new_size_pixels);
 }
 
 void
 workbook_cmd_format_column_auto_fit (GtkWidget *widget, WorkbookControl *wbc)
 {
-	Sheet *sheet = wb_control_cur_sheet (wbc);
-	workbook_cmd_resize_selected_colrow (wbc, TRUE, sheet, -1);
+	workbook_cmd_resize_selected_colrow (wbc,
+		wb_control_cur_sheet (wbc), TRUE, -1);
 }
 
 void
@@ -100,8 +100,8 @@ sheet_dialog_set_column_width (GtkWidget *ignored, WorkbookControlGUI *wbcg)
 void
 workbook_cmd_format_row_auto_fit (GtkWidget *widget, WorkbookControl *wbc)
 {
-	Sheet *sheet = wb_control_cur_sheet (wbc);
-	workbook_cmd_resize_selected_colrow (wbc, FALSE, sheet, -1);
+	workbook_cmd_resize_selected_colrow (wbc, 
+		wb_control_cur_sheet (wbc), FALSE, -1);
 }
 
 void
@@ -113,15 +113,13 @@ sheet_dialog_set_row_height (GtkWidget *ignored, WorkbookControlGUI *wbcg)
 void
 workbook_cmd_format_column_hide (GtkWidget *widget, WorkbookControl *wbc)
 {
-	cmd_colrow_hide_selection (wbc, wb_control_cur_sheet (wbc),
-				   TRUE, FALSE);
+	cmd_selection_colrow_hide (wbc, TRUE, FALSE);
 }
 
 void
 workbook_cmd_format_column_unhide (GtkWidget *widget, WorkbookControl *wbc)
 {
-	cmd_colrow_hide_selection (wbc, wb_control_cur_sheet (wbc),
-				   TRUE, TRUE);
+	cmd_selection_colrow_hide (wbc, TRUE, TRUE);
 }
 
 void
@@ -139,13 +137,11 @@ workbook_cmd_format_row_std_height (GtkWidget *widget, WorkbookControl *wbc)
 void
 workbook_cmd_format_row_hide (GtkWidget *widget, WorkbookControl *wbc)
 {
-	cmd_colrow_hide_selection (wbc, wb_control_cur_sheet (wbc),
-				   FALSE, FALSE);
+	cmd_selection_colrow_hide (wbc, FALSE, FALSE);
 }
 
 void
 workbook_cmd_format_row_unhide (GtkWidget *widget, WorkbookControl *wbc)
 {
-	cmd_colrow_hide_selection (wbc, wb_control_cur_sheet (wbc),
-				   FALSE, TRUE);
+	cmd_selection_colrow_hide (wbc, FALSE, TRUE);
 }
