@@ -7,15 +7,12 @@
 
 #define WORKBOOK_CONTROL_GUI_TYPE     (workbook_control_gui_get_type ())
 #define WORKBOOK_CONTROL_GUI(obj)     (G_TYPE_CHECK_INSTANCE_CAST ((obj), WORKBOOK_CONTROL_GUI_TYPE, WorkbookControlGUI))
-#define WORKBOOK_CONTROL_GUI_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), WORKBOOK_CONTROL_GUI_TYPE, WorkbookControlGUIClass))
 #define IS_WORKBOOK_CONTROL_GUI(o)    (G_TYPE_CHECK_INSTANCE_TYPE ((o), WORKBOOK_CONTROL_GUI_TYPE))
 
-GType            workbook_control_gui_get_type  (void);
-WorkbookControl *workbook_control_gui_new       (WorkbookView *optional_view,
-						 Workbook *optional_wb);
-void		 workbook_control_gui_init      (WorkbookControlGUI *wbcg,
-						 WorkbookView *optional_view,
-						 Workbook *optional_wb);
+GType    	 workbook_control_gui_get_type  (void);
+WorkbookControl *workbook_control_gui_new	(WorkbookView *optional_view,
+						 Workbook *optional_wb,
+						 GdkScreen *optional_screen);
 
 int      wbcg_sheet_to_page_index (WorkbookControlGUI *wbcg, Sheet *sheet,
 				   SheetControlGUI **res);
@@ -30,11 +27,14 @@ void	   wbcg_ui_update_end	  (WorkbookControlGUI *wbcg);
 
 gboolean   wbcg_rangesel_possible (WorkbookControlGUI const *wbcg);
 gboolean   wbcg_is_editing	  (WorkbookControlGUI const *wbcg);
-void       wbcg_toolbar_timer_clear (WorkbookControlGUI *wbcg);
 void       wbcg_autosave_cancel	  (WorkbookControlGUI *wbcg);
 void       wbcg_autosave_set      (WorkbookControlGUI *wbcg,
 				   int minutes, gboolean prompt);
-void	   wb_control_gui_set_status_text (WorkbookControlGUI *wbcg,
-					   char const *text);
+void	   wbcg_set_status_text	  (WorkbookControlGUI *wbcg,
+				   char const *text);
+void       wbcg_toggle_end_mode   (WorkbookControlGUI *wbcg);
+void       wbcg_set_end_mode      (WorkbookControlGUI *wbcg, gboolean flag);
+
+PangoFontDescription *wbcg_get_font_desc (WorkbookControlGUI *wbcg);
 
 #endif /* GNUMERIC_WORKBOOK_CONTROL_GUI_H */
