@@ -9,7 +9,7 @@
  * (C) 2000-2001 Ximian, Inc.
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "workbook-edit.h"
 
@@ -21,6 +21,7 @@
 #include "commands.h"
 #include "gnumeric-gconf.h"
 #include "mstyle.h"
+#include "style-color.h"
 #include "sheet-control-gui-priv.h"
 #include "sheet-style.h"
 #include "sheet-view.h"
@@ -45,10 +46,10 @@ wbcg_auto_complete_destroy (WorkbookControlGUI *wbcg)
 	g_free (wbcg->auto_complete_text);
 	wbcg->auto_complete_text = NULL;
 
-	if (wbcg->edit_line.signal_changed >= 0) {
+	if (wbcg->edit_line.signal_changed) {
 		g_signal_handler_disconnect (GTK_OBJECT (wbcg_get_entry (wbcg)),
 			wbcg->edit_line.signal_changed);
-		wbcg->edit_line.signal_changed = -1;
+		wbcg->edit_line.signal_changed = 0;
 	}
 
 	if (wbcg->auto_complete != NULL) {
@@ -105,9 +106,9 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept,
 		return TRUE;
 	}
 
-	g_return_val_if_fail (IS_SHEET (wbcg->wb_control.editing_sheet), TRUE);
+	g_return_val_if_fail (IS_SHEET (wbc->editing_sheet), TRUE);
 
-	sheet = wbcg->wb_control.editing_sheet;
+	sheet = wbc->editing_sheet;
 	sv = sheet_get_view (sheet, wbv);
 
 	/* Save the results before changing focus */
@@ -120,7 +121,7 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept,
 
 		/* BE CAREFUL the standard fmts must not NOT include '@' */
 		GnmValue *value = format_match (txt, mstyle_get_format (mstyle),
-					     workbook_date_conv (sheet->workbook));
+						workbook_date_conv (sheet->workbook));
 		if (value != NULL)
 			value_release (value);
 		else
@@ -224,31 +225,49 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept,
 	}
 
 	/* Stop editing */
-	wbcg->wb_control.editing = FALSE;
-	wbcg->wb_control.editing_sheet = NULL;
-	wbcg->wb_control.editing_cell = NULL;
+	wbc->editing = FALSE;
+	wbc->editing_sheet = NULL;
+	wbc->editing_cell = NULL;
 
 	if (wbcg->edit_line.guru != NULL) {
 		GtkWidget *w = wbcg->edit_line.guru;
 		wbcg_edit_detach_guru (wbcg);
 		gtk_widget_destroy (w);
 	}
+	if (wbcg->edit_line.full_content != NULL) {
+		pango_attr_list_unref (wbcg->edit_line.full_content);
+		pango_attr_list_unref (wbcg->edit_line.markup);
+		pango_attr_list_unref (wbcg->edit_line.cur_fmt);
+		wbcg->edit_line.full_content =
+			wbcg->edit_line.markup =
+			wbcg->edit_line.cur_fmt = NULL;
+		g_signal_handler_disconnect (GTK_OBJECT (wbcg_get_entry (wbcg)),
+			wbcg->edit_line.signal_insert);
+		wbcg->edit_line.signal_insert = 0;
+		g_signal_handler_disconnect (GTK_OBJECT (wbcg_get_entry (wbcg)),
+			wbcg->edit_line.signal_delete);
+		wbcg->edit_line.signal_delete = 0;
+		g_signal_handler_disconnect (GTK_OBJECT (wbcg_get_entry (wbcg)),
+			wbcg->edit_line.signal_cursor_pos);
+		wbcg->edit_line.signal_cursor_pos = 0;
+	}
 	wb_control_edit_set_sensitive (wbc, FALSE, TRUE);
 
 	/* restore focus to original sheet in case things were being selected
 	 * on a different page.  Do no go through the view, rangesel is
 	 * specific to the control.
 	 */
-	wb_control_sheet_focus (WORKBOOK_CONTROL (wbcg), sheet);
+	wb_control_sheet_focus (wbc, sheet);
 
 	/* Only the edit sheet has an edit cursor */
 	scg_edit_stop (wbcg_cur_scg (wbcg));
 
 	wbcg_auto_complete_destroy (wbcg);
+	wb_control_style_feedback (wbc, NULL); /* in case markup messed with things */
 
 	/* really necessary ? the commands should have taken care of it */
 	if (accept && sheet->workbook->recalc_auto)
-		workbook_recalc (wb_control_workbook (WORKBOOK_CONTROL (wbcg)));
+		workbook_recalc (wb_control_workbook (wbc));
 
 	return TRUE;
 }
@@ -329,7 +348,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 		return TRUE;
 
 	/* Avoid recursion, and do not begin editing if a guru is up */
-	if (inside_editing || wbcg_edit_has_guru (wbcg))
+	if (inside_editing || wbcg_edit_get_guru (wbcg) != NULL)
 		return TRUE;
 	inside_editing = TRUE;
 
@@ -412,7 +431,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 	/* If this assert fails, it means editing was not shut down
 	 * properly before
 	 */
-	g_return_val_if_fail (wbcg->edit_line.signal_changed == -1, TRUE);
+	g_return_val_if_fail (wbcg->edit_line.signal_changed == 0, TRUE);
 	wbcg->edit_line.signal_changed = g_signal_connect (
 		G_OBJECT (wbcg_get_entry (wbcg)),
 		"changed",
@@ -469,14 +488,13 @@ wbcg_edit_attach_guru_main (WorkbookControlGUI *wbcg, GtkWidget *guru)
 	 */
 	gnm_app_clipboard_unant ();
 
-	/* don't sit end 'End' mode when a dialog comes up */
+	/* don't set end 'End' mode when a dialog comes up */
 	wbcg_set_end_mode (wbcg, FALSE);
 
 	wbcg->edit_line.guru = guru;
 	gtk_editable_set_editable (GTK_EDITABLE (wbcg_get_entry (wbcg)), FALSE);
 	wb_control_edit_set_sensitive (wbc, FALSE, FALSE);
 	wb_control_menu_state_update (wbc, MS_GURU_MENU_ITEMS);
-
 }
 
 static void
@@ -539,7 +557,7 @@ wbcg_edit_detach_guru (WorkbookControlGUI *wbcg)
 }
 
 GtkWidget *
-wbcg_edit_has_guru (WorkbookControlGUI const *wbcg)
+wbcg_edit_get_guru (WorkbookControlGUI const *wbcg)
 {
 	return wbcg->edit_line.guru;
 }
@@ -627,12 +645,179 @@ wbcg_edit_ctor (WorkbookControlGUI *wbcg)
 					      NULL);
 	wbcg->edit_line.temp_entry = NULL;
 	wbcg->edit_line.guru = NULL;
-	wbcg->edit_line.signal_changed = -1;
-	wbcg->toolbar_sensitivity_timer = 0;
+	wbcg->edit_line.signal_changed = 0;
+	wbcg->edit_line.full_content = NULL;
+	wbcg->edit_line.markup = NULL;
+	wbcg->edit_line.cur_fmt = NULL;
 }
 
-void
-wbcg_edit_dtor (WorkbookControlGUI *wbcg)
+static gboolean
+cb_set_attr_list_len (PangoAttribute *a, int const *len)
 {
-	wbcg_toolbar_timer_clear (wbcg);
+	a->start_index = 0;
+	a->end_index = *len;
+	return FALSE;
+}
+
+static void
+cb_entry_insert_text (GtkEditable *editable,
+		      gchar const *text,
+		      gint         len,
+		      gint        *pos,
+		      WorkbookControlGUI *wbcg)
+{
+	pango_attr_list_filter (wbcg->edit_line.cur_fmt,
+		(PangoAttrFilterFunc) cb_set_attr_list_len, &len);
+	pango_attr_list_splice (wbcg->edit_line.full_content,
+		wbcg->edit_line.cur_fmt, *pos, len);
+	pango_attr_list_splice (wbcg->edit_line.markup,
+		wbcg->edit_line.cur_fmt, *pos, len);
+}
+
+typedef struct {
+	unsigned start_pos, end_pos, len;
+} EntryDeleteTextClosure;
+
+static gboolean
+cb_delete_filter (PangoAttribute *a, EntryDeleteTextClosure *change)
+{
+	if (change->start_pos >= a->end_index)
+		return FALSE;
+
+	if (change->start_pos <= a->start_index) {
+		if (change->end_pos >= a->end_index)
+			return TRUE;
+
+		a->start_index -= change->len;
+		a->end_index -= change->len;
+		if (a->start_index < change->start_pos)
+			a->start_index = change->start_pos;
+	} else {
+		a->end_index -= change->len;
+		if (a->end_index < change->end_pos)
+			a->end_index = change->end_pos;
+	}
+
+	return FALSE;
+}
+
+static void
+cb_entry_cursor_pos (WorkbookControlGUI *wbcg)
+{
+	gint start, end, target_pos;
+	GtkEditable *entry = GTK_EDITABLE (wbcg_get_entry (wbcg));
+	PangoAttrIterator *iter = pango_attr_list_get_iterator (wbcg->edit_line.full_content);
+	PangoAttribute *attr;
+
+	/* 1) Use first selected character if there is a selection
+	 * 2) Ue the character just before the edit pos if it exists
+	 * 3) Use the first character */
+	if (gtk_editable_get_selection_bounds (entry, &start, &end))
+		target_pos = start;
+	else {
+		target_pos = gtk_editable_get_position (entry);
+		if (target_pos > 0)
+			target_pos--;
+	}
+
+	pango_attr_list_unref (wbcg->edit_line.cur_fmt);
+	wbcg->edit_line.cur_fmt = pango_attr_list_new ();
+	do {
+		pango_attr_iterator_range (iter, &start, &end);
+		if (start <= target_pos && target_pos < end) {
+			GSList *ptr, *attrs = pango_attr_iterator_get_attrs (iter);
+			GnmStyle *style = mstyle_new ();
+
+			for (ptr = attrs; ptr != NULL ; ptr = ptr->next) {
+				attr = (PangoAttribute *)(ptr->data);
+				mstyle_set_from_pango_attribute (style, attr);
+				attr->start_index = 0;
+				attr->end_index = 1;
+				pango_attr_list_insert (wbcg->edit_line.cur_fmt, attr);
+			}
+
+			wb_control_style_feedback (WORKBOOK_CONTROL (wbcg), style);
+			mstyle_unref (style);
+			g_slist_free (attrs);
+			break;
+		}
+	} while (pango_attr_iterator_next (iter));
+	pango_attr_iterator_destroy (iter);
+}
+
+static void
+cb_entry_delete_text (GtkEditable    *editable,
+		      gint            start_pos,
+		      gint            end_pos,
+		      WorkbookControlGUI *wbcg)
+{
+	PangoAttrList *gunk;
+	EntryDeleteTextClosure change;
+
+	change.start_pos = start_pos;
+	change.end_pos = end_pos;
+	change.len = end_pos - start_pos;
+	gunk = pango_attr_list_filter (wbcg->edit_line.full_content,
+		(PangoAttrFilterFunc) cb_delete_filter, &change);
+	if (gunk != NULL)
+		pango_attr_list_unref (gunk);
+	gunk = pango_attr_list_filter (wbcg->edit_line.markup,
+		(PangoAttrFilterFunc) cb_delete_filter, &change);
+	if (gunk != NULL)
+		pango_attr_list_unref (gunk);
+	cb_entry_cursor_pos (wbcg);
+}
+
+/**
+ * wbcg_edit_add_markup :
+ * @wbcg : #WorkbookControlGUI
+ * @attr : #PangoAttribute
+ *
+ * Absorb the ref to @attr and merge it into the list
+ **/
+void
+wbcg_edit_add_markup (WorkbookControlGUI *wbcg, PangoAttribute *attr)
+{
+	GObject *entry = (GObject *)wbcg_get_entry (wbcg);
+	if (wbcg->edit_line.full_content == NULL) {
+		SheetView const *sv  = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
+		wbcg->edit_line.full_content = mstyle_generate_attrs_full (
+			sheet_style_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row));
+		wbcg->edit_line.markup = pango_attr_list_new ();
+		wbcg->edit_line.cur_fmt = pango_attr_list_copy (wbcg->edit_line.full_content);
+		wbcg->edit_line.signal_insert = g_signal_connect (
+			entry, "insert-text",
+			G_CALLBACK (cb_entry_insert_text), wbcg);
+		wbcg->edit_line.signal_delete = g_signal_connect (
+			entry, "delete-text",
+			G_CALLBACK (cb_entry_delete_text), wbcg);
+		wbcg->edit_line.signal_cursor_pos = g_signal_connect_swapped (
+			entry, "notify::cursor-position",
+			G_CALLBACK (cb_entry_cursor_pos), wbcg);
+	}
+
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry),
+					       &attr->start_index, &attr->end_index)) {
+		pango_attr_list_change (wbcg->edit_line.full_content,
+			pango_attribute_copy (attr));
+		pango_attr_list_change (wbcg->edit_line.markup,
+			pango_attribute_copy (attr));
+	}
+	attr->start_index = 0;
+	attr->end_index = 1;
+	pango_attr_list_change (wbcg->edit_line.cur_fmt, attr);
+	g_signal_emit (G_OBJECT (wbcg), wbcg_signals [WBCG_MARKUP_CHANGED], 0);
+}
+
+/**
+ * wbcg_edit_get_markup :
+ * @wbcg : #WorkbookControlGUI
+ *
+ * Returns a potentially NULL PangoAttrList of the current markup while
+ * editing.  The list belongs to @wbcg and should not be freed.
+ **/
+PangoAttrList *
+wbcg_edit_get_markup (WorkbookControlGUI *wbcg, gboolean full)
+{
+	return full ? wbcg->edit_line.full_content : wbcg->edit_line.markup;
 }
