@@ -20,13 +20,12 @@
  * USA
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "workbook-view.h"
 
 #include "workbook-control-priv.h"
 #include "workbook.h"
-#include "history.h"
 #include "application.h"
 #include "sheet.h"
 #include "sheet-view.h"
@@ -48,13 +47,10 @@
 #include "gutils.h"
 #include "io-context.h"
 
-#include <gsf/gsf-input-memory.h>
-#include <gsf/gsf-input-stdio.h>
-#include <gsf/gsf-output-stdio.h>
-#include <gsf/gsf-utils.h>
-#include <gsf-gnome/gsf-input-gnomevfs.h>
-
+#include <gsf/gsf.h>
 #include <gsf/gsf-impl-utils.h>
+#include <gsf/gsf-output-stdio.h>
+#include <gsf/gsf-input.h>
 #include <locale.h>
 #include <string.h>
 #include <unistd.h>
@@ -186,42 +182,41 @@ void
 wb_view_format_feedback (WorkbookView *wbv, gboolean display)
 {
 	SheetView *sv;
+	GnmStyle *style;
+	GnmFormat *sf_style, *sf_cell;
+	GnmCell *cell;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 
 	sv = wbv->current_sheet_view;
-	if (sv != NULL) {
-		GnmStyle *mstyle = sheet_style_get (sv->sheet,
-			sv->edit_pos.col,
-			sv->edit_pos.row);
-		GnmFormat *sf_style = mstyle_get_format (mstyle);
-		GnmFormat *sf_cell;
-		GnmCell *cell;
+	if (sv == NULL)
+		return;
 
-		if (style_format_is_general (sf_style) &&
-		    (cell = sheet_cell_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row)) &&
-		    cell->value && VALUE_FMT (cell->value))
-			sf_cell = VALUE_FMT (cell->value);
-		else
-			sf_cell = sf_style;
+	style = sheet_style_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row);
+	sf_style = mstyle_get_format (style);
+	if (style_format_is_general (sf_style) &&
+	    (cell = sheet_cell_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row)) &&
+	    cell->value && VALUE_FMT (cell->value))
+		sf_cell = VALUE_FMT (cell->value);
+	else
+		sf_cell = sf_style;
 
-		if (style_format_equal (sf_cell, sf_style)) {
-			if (mstyle == wbv->current_format)
-				return;
-			mstyle_ref (mstyle);
-		} else {
-			mstyle = mstyle_copy (mstyle);
-			mstyle_set_format (mstyle, sf_cell);
-		}
+	if (style_format_equal (sf_cell, sf_style)) {
+		if (style == wbv->current_format)
+			return;
+		mstyle_ref (style);
+	} else {
+		style = mstyle_copy (style);
+		mstyle_set_format (style, sf_cell);
+	}
 
-		if (wbv->current_format != NULL)
-			mstyle_unref (wbv->current_format);
-		wbv->current_format = mstyle;
+	if (wbv->current_format != NULL)
+		mstyle_unref (wbv->current_format);
+	wbv->current_format = style;
 
-		if (display) {
-			WORKBOOK_VIEW_FOREACH_CONTROL(wbv, control,
-				wb_control_format_feedback (control););
-		}
+	if (display) {
+		WORKBOOK_VIEW_FOREACH_CONTROL(wbv, control,
+			wb_control_style_feedback (control, NULL););
 	}
 }
 
@@ -570,22 +565,9 @@ wbv_save_to_file (WorkbookView *wbv, GnmFileSaver const *fs,
 
 	if (filename_utf8) {
 		GError *err = NULL;
-		GsfOutput   *output = NULL;
-		GnomeVFSURI *uri = gnome_vfs_uri_new (filename);
-
-		/* cheesy mechanism to prefer stdio */
-		if (uri != NULL && uri->method_string != NULL &&
-		    0 != strcmp ("file", uri->method_string)) {
-			output = (GsfOutput *)gsf_output_gnomevfs_new (filename, &err);
-			g_warning ("vfs");
-		} else {
-			output = (GsfOutput *)gsf_output_stdio_new (filename, &err);
-			g_warning ("stdio");
-		}
-
-		if (uri != NULL)
-			gnome_vfs_uri_unref (uri);
+		GsfOutputStdio *output;
 
+		output = gsf_output_stdio_new (filename, &err);
 		if (output == NULL) {
 			char *str = g_strdup_printf (_("Can't open '%s' for writing: %s"),
 						     filename_utf8, err->message);
@@ -756,7 +738,7 @@ wb_view_sendto (WorkbookView *wbv, GnmCmdContext *context)
 #define GNM_SEND_DIR	".gnm-sendto-"
 #ifdef HAVE_MKDTEMP
 		template = g_build_filename (g_get_tmp_dir (),
-					 GNM_SEND_DIR "XXXXXX", NULL);
+			GNM_SEND_DIR "XXXXXX", NULL);
 		mkdtemp (template);
 #else
 		while (1) {
@@ -934,29 +916,10 @@ wb_view_new_from_file (char const *filename,
 
 	if (filename_utf8) {
 		GError *err = NULL;
-		GsfInputMemory *in_mem;
-		GsfInputStdio  *in_stdio;
-		GsfInput *input = NULL;
-
-		/* Only report error if stdio fails too */
-		in_mem = gsf_input_mmap_new (filename, NULL);
-		if (in_mem == NULL) {
-			in_stdio = gsf_input_stdio_new (filename, &err);
-			if (in_stdio == NULL) {
-				GsfInputGnomeVFS *in_vfs;
-				
-				g_error_free (err);
-				err = NULL;
-
-				in_vfs = gsf_input_gnomevfs_new (filename, &err);
-				if (in_vfs != NULL)
-					input = GSF_INPUT (in_vfs);
-			} else
-				input = GSF_INPUT (in_stdio);
-		} else
-			input = GSF_INPUT (in_mem);
+		GsfInput *input = go_file_open (filename, &err);
 
 		puts (filename);
+
 		if (input != NULL) {
 			WorkbookView *res = wb_view_new_from_input (input,
 								    optional_fmt, io_context,
