@@ -12,7 +12,7 @@
  * (C) 2002 Jody Goldberg
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "workbook-priv.h"
 
@@ -54,7 +54,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static GQuark signals [LAST_SIGNAL] = { 0 };
+static guint signals [LAST_SIGNAL] = { 0 };
 
 static void
 cb_saver_finalize (Workbook *wb, GnmFileSaver *saver)
@@ -138,8 +138,13 @@ workbook_finalize (GObject *wb_object)
 			g_warning ("Unexpected left over views");
 	}
 
+	if (wb->sheet_local_functions != NULL) {
+		g_hash_table_destroy (wb->sheet_local_functions);
+		wb->sheet_local_functions = NULL;
+	}
+
 	/* Remove ourselves from the list of workbooks.  */
-	application_workbook_list_remove (wb);
+	gnm_app_workbook_list_remove (wb);
 
 	/* Now do deletions that will put this workbook into a weird
 	   state.  Careful here.  */
@@ -150,7 +155,7 @@ workbook_finalize (GObject *wb_object)
 	wb->sheets = NULL;
 
 	if (wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
-		application_history_add (wb->filename);
+		gnm_app_history_add (wb->filename);
 
 	if (wb->filename) {
 	       g_free (wb->filename);
@@ -158,7 +163,7 @@ workbook_finalize (GObject *wb_object)
 	}
 
 #warning this has no business being here
-	if (initial_workbook_open_complete && application_workbook_list () == NULL)
+	if (initial_workbook_open_complete && gnm_app_workbook_list () == NULL)
 		gtk_main_quit ();
 	G_OBJECT_CLASS (workbook_parent_class)->finalize (wb_object);
 }
@@ -292,6 +297,7 @@ workbook_init (GObject *object)
 	wb->sheets = g_ptr_array_new ();
 	wb->sheet_hash_private = g_hash_table_new (g_str_hash, g_str_equal);
 	wb->sheet_order_dependents = NULL;
+	wb->sheet_local_functions = NULL;
 	wb->names        = NULL;
 	wb->summary_info = summary_info_new ();
 	summary_info_default (wb->summary_info);
@@ -315,7 +321,7 @@ workbook_init (GObject *object)
 	wb->being_reordered    = FALSE;
 	wb->recursive_dirty_enabled = TRUE;
 
-	application_workbook_list_add (wb);
+	gnm_app_workbook_list_add (wb);
 }
 
 static void
@@ -330,7 +336,7 @@ workbook_class_init (GObjectClass *object_class)
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (WorkbookClass, summary_changed),
 		(GSignalAccumulator) NULL, NULL,
-		gnm__VOID__VOID,
+		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
 
@@ -339,7 +345,7 @@ workbook_class_init (GObjectClass *object_class)
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (WorkbookClass, filename_changed),
 		(GSignalAccumulator) NULL, NULL,
-		gnm__VOID__VOID,
+		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
 
@@ -348,7 +354,7 @@ workbook_class_init (GObjectClass *object_class)
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (WorkbookClass, sheet_order_changed),
 		(GSignalAccumulator) NULL, NULL,
-		gnm__VOID__VOID,
+		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
 
@@ -357,7 +363,7 @@ workbook_class_init (GObjectClass *object_class)
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (WorkbookClass, sheet_added),
 		(GSignalAccumulator) NULL, NULL,
-		gnm__VOID__VOID,
+		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
 
@@ -366,7 +372,7 @@ workbook_class_init (GObjectClass *object_class)
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (WorkbookClass, sheet_deleted),
 		(GSignalAccumulator) NULL, NULL,
-		gnm__VOID__VOID,
+		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
 }
@@ -626,7 +632,7 @@ workbook_unref (Workbook *wb)
  *    to stop (by returning non-NULL).
  */
 GnmValue *
-workbook_foreach_cell_in_range (EvalPos const *pos,
+workbook_foreach_cell_in_range (GnmEvalPos const *pos,
 				GnmValue const	*cell_range,
 				CellIterFlags	 flags,
 				CellIterFunc	 handler,
@@ -673,7 +679,7 @@ workbook_foreach_cell_in_range (EvalPos const *pos,
  * @wb : The workbook to find cells in.
  * @comments: If true, include cells with only comments also.
  *
- * Collects a GPtrArray of EvalPos pointers for all cells in a workbook.
+ * Collects a GPtrArray of GnmEvalPos pointers for all cells in a workbook.
  * No particular order should be assumed.
  */
 GPtrArray *
@@ -696,7 +702,7 @@ workbook_cells (Workbook *wb, gboolean comments)
 		g_ptr_array_set_size (cells, oldlen + scells->len);
 		memcpy (&g_ptr_array_index (cells, oldlen),
 			&g_ptr_array_index (scells, 0),
-			scells->len * sizeof (EvalPos *));
+			scells->len * sizeof (GnmEvalPos *));
 
 		g_ptr_array_free (scells, TRUE);
 	}
@@ -705,6 +711,12 @@ workbook_cells (Workbook *wb, gboolean comments)
 	return cells;
 }
 
+GSList *
+workbook_local_functions (Workbook const *wb)
+{
+	return NULL;
+}
+
 gboolean
 workbook_enable_recursive_dirty (Workbook *wb, gboolean enable)
 {
@@ -815,7 +827,7 @@ workbook_sheet_count (Workbook const *wb)
 }
 
 static void
-cb_dep_unlink (Dependent *dep, gpointer value, gpointer user_data)
+cb_dep_unlink (GnmDependent *dep, gpointer value, gpointer user_data)
 {
 	GnmCellPos *pos = NULL;
 	if (dependent_is_cell (dep))
@@ -836,7 +848,7 @@ pre_sheet_index_change (Workbook *wb)
 }
 
 static void
-cb_dep_link (Dependent *dep, gpointer value, gpointer user_data)
+cb_dep_link (GnmDependent *dep, gpointer value, gpointer user_data)
 {
 	GnmCellPos *pos = NULL;
 	if (dependent_is_cell (dep))
@@ -945,7 +957,7 @@ workbook_sheet_attach (Workbook *wb, Sheet *new_sheet,
 }
 
 static void
-cb_tweak_3d (Dependent *dep, gpointer value, GnmExprRewriteInfo *rwinfo)
+cb_tweak_3d (GnmDependent *dep, gpointer value, GnmExprRewriteInfo *rwinfo)
 {
 	GnmExpr const *newtree = gnm_expr_rewrite (dep->expression, rwinfo);
 	if (newtree != NULL) {
@@ -1080,8 +1092,8 @@ workbook_sheet_delete (Sheet *sheet)
 		wb->undo_commands = NULL;
 		wb->redo_commands = NULL;
 		WORKBOOK_FOREACH_CONTROL (wb, view, control,
-			wb_control_undo_redo_clear (control, TRUE);
-			wb_control_undo_redo_clear (control, FALSE);
+			wb_control_undo_redo_truncate (control, 0, TRUE);
+			wb_control_undo_redo_truncate (control, 0, FALSE);
 			wb_control_undo_redo_labels (control, NULL, NULL);
 		);
 
@@ -1096,8 +1108,8 @@ workbook_sheet_delete (Sheet *sheet)
 				wb_view_sheet_focus (wbv, new_focus);
 		);
 		WORKBOOK_FOREACH_CONTROL (wb, view, control,
-			wb_control_undo_redo_clear (control, TRUE);
-			wb_control_undo_redo_clear (control, FALSE);
+			wb_control_undo_redo_truncate (control, 0, TRUE);
+			wb_control_undo_redo_truncate (control, 0, FALSE);
 			wb_control_undo_redo_labels (control, NULL, NULL);
 		);
 	}
@@ -1212,11 +1224,12 @@ workbook_sheet_get_free_name (Workbook *wb,
  *
  * Returns TRUE when it is possible.
  **/
-gboolean    workbook_sheet_rename_check  (Workbook *wb,
-					  GSList *sheet_indices,
-					  GSList *new_names,
-					  GSList *sheet_indices_deleted,
-					  CommandContext *cc)
+gboolean
+workbook_sheet_rename_check (Workbook *wb,
+			     GSList *sheet_indices,
+			     GSList *new_names,
+			     GSList *sheet_indices_deleted,
+			     GnmCmdContext *cc)
 {
 	GSList *sheet_index = sheet_indices;
 	GSList *new_name = new_names;
@@ -1242,7 +1255,7 @@ gboolean    workbook_sheet_rename_check  (Workbook *wb,
 
 			if (the_new_name == NULL && 
 			    GPOINTER_TO_INT (sheet_index->data) != -1) {
-				gnumeric_error_invalid 
+				gnm_cmd_context_error_invalid 
 					(cc, 
 					 _("Sheet name is NULL"),
 					 the_new_name);
@@ -1253,7 +1266,7 @@ gboolean    workbook_sheet_rename_check  (Workbook *wb,
 				
 				/* Is the sheet name valid utf-8 ?*/
 				if (!g_utf8_validate (the_new_name, -1, NULL)) {
-					gnumeric_error_invalid 
+					gnm_cmd_context_error_invalid 
 						(cc, 
 						 _("Sheet name is not valid utf-8"),
 						 the_new_name);
@@ -1262,7 +1275,7 @@ gboolean    workbook_sheet_rename_check  (Workbook *wb,
 				
 				/* Is the sheet name to short ?*/
 				if (1 > g_utf8_strlen (the_new_name, -1)) {
-					gnumeric_error_invalid 
+					gnm_cmd_context_error_invalid 
 						(cc,
 						 _("Sheet name must have at "
 						   "least 1 letter"),
@@ -1286,7 +1299,7 @@ gboolean    workbook_sheet_rename_check  (Workbook *wb,
 					}
 
 					if (NULL == tmp_sheets) {
-						gnumeric_error_invalid 
+						gnm_cmd_context_error_invalid 
 							(cc,
 							 _("There is already a "
 							   "sheet named"),
@@ -1300,7 +1313,7 @@ gboolean    workbook_sheet_rename_check  (Workbook *wb,
 				    g_slist_find_custom (new_name->next, 
 							 the_new_name, 
 							 g_str_compare) != NULL) {
-					gnumeric_error_invalid 
+					gnm_cmd_context_error_invalid 
 						(cc,
 						 _("You may not use this name twice"),
 						 the_new_name);
@@ -1326,10 +1339,11 @@ gboolean    workbook_sheet_rename_check  (Workbook *wb,
  *
  * Returns FALSE when it was successful
  **/
-gboolean    workbook_sheet_rename        (Workbook *wb,
-					  GSList *sheet_indices,
-					  GSList *new_names,
-					  CommandContext *cc)
+gboolean
+workbook_sheet_rename (Workbook *wb,
+		       GSList *sheet_indices,
+		       GSList *new_names,
+		       GnmCmdContext *cc)
 {
 	GSList *sheet_index = sheet_indices;
 	GSList *new_name = new_names;
@@ -1396,6 +1410,30 @@ workbook_sheet_change_protection  (Workbook *wb,
 	
 }
 
+/**
+ * workbook_find_command :
+ * @wb : #Workbook
+ * @is_undo : undo vs redo
+ * @key : command
+ *
+ * returns the 1 based index of the @key command, or 0 if it is not found
+ **/
+unsigned
+workbook_find_command (Workbook *wb, gboolean is_undo, gpointer cmd)
+{
+	GSList *ptr;
+	unsigned n = 1;
+
+	g_return_val_if_fail (IS_WORKBOOK (wb), 0);
+
+	ptr = is_undo ? wb->undo_commands : wb->redo_commands;
+	for ( ; ptr != NULL ; ptr = ptr->next, n++)
+		if (ptr->data == cmd)
+			return n;
+	g_warning ("%s command : %p not found", is_undo ? "undo" : "redo", cmd);
+	return 0;
+}
+
 /**
  * workbook_sheet_recolor:
  * @wb:          workbook to look for
@@ -1426,10 +1464,10 @@ workbook_sheet_recolor  (Workbook *wb,
 		if (sheet != NULL) {
 			GdkColor *fc = (GdkColor *) fore->data;
 			GdkColor *bc = (GdkColor *) back->data;
-			StyleColor *fore_color = fc ?
+			GnmColor *fore_color = fc ?
 				style_color_new (fc->red, fc->green,
 						 fc->blue) : NULL;
-			StyleColor *back_color = bc ?
+			GnmColor *back_color = bc ?
 				style_color_new (bc->red, bc->green,
 						 bc->blue) : NULL;
 			sheet_set_tab_color (sheet, back_color, fore_color);
@@ -1572,3 +1610,4 @@ workbook_set_1904 (Workbook *wb, gboolean flag)
 GSF_CLASS (Workbook, workbook,
 	   workbook_class_init, workbook_init,
 	   G_TYPE_OBJECT);
+
