@@ -43,15 +43,18 @@ gboolean    workbook_sheet_recolor       (Workbook *wb,
 gboolean    workbook_sheet_rename        (Workbook *wb,
 					  GSList *sheet_indices,
 					  GSList *new_names,
-					  CommandContext *cc);
+					  GnmCmdContext *cc);
 gboolean    workbook_sheet_rename_check  (Workbook *wb,
 					  GSList *sheet_indices,
 					  GSList *new_names,
 					  GSList *sheet_indices_deleted,
-					  CommandContext *cc);
+					  GnmCmdContext *cc);
 gboolean    workbook_sheet_change_protection  (Workbook *wb,
-					  GSList *sheets,
-					  GSList *locks);
+					       GSList *sheets,
+					       GSList *locks);
+
+unsigned    workbook_find_command	(Workbook *wb,
+					 gboolean is_undo, gpointer cmd);
 
 
 /* IO Routines */
@@ -73,12 +76,13 @@ void         workbook_add_summary_info    (Workbook *wb, SummaryItem *sit);
 SummaryInfo *workbook_metadata    	  (Workbook const *wb);
 
 /* See also sheet_cell_foreach_range */
-GnmValue   *workbook_foreach_cell_in_range (EvalPos const  *pos,
+GnmValue   *workbook_foreach_cell_in_range (GnmEvalPos const  *pos,
 					    GnmValue const *cell_range,
 					    CellIterFlags   flags,
 					    CellIterFunc    handler,
 					    gpointer	    closure);
 GPtrArray  *workbook_cells               (Workbook *wb, gboolean comments);
+GSList     *workbook_local_functions	 (Workbook const *wb);
 
 /* Calculation */
 void     workbook_recalc                 (Workbook *wb);	/* in eval.c */
